/**
 * Promisified script loader (we can do this because we're using XWalk)
 */
function loadScript(src) {
  return new Promise(function(resolve, reject) {
    console.info('Loading ' + src);
    var script = document.createElement('script');
    script.addEventListener('load', resolve);
    script.addEventListener('error', reject);
    script.src = src;
    document.head.appendChild(script);
  });
};

var scripts = [
  'dependencies.js',
  'minigames/jump/assets/scripts/displacement.js',
  'minigames/jump/js/plugins/fpsmeter.js',
  'minigames/jump/js/phaser3-nopixi.js',
  'minigames/jump/js/jump.js',
  'pbs.js' // this is actually configured somewhere and probably shouldn't be hardcoded...
];

document.addEventListener('deviceready', function() {
  // load each script SEQUENTIALLY, i.e. load one script, wait for it to finish loading and then move on to the next.
  // The reason we want to do this is because each script depends on the scripts before it (e.g. jump.js depends on
  // PIXI and Phaser to be loaded)
  var allScripts = Promise.resolve();
  scripts.forEach(function(script) {
    var fullPath = window.EXPANSION_PREFIX + script;
    allScripts = allScripts.then(() => loadScript(fullPath));
  });

  allScripts 
    .then(function() {
      pbsGlobals.initialHeight = window.innerHeight;
      realignGameContainer();
      overridePhaserPointerHandler();

      // Now that dependencies are loaded, we can use jQuery
      $(document).on('DOMNodeInserted', function(evt) {
        if(evt.target.id !== 'authorization') {
          return;
        }

        var $frame = $('iframe', evt.target);

        $frame.on('load', function() {
          attachFocusListenersToFrame($frame[0]);
        });
      })
    })
});

var pbsGlobals = {};
pbsGlobals.globalAppScale = 1;
pbsGlobals.initialHeight = 0;
pbsGlobals.isMuted = false;

pbsGlobals.downloadExternalSound = function(url, destination, completeCallback, faultCallback) {
  var fileTransfer = new FileTransfer();
  var uri = encodeURI(url);
  fileTransfer.download(
    uri,
    "cdvfile://localhost/persistent/" + destination,
    completeCallback,
    faultCallback,
    false
  );
}

function realignGameContainer() {
  // The preferred size of the game. Since KK uses pixels for sizing (I think), we can't simply use some sort of
  // viewport units (boo).
  var gameWidth = 1024;
  var gameHeight = 768;
  var aspectRatio = gameWidth / gameHeight;

  // position the container
  var $container = $('#pbsApp');
  var $canvas = $('#scene');

  var containerWidth = window.innerHeight * aspectRatio;
  var containerScaleFactor = window.innerHeight / gameHeight;
  var fitWidth = window.innerWidth / containerScaleFactor;

  $container.css({
    top: 0,
    width: fitWidth + 'px',
    height: gameHeight + 'px',
    transform: 'translate(0px, 0px) scale(' + containerScaleFactor + ')',
    transformOrigin: 'left top',
    display: 'block'
  });

  // save the values later for scale calculation
  var scaleFactor = containerWidth / gameWidth;
  pbsGlobals.globalWidthScale = scaleFactor;
  pbsGlobals.globalAppScale = scaleFactor;
}

var attachFocusListenersToInput = function(input) {
    var parentWindow = window;

    input.addEventListener('focus', function() {
        var shiftHeight = -parentWindow.innerHeight / 3;
        parentWindow.document.body.style.transform = 'translateY(' + shiftHeight + 'px)';
    });

    input.addEventListener('blur', function() {
        parentWindow.document.body.style.transform = '';
        AndroidFullScreen.immersiveMode();
    });
};

var attachFocusListenersToFrame = function(frame) {
    var inputs = frame.contentWindow.document.querySelectorAll('input[type=text]');
    for(var i = 0; i < inputs.length; i++) {
        attachFocusListenersToInput(inputs[i]);
    }
};

var overridePhaserPointerHandler = function() {
  var processInteractiveObjects = Phaser.Pointer.prototype.processInteractiveObjects;
  Phaser.Pointer.prototype.processInteractiveObjects = function (fromClick) {
    this.clientX = this.clientX / pbsGlobals.globalAppScale;
    this.clientY = this.clientY / pbsGlobals.globalAppScale;
    this.pageX = this.pageX / pbsGlobals.globalAppScale;
    this.pageY = this.pageY / pbsGlobals.globalAppScale;
    this.screenX = this.screenX / pbsGlobals.globalAppScale;
    this.screenY = this.screenY / pbsGlobals.globalAppScale;
    this.x = this.x / pbsGlobals.globalAppScale;
    this.y = this.y / pbsGlobals.globalAppScale;
    var boundProcessInteractiveObjects = processInteractiveObjects.bind(this);
    boundProcessInteractiveObjects(fromClick);
  };
};
