(function() {
'use strict';

var Faye = {
  VERSION:          '1.1.2',

  BAYEUX_VERSION:   '1.0',
  ID_LENGTH:        160,
  JSONP_CALLBACK:   'jsonpcallback',
  CONNECTION_TYPES: ['long-polling', 'cross-origin-long-polling', 'callback-polling', 'websocket', 'eventsource', 'in-process'],

  MANDATORY_CONNECTION_TYPES: ['long-polling', 'callback-polling', 'in-process'],

  ENV: (typeof window !== 'undefined') ? window : global,

  extend: function(dest, source, overwrite) {
    if (!source) return dest;
    for (var key in source) {
      if (!source.hasOwnProperty(key)) continue;
      if (dest.hasOwnProperty(key) && overwrite === false) continue;
      if (dest[key] !== source[key])
        dest[key] = source[key];
    }
    return dest;
  },

  random: function(bitlength) {
    bitlength = bitlength || this.ID_LENGTH;
    var maxLength = Math.ceil(bitlength * Math.log(2) / Math.log(36));
    var string = csprng(bitlength, 36);
    while (string.length < maxLength) string = '0' + string;
    return string;
  },

  validateOptions: function(options, validKeys) {
    for (var key in options) {
      if (this.indexOf(validKeys, key) < 0)
        throw new Error('Unrecognized option: ' + key);
    }
  },

  clientIdFromMessages: function(messages) {
    var connect = this.filter([].concat(messages), function(message) {
      return message.channel === '/meta/connect';
    });
    return connect[0] && connect[0].clientId;
  },

  copyObject: function(object) {
    var clone, i, key;
    if (object instanceof Array) {
      clone = [];
      i = object.length;
      while (i--) clone[i] = Faye.copyObject(object[i]);
      return clone;
    } else if (typeof object === 'object') {
      clone = (object === null) ? null : {};
      for (key in object) clone[key] = Faye.copyObject(object[key]);
      return clone;
    } else {
      return object;
    }
  },

  commonElement: function(lista, listb) {
    for (var i = 0, n = lista.length; i < n; i++) {
      if (this.indexOf(listb, lista[i]) !== -1)
        return lista[i];
    }
    return null;
  },

  indexOf: function(list, needle) {
    if (list.indexOf) return list.indexOf(needle);

    for (var i = 0, n = list.length; i < n; i++) {
      if (list[i] === needle) return i;
    }
    return -1;
  },

  map: function(object, callback, context) {
    if (object.map) return object.map(callback, context);
    var result = [];

    if (object instanceof Array) {
      for (var i = 0, n = object.length; i < n; i++) {
        result.push(callback.call(context || null, object[i], i));
      }
    } else {
      for (var key in object) {
        if (!object.hasOwnProperty(key)) continue;
        result.push(callback.call(context || null, key, object[key]));
      }
    }
    return result;
  },

  filter: function(array, callback, context) {
    if (array.filter) return array.filter(callback, context);
    var result = [];
    for (var i = 0, n = array.length; i < n; i++) {
      if (callback.call(context || null, array[i], i))
        result.push(array[i]);
    }
    return result;
  },

  asyncEach: function(list, iterator, callback, context) {
    var n       = list.length,
        i       = -1,
        calls   = 0,
        looping = false;

    var iterate = function() {
      calls -= 1;
      i += 1;
      if (i === n) return callback && callback.call(context);
      iterator(list[i], resume);
    };

    var loop = function() {
      if (looping) return;
      looping = true;
      while (calls > 0) iterate();
      looping = false;
    };

    var resume = function() {
      calls += 1;
      loop();
    };
    resume();
  },

  // http://assanka.net/content/tech/2009/09/02/json2-js-vs-prototype/
  toJSON: function(object) {
    if (!this.stringify) return JSON.stringify(object);

    return this.stringify(object, function(key, value) {
      return (this[key] instanceof Array) ? this[key] : value;
    });
  }
};

if (typeof module !== 'undefined')
  module.exports = Faye;
else if (typeof window !== 'undefined')
  window.Faye = Faye;

Faye.Class = function(parent, methods) {
  if (typeof parent !== 'function') {
    methods = parent;
    parent  = Object;
  }

  var klass = function() {
    if (!this.initialize) return this;
    return this.initialize.apply(this, arguments) || this;
  };

  var bridge = function() {};
  bridge.prototype = parent.prototype;

  klass.prototype = new bridge();
  Faye.extend(klass.prototype, methods);

  return klass;
};

(function() {
var EventEmitter = Faye.EventEmitter = function() {};

/*
Copyright Joyent, Inc. and other Node contributors. All rights reserved.
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

var isArray = typeof Array.isArray === 'function'
    ? Array.isArray
    : function (xs) {
        return Object.prototype.toString.call(xs) === '[object Array]'
    }
;
function indexOf (xs, x) {
    if (xs.indexOf) return xs.indexOf(x);
    for (var i = 0; i < xs.length; i++) {
        if (x === xs[i]) return i;
    }
    return -1;
}


EventEmitter.prototype.emit = function(type) {
  // If there is no 'error' event listener then throw.
  if (type === 'error') {
    if (!this._events || !this._events.error ||
        (isArray(this._events.error) && !this._events.error.length))
    {
      if (arguments[1] instanceof Error) {
        throw arguments[1]; // Unhandled 'error' event
      } else {
        throw new Error("Uncaught, unspecified 'error' event.");
      }
      return false;
    }
  }

  if (!this._events) return false;
  var handler = this._events[type];
  if (!handler) return false;

  if (typeof handler == 'function') {
    switch (arguments.length) {
      // fast cases
      case 1:
        handler.call(this);
        break;
      case 2:
        handler.call(this, arguments[1]);
        break;
      case 3:
        handler.call(this, arguments[1], arguments[2]);
        break;
      // slower
      default:
        var args = Array.prototype.slice.call(arguments, 1);
        handler.apply(this, args);
    }
    return true;

  } else if (isArray(handler)) {
    var args = Array.prototype.slice.call(arguments, 1);

    var listeners = handler.slice();
    for (var i = 0, l = listeners.length; i < l; i++) {
      listeners[i].apply(this, args);
    }
    return true;

  } else {
    return false;
  }
};

// EventEmitter is defined in src/node_events.cc
// EventEmitter.prototype.emit() is also defined there.
EventEmitter.prototype.addListener = function(type, listener) {
  if ('function' !== typeof listener) {
    throw new Error('addListener only takes instances of Function');
  }

  if (!this._events) this._events = {};

  // To avoid recursion in the case that type == "newListeners"! Before
  // adding it to the listeners, first emit "newListeners".
  this.emit('newListener', type, listener);

  if (!this._events[type]) {
    // Optimize the case of one listener. Don't need the extra array object.
    this._events[type] = listener;
  } else if (isArray(this._events[type])) {
    // If we've already got an array, just append.
    this._events[type].push(listener);
  } else {
    // Adding the second element, need to change to array.
    this._events[type] = [this._events[type], listener];
  }

  return this;
};

EventEmitter.prototype.on = EventEmitter.prototype.addListener;

EventEmitter.prototype.once = function(type, listener) {
  var self = this;
  self.on(type, function g() {
    self.removeListener(type, g);
    listener.apply(this, arguments);
  });

  return this;
};

EventEmitter.prototype.removeListener = function(type, listener) {
  if ('function' !== typeof listener) {
    throw new Error('removeListener only takes instances of Function');
  }

  // does not use listeners(), so no side effect of creating _events[type]
  if (!this._events || !this._events[type]) return this;

  var list = this._events[type];

  if (isArray(list)) {
    var i = indexOf(list, listener);
    if (i < 0) return this;
    list.splice(i, 1);
    if (list.length == 0)
      delete this._events[type];
  } else if (this._events[type] === listener) {
    delete this._events[type];
  }

  return this;
};

EventEmitter.prototype.removeAllListeners = function(type) {
  if (arguments.length === 0) {
    this._events = {};
    return this;
  }

  // does not use listeners(), so no side effect of creating _events[type]
  if (type && this._events && this._events[type]) this._events[type] = null;
  return this;
};

EventEmitter.prototype.listeners = function(type) {
  if (!this._events) this._events = {};
  if (!this._events[type]) this._events[type] = [];
  if (!isArray(this._events[type])) {
    this._events[type] = [this._events[type]];
  }
  return this._events[type];
};

})();

Faye.Namespace = Faye.Class({
  initialize: function() {
    this._used = {};
  },

  exists: function(id) {
    return this._used.hasOwnProperty(id);
  },

  generate: function() {
    var name = Faye.random();
    while (this._used.hasOwnProperty(name))
      name = Faye.random();
    return this._used[name] = name;
  },

  release: function(id) {
    delete this._used[id];
  }
});

(function() {
'use strict';

var timeout = setTimeout, defer;

if (typeof setImmediate === 'function')
  defer = function(fn) { setImmediate(fn) };
else if (typeof process === 'object' && process.nextTick)
  defer = function(fn) { process.nextTick(fn) };
else
  defer = function(fn) { timeout(fn, 0) };

var PENDING   = 0,
    FULFILLED = 1,
    REJECTED  = 2;

var RETURN = function(x) { return x },
    THROW  = function(x) { throw  x };

var Promise = function(task) {
  this._state       = PENDING;
  this._onFulfilled = [];
  this._onRejected  = [];

  if (typeof task !== 'function') return;
  var self = this;

  task(function(value)  { fulfill(self, value) },
       function(reason) { reject(self, reason) });
};

Promise.prototype.then = function(onFulfilled, onRejected) {
  var next = new Promise();
  registerOnFulfilled(this, onFulfilled, next);
  registerOnRejected(this, onRejected, next);
  return next;
};

var registerOnFulfilled = function(promise, onFulfilled, next) {
  if (typeof onFulfilled !== 'function') onFulfilled = RETURN;
  var handler = function(value) { invoke(onFulfilled, value, next) };

  if (promise._state === PENDING) {
    promise._onFulfilled.push(handler);
  } else if (promise._state === FULFILLED) {
    handler(promise._value);
  }
};

var registerOnRejected = function(promise, onRejected, next) {
  if (typeof onRejected !== 'function') onRejected = THROW;
  var handler = function(reason) { invoke(onRejected, reason, next) };

  if (promise._state === PENDING) {
    promise._onRejected.push(handler);
  } else if (promise._state === REJECTED) {
    handler(promise._reason);
  }
};

var invoke = function(fn, value, next) {
  defer(function() { _invoke(fn, value, next) });
};

var _invoke = function(fn, value, next) {
  var outcome;

  try {
    outcome = fn(value);
  } catch (error) {
    return reject(next, error);
  }

  if (outcome === next) {
    reject(next, new TypeError('Recursive promise chain detected'));
  } else {
    fulfill(next, outcome);
  }
};

var fulfill = Promise.fulfill = Promise.resolve = function(promise, value) {
  var called = false, type, then;

  try {
    type = typeof value;
    then = value !== null && (type === 'function' || type === 'object') && value.then;

    if (typeof then !== 'function') return _fulfill(promise, value);

    then.call(value, function(v) {
      if (!(called ^ (called = true))) return;
      fulfill(promise, v);
    }, function(r) {
      if (!(called ^ (called = true))) return;
      reject(promise, r);
    });
  } catch (error) {
    if (!(called ^ (called = true))) return;
    reject(promise, error);
  }
};

var _fulfill = function(promise, value) {
  if (promise._state !== PENDING) return;

  promise._state      = FULFILLED;
  promise._value      = value;
  promise._onRejected = [];

  var onFulfilled = promise._onFulfilled, fn;
  while (fn = onFulfilled.shift()) fn(value);
};

var reject = Promise.reject = function(promise, reason) {
  if (promise._state !== PENDING) return;

  promise._state       = REJECTED;
  promise._reason      = reason;
  promise._onFulfilled = [];

  var onRejected = promise._onRejected, fn;
  while (fn = onRejected.shift()) fn(reason);
};

Promise.all = function(promises) {
  return new Promise(function(fulfill, reject) {
    var list = [],
         n   = promises.length,
         i;

    if (n === 0) return fulfill(list);

    for (i = 0; i < n; i++) (function(promise, i) {
      Promise.fulfilled(promise).then(function(value) {
        list[i] = value;
        if (--n === 0) fulfill(list);
      }, reject);
    })(promises[i], i);
  });
};

Promise.defer = defer;

Promise.deferred = Promise.pending = function() {
  var tuple = {};

  tuple.promise = new Promise(function(fulfill, reject) {
    tuple.fulfill = tuple.resolve = fulfill;
    tuple.reject  = reject;
  });
  return tuple;
};

Promise.fulfilled = Promise.resolved = function(value) {
  return new Promise(function(fulfill, reject) { fulfill(value) });
};

Promise.rejected = function(reason) {
  return new Promise(function(fulfill, reject) { reject(reason) });
};

if (typeof Faye === 'undefined')
  module.exports = Promise;
else
  Faye.Promise = Promise;

})();

Faye.Set = Faye.Class({
  initialize: function() {
    this._index = {};
  },

  add: function(item) {
    var key = (item.id !== undefined) ? item.id : item;
    if (this._index.hasOwnProperty(key)) return false;
    this._index[key] = item;
    return true;
  },

  forEach: function(block, context) {
    for (var key in this._index) {
      if (this._index.hasOwnProperty(key))
        block.call(context, this._index[key]);
    }
  },

  isEmpty: function() {
    for (var key in this._index) {
      if (this._index.hasOwnProperty(key)) return false;
    }
    return true;
  },

  member: function(item) {
    for (var key in this._index) {
      if (this._index[key] === item) return true;
    }
    return false;
  },

  remove: function(item) {
    var key = (item.id !== undefined) ? item.id : item;
    var removed = this._index[key];
    delete this._index[key];
    return removed;
  },

  toArray: function() {
    var array = [];
    this.forEach(function(item) { array.push(item) });
    return array;
  }
});

Faye.URI = {
  isURI: function(uri) {
    return uri && uri.protocol && uri.host && uri.path;
  },

  isSameOrigin: function(uri) {
    var location = Faye.ENV.location;
    return uri.protocol === location.protocol &&
           uri.hostname === location.hostname &&
           uri.port     === location.port;
  },

  parse: function(url) {
    if (typeof url !== 'string') return url;
    var uri = {}, parts, query, pairs, i, n, data;

    var consume = function(name, pattern) {
      url = url.replace(pattern, function(match) {
        uri[name] = match;
        return '';
      });
      uri[name] = uri[name] || '';
    };

    consume('protocol', /^[a-z]+\:/i);
    consume('host',     /^\/\/[^\/\?#]+/);

    if (!/^\//.test(url) && !uri.host)
      url = Faye.ENV.location.pathname.replace(/[^\/]*$/, '') + url;

    consume('pathname', /^[^\?#]*/);
    consume('search',   /^\?[^#]*/);
    consume('hash',     /^#.*/);

    uri.protocol = uri.protocol || Faye.ENV.location.protocol;

    if (uri.host) {
      uri.host     = uri.host.substr(2);
      parts        = uri.host.split(':');
      uri.hostname = parts[0];
      uri.port     = parts[1] || '';
    } else {
      uri.host     = Faye.ENV.location.host;
      uri.hostname = Faye.ENV.location.hostname;
      uri.port     = Faye.ENV.location.port;
    }

    uri.pathname = uri.pathname || '/';
    uri.path = uri.pathname + uri.search;

    query = uri.search.replace(/^\?/, '');
    pairs = query ? query.split('&') : [];
    data  = {};

    for (i = 0, n = pairs.length; i < n; i++) {
      parts = pairs[i].split('=');
      data[decodeURIComponent(parts[0] || '')] = decodeURIComponent(parts[1] || '');
    }

    uri.query = data;

    uri.href = this.stringify(uri);
    return uri;
  },

  stringify: function(uri) {
    var string = uri.protocol + '//' + uri.hostname;
    if (uri.port) string += ':' + uri.port;
    string += uri.pathname + this.queryString(uri.query) + (uri.hash || '');
    return string;
  },

  queryString: function(query) {
    var pairs = [];
    for (var key in query) {
      if (!query.hasOwnProperty(key)) continue;
      pairs.push(encodeURIComponent(key) + '=' + encodeURIComponent(query[key]));
    }
    if (pairs.length === 0) return '';
    return '?' + pairs.join('&');
  }
};

Faye.Error = Faye.Class({
  initialize: function(code, params, message) {
    this.code    = code;
    this.params  = Array.prototype.slice.call(params);
    this.message = message;
  },

  toString: function() {
    return this.code + ':' +
           this.params.join(',') + ':' +
           this.message;
  }
});

Faye.Error.parse = function(message) {
  message = message || '';
  if (!Faye.Grammar.ERROR.test(message)) return new this(null, [], message);

  var parts   = message.split(':'),
      code    = parseInt(parts[0]),
      params  = parts[1].split(','),
      message = parts[2];

  return new this(code, params, message);
};




Faye.Error.versionMismatch = function() {
  return new this(300, arguments, 'Version mismatch').toString();
};

Faye.Error.conntypeMismatch = function() {
  return new this(301, arguments, 'Connection types not supported').toString();
};

Faye.Error.extMismatch = function() {
  return new this(302, arguments, 'Extension mismatch').toString();
};

Faye.Error.badRequest = function() {
  return new this(400, arguments, 'Bad request').toString();
};

Faye.Error.clientUnknown = function() {
  return new this(401, arguments, 'Unknown client').toString();
};

Faye.Error.parameterMissing = function() {
  return new this(402, arguments, 'Missing required parameter').toString();
};

Faye.Error.channelForbidden = function() {
  return new this(403, arguments, 'Forbidden channel').toString();
};

Faye.Error.channelUnknown = function() {
  return new this(404, arguments, 'Unknown channel').toString();
};

Faye.Error.channelInvalid = function() {
  return new this(405, arguments, 'Invalid channel').toString();
};

Faye.Error.extUnknown = function() {
  return new this(406, arguments, 'Unknown extension').toString();
};

Faye.Error.publishFailed = function() {
  return new this(407, arguments, 'Failed to publish').toString();
};

Faye.Error.serverError = function() {
  return new this(500, arguments, 'Internal server error').toString();
};


Faye.Deferrable = {
  then: function(callback, errback) {
    var self = this;
    if (!this._promise)
      this._promise = new Faye.Promise(function(fulfill, reject) {
        self._fulfill = fulfill;
        self._reject  = reject;
      });

    if (arguments.length === 0)
      return this._promise;
    else
      return this._promise.then(callback, errback);
  },

  callback: function(callback, context) {
    return this.then(function(value) { callback.call(context, value) });
  },

  errback: function(callback, context) {
    return this.then(null, function(reason) { callback.call(context, reason) });
  },

  timeout: function(seconds, message) {
    this.then();
    var self = this;
    this._timer = Faye.ENV.setTimeout(function() {
      self._reject(message);
    }, seconds * 1000);
  },

  setDeferredStatus: function(status, value) {
    if (this._timer) Faye.ENV.clearTimeout(this._timer);

    this.then();

    if (status === 'succeeded')
      this._fulfill(value);
    else if (status === 'failed')
      this._reject(value);
    else
      delete this._promise;
  }
};

Faye.Publisher = {
  countListeners: function(eventType) {
    return this.listeners(eventType).length;
  },

  bind: function(eventType, listener, context) {
    var slice   = Array.prototype.slice,
        handler = function() { listener.apply(context, slice.call(arguments)) };

    this._listeners = this._listeners || [];
    this._listeners.push([eventType, listener, context, handler]);
    return this.on(eventType, handler);
  },

  unbind: function(eventType, listener, context) {
    this._listeners = this._listeners || [];
    var n = this._listeners.length, tuple;

    while (n--) {
      tuple = this._listeners[n];
      if (tuple[0] !== eventType) continue;
      if (listener && (tuple[1] !== listener || tuple[2] !== context)) continue;
      this._listeners.splice(n, 1);
      this.removeListener(eventType, tuple[3]);
    }
  }
};

Faye.extend(Faye.Publisher, Faye.EventEmitter.prototype);
Faye.Publisher.trigger = Faye.Publisher.emit;

Faye.Timeouts = {
  addTimeout: function(name, delay, callback, context) {
    this._timeouts = this._timeouts || {};
    if (this._timeouts.hasOwnProperty(name)) return;
    var self = this;
    this._timeouts[name] = Faye.ENV.setTimeout(function() {
      delete self._timeouts[name];
      callback.call(context);
    }, 1000 * delay);
  },

  removeTimeout: function(name) {
    this._timeouts = this._timeouts || {};
    var timeout = this._timeouts[name];
    if (!timeout) return;
    Faye.ENV.clearTimeout(timeout);
    delete this._timeouts[name];
  },

  removeAllTimeouts: function() {
    this._timeouts = this._timeouts || {};
    for (var name in this._timeouts) this.removeTimeout(name);
  }
};

Faye.Logging = {
  LOG_LEVELS: {
    fatal:  4,
    error:  3,
    warn:   2,
    info:   1,
    debug:  0
  },

  writeLog: function(messageArgs, level) {
    if (!Faye.logger) return;

    var args   = Array.prototype.slice.apply(messageArgs),
        banner = '[Faye',
        klass  = this.className,

        message = args.shift().replace(/\?/g, function() {
          try {
            return Faye.toJSON(args.shift());
          } catch (e) {
            return '[Object]';
          }
        });

    for (var key in Faye) {
      if (klass) continue;
      if (typeof Faye[key] !== 'function') continue;
      if (this instanceof Faye[key]) klass = key;
    }
    if (klass) banner += '.' + klass;
    banner += '] ';

    if (typeof Faye.logger[level] === 'function')
      Faye.logger[level](banner + message);
    else if (typeof Faye.logger === 'function')
      Faye.logger(banner + message);
  }
};

(function() {
  for (var key in Faye.Logging.LOG_LEVELS)
    (function(level) {
      Faye.Logging[level] = function() {
        this.writeLog(arguments, level);
      };
    })(key);
})();

Faye.Grammar = {
  CHANNEL_NAME:     /^\/(((([a-z]|[A-Z])|[0-9])|(\-|\_|\!|\~|\(|\)|\$|\@)))+(\/(((([a-z]|[A-Z])|[0-9])|(\-|\_|\!|\~|\(|\)|\$|\@)))+)*$/,
  CHANNEL_PATTERN:  /^(\/(((([a-z]|[A-Z])|[0-9])|(\-|\_|\!|\~|\(|\)|\$|\@)))+)*\/\*{1,2}$/,
  ERROR:            /^([0-9][0-9][0-9]:(((([a-z]|[A-Z])|[0-9])|(\-|\_|\!|\~|\(|\)|\$|\@)| |\/|\*|\.))*(,(((([a-z]|[A-Z])|[0-9])|(\-|\_|\!|\~|\(|\)|\$|\@)| |\/|\*|\.))*)*:(((([a-z]|[A-Z])|[0-9])|(\-|\_|\!|\~|\(|\)|\$|\@)| |\/|\*|\.))*|[0-9][0-9][0-9]::(((([a-z]|[A-Z])|[0-9])|(\-|\_|\!|\~|\(|\)|\$|\@)| |\/|\*|\.))*)$/,
  VERSION:          /^([0-9])+(\.(([a-z]|[A-Z])|[0-9])(((([a-z]|[A-Z])|[0-9])|\-|\_))*)*$/
};

Faye.Extensible = {
  addExtension: function(extension) {
    this._extensions = this._extensions || [];
    this._extensions.push(extension);
    if (extension.added) extension.added(this);
  },

  removeExtension: function(extension) {
    if (!this._extensions) return;
    var i = this._extensions.length;
    while (i--) {
      if (this._extensions[i] !== extension) continue;
      this._extensions.splice(i,1);
      if (extension.removed) extension.removed(this);
    }
  },

  pipeThroughExtensions: function(stage, message, request, callback, context) {
    this.debug('Passing through ? extensions: ?', stage, message);

    if (!this._extensions) return callback.call(context, message);
    var extensions = this._extensions.slice();

    var pipe = function(message) {
      if (!message) return callback.call(context, message);

      var extension = extensions.shift();
      if (!extension) return callback.call(context, message);

      var fn = extension[stage];
      if (!fn) return pipe(message);

      if (fn.length >= 3) extension[stage](message, request, pipe);
      else                extension[stage](message, pipe);
    };
    pipe(message);
  }
};

Faye.extend(Faye.Extensible, Faye.Logging);

Faye.Channel = Faye.Class({
  initialize: function(name) {
    this.id = this.name = name;
  },

  push: function(message) {
    this.trigger('message', message);
  },

  isUnused: function() {
    return this.countListeners('message') === 0;
  }
});

Faye.extend(Faye.Channel.prototype, Faye.Publisher);

Faye.extend(Faye.Channel, {
  HANDSHAKE:    '/meta/handshake',
  CONNECT:      '/meta/connect',
  SUBSCRIBE:    '/meta/subscribe',
  UNSUBSCRIBE:  '/meta/unsubscribe',
  DISCONNECT:   '/meta/disconnect',

  META:         'meta',
  SERVICE:      'service',

  expand: function(name) {
    var segments = this.parse(name),
        channels = ['/**', name];

    var copy = segments.slice();
    copy[copy.length - 1] = '*';
    channels.push(this.unparse(copy));

    for (var i = 1, n = segments.length; i < n; i++) {
      copy = segments.slice(0, i);
      copy.push('**');
      channels.push(this.unparse(copy));
    }

    return channels;
  },

  isValid: function(name) {
    return Faye.Grammar.CHANNEL_NAME.test(name) ||
           Faye.Grammar.CHANNEL_PATTERN.test(name);
  },

  parse: function(name) {
    if (!this.isValid(name)) return null;
    return name.split('/').slice(1);
  },

  unparse: function(segments) {
    return '/' + segments.join('/');
  },

  isMeta: function(name) {
    var segments = this.parse(name);
    return segments ? (segments[0] === this.META) : null;
  },

  isService: function(name) {
    var segments = this.parse(name);
    return segments ? (segments[0] === this.SERVICE) : null;
  },

  isSubscribable: function(name) {
    if (!this.isValid(name)) return null;
    return !this.isMeta(name) && !this.isService(name);
  },

  Set: Faye.Class({
    initialize: function() {
      this._channels = {};
    },

    getKeys: function() {
      var keys = [];
      for (var key in this._channels) keys.push(key);
      return keys;
    },

    remove: function(name) {
      delete this._channels[name];
    },

    hasSubscription: function(name) {
      return this._channels.hasOwnProperty(name);
    },

    subscribe: function(names, callback, context) {
      var name;
      for (var i = 0, n = names.length; i < n; i++) {
        name = names[i];
        var channel = this._channels[name] = this._channels[name] || new Faye.Channel(name);
        if (callback) channel.bind('message', callback, context);
      }
    },

    unsubscribe: function(name, callback, context) {
      var channel = this._channels[name];
      if (!channel) return false;
      channel.unbind('message', callback, context);

      if (channel.isUnused()) {
        this.remove(name);
        return true;
      } else {
        return false;
      }
    },

    distributeMessage: function(message) {
      var channels = Faye.Channel.expand(message.channel);

      for (var i = 0, n = channels.length; i < n; i++) {
        var channel = this._channels[channels[i]];
        if (channel) channel.trigger('message', message.data);
      }
    }
  })
});

Faye.Publication = Faye.Class(Faye.Deferrable);

Faye.Subscription = Faye.Class({
  initialize: function(client, channels, callback, context) {
    this._client    = client;
    this._channels  = channels;
    this._callback  = callback;
    this._context     = context;
    this._cancelled = false;
  },

  cancel: function() {
    if (this._cancelled) return;
    this._client.unsubscribe(this._channels, this._callback, this._context);
    this._cancelled = true;
  },

  unsubscribe: function() {
    this.cancel();
  }
});

Faye.extend(Faye.Subscription.prototype, Faye.Deferrable);

Faye.Client = Faye.Class({
  UNCONNECTED:        1,
  CONNECTING:         2,
  CONNECTED:          3,
  DISCONNECTED:       4,

  HANDSHAKE:          'handshake',
  RETRY:              'retry',
  NONE:               'none',

  CONNECTION_TIMEOUT: 60,

  DEFAULT_ENDPOINT:   '/bayeux',
  INTERVAL:           0,

  initialize: function(endpoint, options) {
    this.info('New client created for ?', endpoint);
    options = options || {};

    Faye.validateOptions(options, ['interval', 'timeout', 'endpoints', 'proxy', 'retry', 'scheduler', 'websocketExtensions', 'tls', 'ca']);

    this._endpoint   = endpoint || this.DEFAULT_ENDPOINT;
    this._channels   = new Faye.Channel.Set();
    this._dispatcher = new Faye.Dispatcher(this, this._endpoint, options);

    this._messageId = 0;
    this._state     = this.UNCONNECTED;

    this._responseCallbacks = {};

    this._advice = {
      reconnect: this.RETRY,
      interval:  1000 * (options.interval || this.INTERVAL),
      timeout:   1000 * (options.timeout  || this.CONNECTION_TIMEOUT)
    };
    this._dispatcher.timeout = this._advice.timeout / 1000;

    this._dispatcher.bind('message', this._receiveMessage, this);

    if (Faye.Event && Faye.ENV.onbeforeunload !== undefined)
      Faye.Event.on(Faye.ENV, 'beforeunload', function() {
        if (Faye.indexOf(this._dispatcher._disabled, 'autodisconnect') < 0)
          this.disconnect();
      }, this);
  },

  addWebsocketExtension: function(extension) {
    return this._dispatcher.addWebsocketExtension(extension);
  },

  disable: function(feature) {
    return this._dispatcher.disable(feature);
  },

  setHeader: function(name, value) {
    return this._dispatcher.setHeader(name, value);
  },

  // Request
  // MUST include:  * channel
  //                * version
  //                * supportedConnectionTypes
  // MAY include:   * minimumVersion
  //                * ext
  //                * id
  //
  // Success Response                             Failed Response
  // MUST include:  * channel                     MUST include:  * channel
  //                * version                                    * successful
  //                * supportedConnectionTypes                   * error
  //                * clientId                    MAY include:   * supportedConnectionTypes
  //                * successful                                 * advice
  // MAY include:   * minimumVersion                             * version
  //                * advice                                     * minimumVersion
  //                * ext                                        * ext
  //                * id                                         * id
  //                * authSuccessful
  handshake: function(callback, context) {
    if (this._advice.reconnect === this.NONE) return;
    if (this._state !== this.UNCONNECTED) return;

    this._state = this.CONNECTING;
    var self = this;

    this.info('Initiating handshake with ?', Faye.URI.stringify(this._endpoint));
    this._dispatcher.selectTransport(Faye.MANDATORY_CONNECTION_TYPES);

    this._sendMessage({
      channel:                  Faye.Channel.HANDSHAKE,
      version:                  Faye.BAYEUX_VERSION,
      supportedConnectionTypes: this._dispatcher.getConnectionTypes()

    }, {}, function(response) {

      if (response.successful) {
        this._state = this.CONNECTED;
        this._dispatcher.clientId  = response.clientId;

        this._dispatcher.selectTransport(response.supportedConnectionTypes);

        this.info('Handshake successful: ?', this._dispatcher.clientId);

        this.subscribe(this._channels.getKeys(), true);
        if (callback) Faye.Promise.defer(function() { callback.call(context) });

      } else {
        this.info('Handshake unsuccessful');
        Faye.ENV.setTimeout(function() { self.handshake(callback, context) }, this._dispatcher.retry * 1000);
        this._state = this.UNCONNECTED;
      }
    }, this);
  },

  // Request                              Response
  // MUST include:  * channel             MUST include:  * channel
  //                * clientId                           * successful
  //                * connectionType                     * clientId
  // MAY include:   * ext                 MAY include:   * error
  //                * id                                 * advice
  //                                                     * ext
  //                                                     * id
  //                                                     * timestamp
  connect: function(callback, context) {
    if (this._advice.reconnect === this.NONE) return;
    if (this._state === this.DISCONNECTED) return;

    if (this._state === this.UNCONNECTED)
      return this.handshake(function() { this.connect(callback, context) }, this);

    this.callback(callback, context);
    if (this._state !== this.CONNECTED) return;

    this.info('Calling deferred actions for ?', this._dispatcher.clientId);
    this.setDeferredStatus('succeeded');
    this.setDeferredStatus('unknown');

    if (this._connectRequest) return;
    this._connectRequest = true;

    this.info('Initiating connection for ?', this._dispatcher.clientId);

    this._sendMessage({
      channel:        Faye.Channel.CONNECT,
      clientId:       this._dispatcher.clientId,
      connectionType: this._dispatcher.connectionType

    }, {}, this._cycleConnection, this);
  },

  // Request                              Response
  // MUST include:  * channel             MUST include:  * channel
  //                * clientId                           * successful
  // MAY include:   * ext                                * clientId
  //                * id                  MAY include:   * error
  //                                                     * ext
  //                                                     * id
  disconnect: function() {
    if (this._state !== this.CONNECTED) return;
    this._state = this.DISCONNECTED;

    this.info('Disconnecting ?', this._dispatcher.clientId);
    var promise = new Faye.Publication();

    this._sendMessage({
      channel:  Faye.Channel.DISCONNECT,
      clientId: this._dispatcher.clientId

    }, {}, function(response) {
      if (response.successful) {
        this._dispatcher.close();
        promise.setDeferredStatus('succeeded');
      } else {
        promise.setDeferredStatus('failed', Faye.Error.parse(response.error));
      }
    }, this);

    this.info('Clearing channel listeners for ?', this._dispatcher.clientId);
    this._channels = new Faye.Channel.Set();

    return promise;
  },

  // Request                              Response
  // MUST include:  * channel             MUST include:  * channel
  //                * clientId                           * successful
  //                * subscription                       * clientId
  // MAY include:   * ext                                * subscription
  //                * id                  MAY include:   * error
  //                                                     * advice
  //                                                     * ext
  //                                                     * id
  //                                                     * timestamp
  subscribe: function(channel, callback, context) {
    if (channel instanceof Array)
      return Faye.map(channel, function(c) {
        return this.subscribe(c, callback, context);
      }, this);

    var subscription = new Faye.Subscription(this, channel, callback, context),
        force        = (callback === true),
        hasSubscribe = this._channels.hasSubscription(channel);

    if (hasSubscribe && !force) {
      this._channels.subscribe([channel], callback, context);
      subscription.setDeferredStatus('succeeded');
      return subscription;
    }

    this.connect(function() {
      this.info('Client ? attempting to subscribe to ?', this._dispatcher.clientId, channel);
      if (!force) this._channels.subscribe([channel], callback, context);

      this._sendMessage({
        channel:      Faye.Channel.SUBSCRIBE,
        clientId:     this._dispatcher.clientId,
        subscription: channel

      }, {}, function(response) {
        if (!response.successful) {
          subscription.setDeferredStatus('failed', Faye.Error.parse(response.error));
          return this._channels.unsubscribe(channel, callback, context);
        }

        var channels = [].concat(response.subscription);
        this.info('Subscription acknowledged for ? to ?', this._dispatcher.clientId, channels);
        subscription.setDeferredStatus('succeeded');
      }, this);
    }, this);

    return subscription;
  },

  // Request                              Response
  // MUST include:  * channel             MUST include:  * channel
  //                * clientId                           * successful
  //                * subscription                       * clientId
  // MAY include:   * ext                                * subscription
  //                * id                  MAY include:   * error
  //                                                     * advice
  //                                                     * ext
  //                                                     * id
  //                                                     * timestamp
  unsubscribe: function(channel, callback, context) {
    if (channel instanceof Array)
      return Faye.map(channel, function(c) {
        return this.unsubscribe(c, callback, context);
      }, this);

    var dead = this._channels.unsubscribe(channel, callback, context);
    if (!dead) return;

    this.connect(function() {
      this.info('Client ? attempting to unsubscribe from ?', this._dispatcher.clientId, channel);

      this._sendMessage({
        channel:      Faye.Channel.UNSUBSCRIBE,
        clientId:     this._dispatcher.clientId,
        subscription: channel

      }, {}, function(response) {
        if (!response.successful) return;

        var channels = [].concat(response.subscription);
        this.info('Unsubscription acknowledged for ? from ?', this._dispatcher.clientId, channels);
      }, this);
    }, this);
  },

  // Request                              Response
  // MUST include:  * channel             MUST include:  * channel
  //                * data                               * successful
  // MAY include:   * clientId            MAY include:   * id
  //                * id                                 * error
  //                * ext                                * ext
  publish: function(channel, data, options) {
    Faye.validateOptions(options || {}, ['attempts', 'deadline']);
    var publication = new Faye.Publication();

    this.connect(function() {
      this.info('Client ? queueing published message to ?: ?', this._dispatcher.clientId, channel, data);

      this._sendMessage({
        channel:  channel,
        data:     data,
        clientId: this._dispatcher.clientId

      }, options, function(response) {
        if (response.successful)
          publication.setDeferredStatus('succeeded');
        else
          publication.setDeferredStatus('failed', Faye.Error.parse(response.error));
      }, this);
    }, this);

    return publication;
  },

  _sendMessage: function(message, options, callback, context) {
    message.id = this._generateMessageId();

    var timeout = this._advice.timeout
                ? 1.2 * this._advice.timeout / 1000
                : 1.2 * this._dispatcher.retry;

    this.pipeThroughExtensions('outgoing', message, null, function(message) {
      if (!message) return;
      if (callback) this._responseCallbacks[message.id] = [callback, context];
      this._dispatcher.sendMessage(message, timeout, options || {});
    }, this);
  },

  _generateMessageId: function() {
    this._messageId += 1;
    if (this._messageId >= Math.pow(2,32)) this._messageId = 0;
    return this._messageId.toString(36);
  },

  _receiveMessage: function(message) {
    var id = message.id, callback;

    if (message.successful !== undefined) {
      callback = this._responseCallbacks[id];
      delete this._responseCallbacks[id];
    }

    this.pipeThroughExtensions('incoming', message, null, function(message) {
      if (!message) return;
      if (message.advice) this._handleAdvice(message.advice);
      this._deliverMessage(message);
      if (callback) callback[0].call(callback[1], message);
    }, this);
  },

  _handleAdvice: function(advice) {
    Faye.extend(this._advice, advice);
    this._dispatcher.timeout = this._advice.timeout / 1000;

    if (this._advice.reconnect === this.HANDSHAKE && this._state !== this.DISCONNECTED) {
      this._state = this.UNCONNECTED;
      this._dispatcher.clientId = null;
      this._cycleConnection();
    }
  },

  _deliverMessage: function(message) {
    if (!message.channel || message.data === undefined) return;
    this.info('Client ? calling listeners for ? with ?', this._dispatcher.clientId, message.channel, message.data);
    this._channels.distributeMessage(message);
  },

  _cycleConnection: function() {
    if (this._connectRequest) {
      this._connectRequest = null;
      this.info('Closed connection for ?', this._dispatcher.clientId);
    }
    var self = this;
    Faye.ENV.setTimeout(function() { self.connect() }, this._advice.interval);
  }
});

Faye.extend(Faye.Client.prototype, Faye.Deferrable);
Faye.extend(Faye.Client.prototype, Faye.Publisher);
Faye.extend(Faye.Client.prototype, Faye.Logging);
Faye.extend(Faye.Client.prototype, Faye.Extensible);

Faye.Dispatcher = Faye.Class({
  MAX_REQUEST_SIZE: 2048,
  DEFAULT_RETRY:    5,

  UP:   1,
  DOWN: 2,

  initialize: function(client, endpoint, options) {
    this._client     = client;
    this.endpoint    = Faye.URI.parse(endpoint);
    this._alternates = options.endpoints || {};

    this.cookies      = Faye.Cookies && new Faye.Cookies.CookieJar();
    this._disabled    = [];
    this._envelopes   = {};
    this.headers      = {};
    this.retry        = options.retry || this.DEFAULT_RETRY;
    this._scheduler   = options.scheduler || Faye.Scheduler;
    this._state       = 0;
    this.transports   = {};
    this.wsExtensions = [];

    this.proxy = options.proxy || {};
    if (typeof this._proxy === 'string') this._proxy = {origin: this._proxy};

    var exts = options.websocketExtensions;
    if (exts) {
      exts = [].concat(exts);
      for (var i = 0, n = exts.length; i < n; i++)
        this.addWebsocketExtension(exts[i]);
    }

    this.tls = options.tls || {};
    this.tls.ca = this.tls.ca || options.ca;

    for (var type in this._alternates)
      this._alternates[type] = Faye.URI.parse(this._alternates[type]);

    this.maxRequestSize = this.MAX_REQUEST_SIZE;
  },

  endpointFor: function(connectionType) {
    return this._alternates[connectionType] || this.endpoint;
  },

  addWebsocketExtension: function(extension) {
    this.wsExtensions.push(extension);
  },

  disable: function(feature) {
    this._disabled.push(feature);
  },

  setHeader: function(name, value) {
    this.headers[name] = value;
  },

  close: function() {
    var transport = this._transport;
    delete this._transport;
    if (transport) transport.close();
  },

  getConnectionTypes: function() {
    return Faye.Transport.getConnectionTypes();
  },

  selectTransport: function(transportTypes) {
    Faye.Transport.get(this, transportTypes, this._disabled, function(transport) {
      this.debug('Selected ? transport for ?', transport.connectionType, Faye.URI.stringify(transport.endpoint));

      if (transport === this._transport) return;
      if (this._transport) this._transport.close();

      this._transport = transport;
      this.connectionType = transport.connectionType;
    }, this);
  },

  sendMessage: function(message, timeout, options) {
    options = options || {};

    var id       = message.id,
        attempts = options.attempts,
        deadline = options.deadline && new Date().getTime() + (options.deadline * 1000),
        envelope = this._envelopes[id],
        scheduler;

    if (!envelope) {
      scheduler = new this._scheduler(message, {timeout: timeout, interval: this.retry, attempts: attempts, deadline: deadline});
      envelope  = this._envelopes[id] = {message: message, scheduler: scheduler};
    }

    this._sendEnvelope(envelope);
  },

  _sendEnvelope: function(envelope) {
    if (!this._transport) return;
    if (envelope.request || envelope.timer) return;

    var message   = envelope.message,
        scheduler = envelope.scheduler,
        self      = this;

    if (!scheduler.isDeliverable()) {
      scheduler.abort();
      delete this._envelopes[message.id];
      return;
    }

    envelope.timer = Faye.ENV.setTimeout(function() {
      self.handleError(message);
    }, scheduler.getTimeout() * 1000);

    scheduler.send();
    envelope.request = this._transport.sendMessage(message);
  },

  handleResponse: function(reply) {
    var envelope = this._envelopes[reply.id];

    if (reply.successful !== undefined && envelope) {
      envelope.scheduler.succeed();
      delete this._envelopes[reply.id];
      Faye.ENV.clearTimeout(envelope.timer);
    }

    this.trigger('message', reply);

    if (this._state === this.UP) return;
    this._state = this.UP;
    this._client.trigger('transport:up');
  },

  handleError: function(message, immediate) {
    var envelope = this._envelopes[message.id],
        request  = envelope && envelope.request,
        self     = this;

    if (!request) return;

    request.then(function(req) {
      if (req && req.abort) req.abort();
    });

    var scheduler = envelope.scheduler;
    scheduler.fail();

    Faye.ENV.clearTimeout(envelope.timer);
    envelope.request = envelope.timer = null;

    if (immediate) {
      this._sendEnvelope(envelope);
    } else {
      envelope.timer = Faye.ENV.setTimeout(function() {
        envelope.timer = null;
        self._sendEnvelope(envelope);
      }, scheduler.getInterval() * 1000);
    }

    if (this._state === this.DOWN) return;
    this._state = this.DOWN;
    this._client.trigger('transport:down');
  }
});

Faye.extend(Faye.Dispatcher.prototype, Faye.Publisher);
Faye.extend(Faye.Dispatcher.prototype, Faye.Logging);

Faye.Scheduler = function(message, options) {
  this.message  = message;
  this.options  = options;
  this.attempts = 0;
};

Faye.extend(Faye.Scheduler.prototype, {
  getTimeout: function() {
    return this.options.timeout;
  },

  getInterval: function() {
    return this.options.interval;
  },

  isDeliverable: function() {
    var attempts = this.options.attempts,
        made     = this.attempts,
        deadline = this.options.deadline,
        now      = new Date().getTime();

    if (attempts !== undefined && made >= attempts)
      return false;

    if (deadline !== undefined && now > deadline)
      return false;

    return true;
  },

  send: function() {
    this.attempts += 1;
  },

  succeed: function() {},

  fail: function() {},

  abort: function() {}
});

Faye.Transport = Faye.extend(Faye.Class({
  DEFAULT_PORTS:    {'http:': 80, 'https:': 443, 'ws:': 80, 'wss:': 443},
  SECURE_PROTOCOLS: ['https:', 'wss:'],
  MAX_DELAY:        0,

  batching:  true,

  initialize: function(dispatcher, endpoint) {
    this._dispatcher = dispatcher;
    this.endpoint    = endpoint;
    this._outbox     = [];
    this._proxy      = Faye.extend({}, this._dispatcher.proxy);

    if (!this._proxy.origin && Faye.NodeAdapter) {
      this._proxy.origin = Faye.indexOf(this.SECURE_PROTOCOLS, this.endpoint.protocol) >= 0
                         ? (process.env.HTTPS_PROXY || process.env.https_proxy)
                         : (process.env.HTTP_PROXY  || process.env.http_proxy);
    }
  },

  close: function() {},

  encode: function(messages) {
    return '';
  },

  sendMessage: function(message) {
    this.debug('Client ? sending message to ?: ?',
               this._dispatcher.clientId, Faye.URI.stringify(this.endpoint), message);

    if (!this.batching) return Faye.Promise.fulfilled(this.request([message]));

    this._outbox.push(message);
    this._promise = this._promise || new Faye.Promise();
    this._flushLargeBatch();

    if (message.channel === Faye.Channel.HANDSHAKE) {
      this.addTimeout('publish', 0.01, this._flush, this);
      return this._promise;
    }

    if (message.channel === Faye.Channel.CONNECT)
      this._connectMessage = message;

    this.addTimeout('publish', this.MAX_DELAY, this._flush, this);
    return this._promise;
  },

  _flush: function() {
    this.removeTimeout('publish');

    if (this._outbox.length > 1 && this._connectMessage)
      this._connectMessage.advice = {timeout: 0};

    Faye.Promise.fulfill(this._promise, this.request(this._outbox));
    delete this._promise;

    this._connectMessage = null;
    this._outbox = [];
  },

  _flushLargeBatch: function() {
    var string = this.encode(this._outbox);
    if (string.length < this._dispatcher.maxRequestSize) return;
    var last = this._outbox.pop();
    this._flush();
    if (last) this._outbox.push(last);
  },

  _receive: function(replies) {
    if (!replies) return;
    replies = [].concat(replies);

    this.debug('Client ? received from ? via ?: ?',
               this._dispatcher.clientId, Faye.URI.stringify(this.endpoint), this.connectionType, replies);

    for (var i = 0, n = replies.length; i < n; i++)
      this._dispatcher.handleResponse(replies[i]);
  },

  _handleError: function(messages, immediate) {
    messages = [].concat(messages);

    this.debug('Client ? failed to send to ? via ?: ?',
               this._dispatcher.clientId, Faye.URI.stringify(this.endpoint), this.connectionType, messages);

    for (var i = 0, n = messages.length; i < n; i++)
      this._dispatcher.handleError(messages[i]);
  },

  _getCookies: function() {
    var cookies = this._dispatcher.cookies,
        url     = Faye.URI.stringify(this.endpoint);

    if (!cookies) return '';

    return Faye.map(cookies.getCookiesSync(url), function(cookie) {
      return cookie.cookieString();
    }).join('; ');
  },

  _storeCookies: function(setCookie) {
    var cookies = this._dispatcher.cookies,
        url     = Faye.URI.stringify(this.endpoint),
        cookie;

    if (!setCookie || !cookies) return;
    setCookie = [].concat(setCookie);

    for (var i = 0, n = setCookie.length; i < n; i++) {
      cookie = Faye.Cookies.Cookie.parse(setCookie[i]);
      cookies.setCookieSync(cookie, url);
    }
  }

}), {
  get: function(dispatcher, allowed, disabled, callback, context) {
    var endpoint = dispatcher.endpoint;

    Faye.asyncEach(this._transports, function(pair, resume) {
      var connType     = pair[0], klass = pair[1],
          connEndpoint = dispatcher.endpointFor(connType);

      if (Faye.indexOf(disabled, connType) >= 0)
        return resume();

      if (Faye.indexOf(allowed, connType) < 0) {
        klass.isUsable(dispatcher, connEndpoint, function() {});
        return resume();
      }

      klass.isUsable(dispatcher, connEndpoint, function(isUsable) {
        if (!isUsable) return resume();
        var transport = klass.hasOwnProperty('create') ? klass.create(dispatcher, connEndpoint) : new klass(dispatcher, connEndpoint);
        callback.call(context, transport);
      });
    }, function() {
      throw new Error('Could not find a usable connection type for ' + Faye.URI.stringify(endpoint));
    });
  },

  register: function(type, klass) {
    this._transports.push([type, klass]);
    klass.prototype.connectionType = type;
  },

  getConnectionTypes: function() {
    return Faye.map(this._transports, function(t) { return t[0] });
  },

  _transports: []
});

Faye.extend(Faye.Transport.prototype, Faye.Logging);
Faye.extend(Faye.Transport.prototype, Faye.Timeouts);

Faye.Event = {
  _registry: [],

  on: function(element, eventName, callback, context) {
    var wrapped = function() { callback.call(context) };

    if (element.addEventListener)
      element.addEventListener(eventName, wrapped, false);
    else
      element.attachEvent('on' + eventName, wrapped);

    this._registry.push({
      _element:   element,
      _type:      eventName,
      _callback:  callback,
      _context:     context,
      _handler:   wrapped
    });
  },

  detach: function(element, eventName, callback, context) {
    var i = this._registry.length, register;
    while (i--) {
      register = this._registry[i];

      if ((element    && element    !== register._element)   ||
          (eventName  && eventName  !== register._type)      ||
          (callback   && callback   !== register._callback)  ||
          (context      && context      !== register._context))
        continue;

      if (register._element.removeEventListener)
        register._element.removeEventListener(register._type, register._handler, false);
      else
        register._element.detachEvent('on' + register._type, register._handler);

      this._registry.splice(i,1);
      register = null;
    }
  }
};

if (Faye.ENV.onunload !== undefined) Faye.Event.on(Faye.ENV, 'unload', Faye.Event.detach, Faye.Event);

/*
    json2.js
    2013-05-26

    Public Domain.

    NO WARRANTY EXPRESSED OR IMPLIED. USE AT YOUR OWN RISK.

    See http://www.JSON.org/js.html


    This code should be minified before deployment.
    See http://javascript.crockford.com/jsmin.html

    USE YOUR OWN COPY. IT IS EXTREMELY UNWISE TO LOAD CODE FROM SERVERS YOU DO
    NOT CONTROL.


    This file creates a global JSON object containing two methods: stringify
    and parse.

        JSON.stringify(value, replacer, space)
            value       any JavaScript value, usually an object or array.

            replacer    an optional parameter that determines how object
                        values are stringified for objects. It can be a
                        function or an array of strings.

            space       an optional parameter that specifies the indentation
                        of nested structures. If it is omitted, the text will
                        be packed without extra whitespace. If it is a number,
                        it will specify the number of spaces to indent at each
                        level. If it is a string (such as '\t' or '&nbsp;'),
                        it contains the characters used to indent at each level.

            This method produces a JSON text from a JavaScript value.

            When an object value is found, if the object contains a toJSON
            method, its toJSON method will be called and the result will be
            stringified. A toJSON method does not serialize: it returns the
            value represented by the name/value pair that should be serialized,
            or undefined if nothing should be serialized. The toJSON method
            will be passed the key associated with the value, and this will be
            bound to the value

            For example, this would serialize Dates as ISO strings.

                Date.prototype.toJSON = function (key) {
                    function f(n) {
                        // Format integers to have at least two digits.
                        return n < 10 ? '0' + n : n;
                    }

                    return this.getUTCFullYear()   + '-' +
                         f(this.getUTCMonth() + 1) + '-' +
                         f(this.getUTCDate())      + 'T' +
                         f(this.getUTCHours())     + ':' +
                         f(this.getUTCMinutes())   + ':' +
                         f(this.getUTCSeconds())   + 'Z';
                };

            You can provide an optional replacer method. It will be passed the
            key and value of each member, with this bound to the containing
            object. The value that is returned from your method will be
            serialized. If your method returns undefined, then the member will
            be excluded from the serialization.

            If the replacer parameter is an array of strings, then it will be
            used to select the members to be serialized. It filters the results
            such that only members with keys listed in the replacer array are
            stringified.

            Values that do not have JSON representations, such as undefined or
            functions, will not be serialized. Such values in objects will be
            dropped; in arrays they will be replaced with null. You can use
            a replacer function to replace those with JSON values.
            JSON.stringify(undefined) returns undefined.

            The optional space parameter produces a stringification of the
            value that is filled with line breaks and indentation to make it
            easier to read.

            If the space parameter is a non-empty string, then that string will
            be used for indentation. If the space parameter is a number, then
            the indentation will be that many spaces.

            Example:

            text = JSON.stringify(['e', {pluribus: 'unum'}]);
            // text is '["e",{"pluribus":"unum"}]'


            text = JSON.stringify(['e', {pluribus: 'unum'}], null, '\t');
            // text is '[\n\t"e",\n\t{\n\t\t"pluribus": "unum"\n\t}\n]'

            text = JSON.stringify([new Date()], function (key, value) {
                return this[key] instanceof Date ?
                    'Date(' + this[key] + ')' : value;
            });
            // text is '["Date(---current time---)"]'


        JSON.parse(text, reviver)
            This method parses a JSON text to produce an object or array.
            It can throw a SyntaxError exception.

            The optional reviver parameter is a function that can filter and
            transform the results. It receives each of the keys and values,
            and its return value is used instead of the original value.
            If it returns what it received, then the structure is not modified.
            If it returns undefined then the member is deleted.

            Example:

            // Parse the text. Values that look like ISO date strings will
            // be converted to Date objects.

            myData = JSON.parse(text, function (key, value) {
                var a;
                if (typeof value === 'string') {
                    a =
/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/.exec(value);
                    if (a) {
                        return new Date(Date.UTC(+a[1], +a[2] - 1, +a[3], +a[4],
                            +a[5], +a[6]));
                    }
                }
                return value;
            });

            myData = JSON.parse('["Date(09/09/2001)"]', function (key, value) {
                var d;
                if (typeof value === 'string' &&
                        value.slice(0, 5) === 'Date(' &&
                        value.slice(-1) === ')') {
                    d = new Date(value.slice(5, -1));
                    if (d) {
                        return d;
                    }
                }
                return value;
            });


    This is a reference implementation. You are free to copy, modify, or
    redistribute.
*/

/*jslint evil: true, regexp: true */

/*members "", "\b", "\t", "\n", "\f", "\r", "\"", JSON, "\\", apply,
    call, charCodeAt, getUTCDate, getUTCFullYear, getUTCHours,
    getUTCMinutes, getUTCMonth, getUTCSeconds, hasOwnProperty, join,
    lastIndex, length, parse, prototype, push, replace, slice, stringify,
    test, toJSON, toString, valueOf
*/


// Create a JSON object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (typeof JSON !== 'object') {
    JSON = {};
}

(function () {
    'use strict';

    function f(n) {
        // Format integers to have at least two digits.
        return n < 10 ? '0' + n : n;
    }

    if (typeof Date.prototype.toJSON !== 'function') {

        Date.prototype.toJSON = function () {

            return isFinite(this.valueOf())
                ? this.getUTCFullYear()     + '-' +
                    f(this.getUTCMonth() + 1) + '-' +
                    f(this.getUTCDate())      + 'T' +
                    f(this.getUTCHours())     + ':' +
                    f(this.getUTCMinutes())   + ':' +
                    f(this.getUTCSeconds())   + 'Z'
                : null;
        };

        String.prototype.toJSON      =
            Number.prototype.toJSON  =
            Boolean.prototype.toJSON = function () {
                return this.valueOf();
            };
    }

    var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        gap,
        indent,
        meta = {    // table of character substitutions
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        rep;


    function quote(string) {

// If the string contains no control characters, no quote characters, and no
// backslash characters, then we can safely slap some quotes around it.
// Otherwise we must also replace the offending characters with safe escape
// sequences.

        escapable.lastIndex = 0;
        return escapable.test(string) ? '"' + string.replace(escapable, function (a) {
            var c = meta[a];
            return typeof c === 'string'
                ? c
                : '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
        }) + '"' : '"' + string + '"';
    }


    function str(key, holder) {

// Produce a string from holder[key].

        var i,          // The loop counter.
            k,          // The member key.
            v,          // The member value.
            length,
            mind = gap,
            partial,
            value = holder[key];

// If the value has a toJSON method, call it to obtain a replacement value.

        if (value && typeof value === 'object' &&
                typeof value.toJSON === 'function') {
            value = value.toJSON(key);
        }

// If we were called with a replacer function, then call the replacer to
// obtain a replacement value.

        if (typeof rep === 'function') {
            value = rep.call(holder, key, value);
        }

// What happens next depends on the value's type.

        switch (typeof value) {
        case 'string':
            return quote(value);

        case 'number':

// JSON numbers must be finite. Encode non-finite numbers as null.

            return isFinite(value) ? String(value) : 'null';

        case 'boolean':
        case 'null':

// If the value is a boolean or null, convert it to a string. Note:
// typeof null does not produce 'null'. The case is included here in
// the remote chance that this gets fixed someday.

            return String(value);

// If the type is 'object', we might be dealing with an object or an array or
// null.

        case 'object':

// Due to a specification blunder in ECMAScript, typeof null is 'object',
// so watch out for that case.

            if (!value) {
                return 'null';
            }

// Make an array to hold the partial results of stringifying this object value.

            gap += indent;
            partial = [];

// Is the value an array?

            if (Object.prototype.toString.apply(value) === '[object Array]') {

// The value is an array. Stringify every element. Use null as a placeholder
// for non-JSON values.

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || 'null';
                }

// Join all of the elements together, separated with commas, and wrap them in
// brackets.

                v = partial.length === 0
                    ? '[]'
                    : gap
                    ? '[\n' + gap + partial.join(',\n' + gap) + '\n' + mind + ']'
                    : '[' + partial.join(',') + ']';
                gap = mind;
                return v;
            }

// If the replacer is an array, use it to select the members to be stringified.

            if (rep && typeof rep === 'object') {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    if (typeof rep[i] === 'string') {
                        k = rep[i];
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            } else {

// Otherwise, iterate through all of the keys in the object.

                for (k in value) {
                    if (Object.prototype.hasOwnProperty.call(value, k)) {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            }

// Join all of the member texts together, separated with commas,
// and wrap them in braces.

            v = partial.length === 0
                ? '{}'
                : gap
                ? '{\n' + gap + partial.join(',\n' + gap) + '\n' + mind + '}'
                : '{' + partial.join(',') + '}';
            gap = mind;
            return v;
        }
    }

// If the JSON object does not yet have a stringify method, give it one.

    Faye.stringify = function (value, replacer, space) {

// The stringify method takes a value and an optional replacer, and an optional
// space parameter, and returns a JSON text. The replacer can be a function
// that can replace values, or an array of strings that will select the keys.
// A default replacer method can be provided. Use of the space parameter can
// produce text that is more easily readable.

        var i;
        gap = '';
        indent = '';

// If the space parameter is a number, make an indent string containing that
// many spaces.

        if (typeof space === 'number') {
            for (i = 0; i < space; i += 1) {
                indent += ' ';
            }

// If the space parameter is a string, it will be used as the indent string.

        } else if (typeof space === 'string') {
            indent = space;
        }

// If there is a replacer, it must be a function or an array.
// Otherwise, throw an error.

        rep = replacer;
        if (replacer && typeof replacer !== 'function' &&
                (typeof replacer !== 'object' ||
                typeof replacer.length !== 'number')) {
            throw new Error('JSON.stringify');
        }

// Make a fake root object containing our value under the key of ''.
// Return the result of stringifying the value.

        return str('', {'': value});
    };

    if (typeof JSON.stringify !== 'function') {
        JSON.stringify = Faye.stringify;
    }

// If the JSON object does not yet have a parse method, give it one.

    if (typeof JSON.parse !== 'function') {
        JSON.parse = function (text, reviver) {

// The parse method takes a text and an optional reviver function, and returns
// a JavaScript value if the text is a valid JSON text.

            var j;

            function walk(holder, key) {

// The walk method is used to recursively walk the resulting structure so
// that modifications can be made.

                var k, v, value = holder[key];
                if (value && typeof value === 'object') {
                    for (k in value) {
                        if (Object.prototype.hasOwnProperty.call(value, k)) {
                            v = walk(value, k);
                            if (v !== undefined) {
                                value[k] = v;
                            } else {
                                delete value[k];
                            }
                        }
                    }
                }
                return reviver.call(holder, key, value);
            }


// Parsing happens in four stages. In the first stage, we replace certain
// Unicode characters with escape sequences. JavaScript handles many characters
// incorrectly, either silently deleting them, or treating them as line endings.

            text = String(text);
            cx.lastIndex = 0;
            if (cx.test(text)) {
                text = text.replace(cx, function (a) {
                    return '\\u' +
                        ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
                });
            }

// In the second stage, we run the text against regular expressions that look
// for non-JSON patterns. We are especially concerned with '()' and 'new'
// because they can cause invocation, and '=' because it can cause mutation.
// But just to be safe, we want to reject all unexpected forms.

// We split the second stage into 4 regexp operations in order to work around
// crippling inefficiencies in IE's and Safari's regexp engines. First we
// replace the JSON backslash pairs with '@' (a non-JSON character). Second, we
// replace all simple value tokens with ']' characters. Third, we delete all
// open brackets that follow a colon or comma or that begin the text. Finally,
// we look to see that the remaining characters are only whitespace or ']' or
// ',' or ':' or '{' or '}'. If that is so, then the text is safe for eval.

            if (/^[\],:{}\s]*$/
                    .test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@')
                        .replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']')
                        .replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

// In the third stage we use the eval function to compile the text into a
// JavaScript structure. The '{' operator is subject to a syntactic ambiguity
// in JavaScript: it can begin a block or an object literal. We wrap the text
// in parens to eliminate the ambiguity.

                j = eval('(' + text + ')');

// In the optional fourth stage, we recursively walk the new structure, passing
// each name/value pair to a reviver function for possible transformation.

                return typeof reviver === 'function'
                    ? walk({'': j}, '')
                    : j;
            }

// If the text is not JSON parseable, then a SyntaxError is thrown.

            throw new SyntaxError('JSON.parse');
        };
    }
}());

Faye.Transport.WebSocket = Faye.extend(Faye.Class(Faye.Transport, {
  UNCONNECTED:  1,
  CONNECTING:   2,
  CONNECTED:    3,

  batching:     false,

  isUsable: function(callback, context) {
    this.callback(function() { callback.call(context, true) });
    this.errback(function() { callback.call(context, false) });
    this.connect();
  },

  request: function(messages) {
    this._pending = this._pending || new Faye.Set();
    for (var i = 0, n = messages.length; i < n; i++) this._pending.add(messages[i]);

    var promise = new Faye.Promise();

    this.callback(function(socket) {
      if (!socket || socket.readyState !== 1) return;
      socket.send(Faye.toJSON(messages));
      Faye.Promise.fulfill(promise, socket);
    }, this);

    this.connect();

    return {
      abort: function() { promise.then(function(ws) { ws.close() }) }
    };
  },

  connect: function() {
    if (Faye.Transport.WebSocket._unloaded) return;

    this._state = this._state || this.UNCONNECTED;
    if (this._state !== this.UNCONNECTED) return;
    this._state = this.CONNECTING;

    var socket = this._createSocket();
    if (!socket) return this.setDeferredStatus('failed');

    var self = this;

    socket.onopen = function() {
      if (socket.headers) self._storeCookies(socket.headers['set-cookie']);
      self._socket = socket;
      self._state = self.CONNECTED;
      self._everConnected = true;
      self._ping();
      self.setDeferredStatus('succeeded', socket);
    };

    var closed = false;
    socket.onclose = socket.onerror = function() {
      if (closed) return;
      closed = true;

      var wasConnected = (self._state === self.CONNECTED);
      socket.onopen = socket.onclose = socket.onerror = socket.onmessage = null;

      delete self._socket;
      self._state = self.UNCONNECTED;
      self.removeTimeout('ping');
      self.setDeferredStatus('unknown');

      var pending = self._pending ? self._pending.toArray() : [];
      delete self._pending;

      if (wasConnected) {
        self._handleError(pending, true);
      } else if (self._everConnected) {
        self._handleError(pending);
      } else {
        self.setDeferredStatus('failed');
      }
    };

    socket.onmessage = function(event) {
      var replies = JSON.parse(event.data);
      if (!replies) return;

      replies = [].concat(replies);

      for (var i = 0, n = replies.length; i < n; i++) {
        if (replies[i].successful === undefined) continue;
        self._pending.remove(replies[i]);
      }
      self._receive(replies);
    };
  },

  close: function() {
    if (!this._socket) return;
    this._socket.close();
  },

  _createSocket: function() {
    var url        = Faye.Transport.WebSocket.getSocketUrl(this.endpoint),
        headers    = this._dispatcher.headers,
        extensions = this._dispatcher.wsExtensions,
        cookie     = this._getCookies(),
        tls        = this._dispatcher.tls,
        options    = {extensions: extensions, headers: headers, proxy: this._proxy, tls: tls};

    if (cookie !== '') options.headers['Cookie'] = cookie;

    if (Faye.WebSocket)        return new Faye.WebSocket.Client(url, [], options);
    if (Faye.ENV.MozWebSocket) return new MozWebSocket(url);
    if (Faye.ENV.WebSocket)    return new WebSocket(url);
  },

  _ping: function() {
    if (!this._socket) return;
    this._socket.send('[]');
    this.addTimeout('ping', this._dispatcher.timeout / 2, this._ping, this);
  }

}), {
  PROTOCOLS: {
    'http:':  'ws:',
    'https:': 'wss:'
  },

  create: function(dispatcher, endpoint) {
    var sockets = dispatcher.transports.websocket = dispatcher.transports.websocket || {};
    sockets[endpoint.href] = sockets[endpoint.href] || new this(dispatcher, endpoint);
    return sockets[endpoint.href];
  },

  getSocketUrl: function(endpoint) {
    endpoint = Faye.copyObject(endpoint);
    endpoint.protocol = this.PROTOCOLS[endpoint.protocol];
    return Faye.URI.stringify(endpoint);
  },

  isUsable: function(dispatcher, endpoint, callback, context) {
    this.create(dispatcher, endpoint).isUsable(callback, context);
  }
});

Faye.extend(Faye.Transport.WebSocket.prototype, Faye.Deferrable);
Faye.Transport.register('websocket', Faye.Transport.WebSocket);

if (Faye.Event && Faye.ENV.onbeforeunload !== undefined)
  Faye.Event.on(Faye.ENV, 'beforeunload', function() {
    Faye.Transport.WebSocket._unloaded = true;
  });

Faye.Transport.EventSource = Faye.extend(Faye.Class(Faye.Transport, {
  initialize: function(dispatcher, endpoint) {
    Faye.Transport.prototype.initialize.call(this, dispatcher, endpoint);
    if (!Faye.ENV.EventSource) return this.setDeferredStatus('failed');

    this._xhr = new Faye.Transport.XHR(dispatcher, endpoint);

    endpoint = Faye.copyObject(endpoint);
    endpoint.pathname += '/' + dispatcher.clientId;

    var socket = new EventSource(Faye.URI.stringify(endpoint)),
        self   = this;

    socket.onopen = function() {
      self._everConnected = true;
      self.setDeferredStatus('succeeded');
    };

    socket.onerror = function() {
      if (self._everConnected) {
        self._handleError([]);
      } else {
        self.setDeferredStatus('failed');
        socket.close();
      }
    };

    socket.onmessage = function(event) {
      self._receive(JSON.parse(event.data));
    };

    this._socket = socket;
  },

  close: function() {
    if (!this._socket) return;
    this._socket.onopen = this._socket.onerror = this._socket.onmessage = null;
    this._socket.close();
    delete this._socket;
  },

  isUsable: function(callback, context) {
    this.callback(function() { callback.call(context, true) });
    this.errback(function() { callback.call(context, false) });
  },

  encode: function(messages) {
    return this._xhr.encode(messages);
  },

  request: function(messages) {
    return this._xhr.request(messages);
  }

}), {
  isUsable: function(dispatcher, endpoint, callback, context) {
    var id = dispatcher.clientId;
    if (!id) return callback.call(context, false);

    Faye.Transport.XHR.isUsable(dispatcher, endpoint, function(usable) {
      if (!usable) return callback.call(context, false);
      this.create(dispatcher, endpoint).isUsable(callback, context);
    }, this);
  },

  create: function(dispatcher, endpoint) {
    var sockets = dispatcher.transports.eventsource = dispatcher.transports.eventsource || {},
        id      = dispatcher.clientId;

    var url = Faye.copyObject(endpoint);
    url.pathname += '/' + (id || '');
    url = Faye.URI.stringify(url);

    sockets[url] = sockets[url] || new this(dispatcher, endpoint);
    return sockets[url];
  }
});

Faye.extend(Faye.Transport.EventSource.prototype, Faye.Deferrable);
Faye.Transport.register('eventsource', Faye.Transport.EventSource);

Faye.Transport.XHR = Faye.extend(Faye.Class(Faye.Transport, {
  encode: function(messages) {
    return Faye.toJSON(messages);
  },

  request: function(messages) {
    var href = this.endpoint.href,
        xhr  = Faye.ENV.ActiveXObject ? new ActiveXObject('Microsoft.XMLHTTP') : new XMLHttpRequest(),
        self = this;

    xhr.open('POST', href, true);
    xhr.setRequestHeader('Content-Type', 'application/json');
    xhr.setRequestHeader('Pragma', 'no-cache');
    xhr.setRequestHeader('X-Requested-With', 'XMLHttpRequest');

    var headers = this._dispatcher.headers;
    for (var key in headers) {
      if (!headers.hasOwnProperty(key)) continue;
      xhr.setRequestHeader(key, headers[key]);
    }

    var abort = function() { xhr.abort() };
    if (Faye.ENV.onbeforeunload !== undefined) Faye.Event.on(Faye.ENV, 'beforeunload', abort);

    xhr.onreadystatechange = function() {
      if (!xhr || xhr.readyState !== 4) return;

      var replies    = null,
          status     = xhr.status,
          text       = xhr.responseText,
          successful = (status >= 200 && status < 300) || status === 304 || status === 1223;

      if (Faye.ENV.onbeforeunload !== undefined) Faye.Event.detach(Faye.ENV, 'beforeunload', abort);
      xhr.onreadystatechange = function() {};
      xhr = null;

      if (!successful) return self._handleError(messages);

      try {
        replies = JSON.parse(text);
      } catch (e) {}

      if (replies)
        self._receive(replies);
      else
        self._handleError(messages);
    };

    xhr.send(this.encode(messages));
    return xhr;
  }
}), {
  isUsable: function(dispatcher, endpoint, callback, context) {
    callback.call(context, Faye.URI.isSameOrigin(endpoint));
  }
});

Faye.Transport.register('long-polling', Faye.Transport.XHR);

Faye.Transport.CORS = Faye.extend(Faye.Class(Faye.Transport, {
  encode: function(messages) {
    return 'message=' + encodeURIComponent(Faye.toJSON(messages));
  },

  request: function(messages) {
    var xhrClass = Faye.ENV.XDomainRequest ? XDomainRequest : XMLHttpRequest,
        xhr      = new xhrClass(),
        id       = ++Faye.Transport.CORS._id,
        headers  = this._dispatcher.headers,
        self     = this,
        key;

    xhr.open('POST', Faye.URI.stringify(this.endpoint), true);

    if (xhr.setRequestHeader) {
      xhr.setRequestHeader('Pragma', 'no-cache');
      for (key in headers) {
        if (!headers.hasOwnProperty(key)) continue;
        xhr.setRequestHeader(key, headers[key]);
      }
    }

    var cleanUp = function() {
      if (!xhr) return false;
      Faye.Transport.CORS._pending.remove(id);
      xhr.onload = xhr.onerror = xhr.ontimeout = xhr.onprogress = null;
      xhr = null;
    };

    xhr.onload = function() {
      var replies = null;
      try {
        replies = JSON.parse(xhr.responseText);
      } catch (e) {}

      cleanUp();

      if (replies)
        self._receive(replies);
      else
        self._handleError(messages);
    };

    xhr.onerror = xhr.ontimeout = function() {
      cleanUp();
      self._handleError(messages);
    };

    xhr.onprogress = function() {};

    if (xhrClass === Faye.ENV.XDomainRequest)
      Faye.Transport.CORS._pending.add({id: id, xhr: xhr});

    xhr.send(this.encode(messages));
    return xhr;
  }
}), {
  _id:      0,
  _pending: new Faye.Set(),

  isUsable: function(dispatcher, endpoint, callback, context) {
    if (Faye.URI.isSameOrigin(endpoint))
      return callback.call(context, false);

    if (Faye.ENV.XDomainRequest)
      return callback.call(context, endpoint.protocol === Faye.ENV.location.protocol);

    if (Faye.ENV.XMLHttpRequest) {
      var xhr = new Faye.ENV.XMLHttpRequest();
      return callback.call(context, xhr.withCredentials !== undefined);
    }
    return callback.call(context, false);
  }
});

Faye.Transport.register('cross-origin-long-polling', Faye.Transport.CORS);

Faye.Transport.JSONP = Faye.extend(Faye.Class(Faye.Transport, {
 encode: function(messages) {
    var url = Faye.copyObject(this.endpoint);
    url.query.message = Faye.toJSON(messages);
    url.query.jsonp   = '__jsonp' + Faye.Transport.JSONP._cbCount + '__';
    return Faye.URI.stringify(url);
  },

  request: function(messages) {
    var head         = document.getElementsByTagName('head')[0],
        script       = document.createElement('script'),
        callbackName = Faye.Transport.JSONP.getCallbackName(),
        endpoint     = Faye.copyObject(this.endpoint),
        self         = this;

    endpoint.query.message = Faye.toJSON(messages);
    endpoint.query.jsonp   = callbackName;

    var cleanup = function() {
      if (!Faye.ENV[callbackName]) return false;
      Faye.ENV[callbackName] = undefined;
      try { delete Faye.ENV[callbackName] } catch (e) {}
      script.parentNode.removeChild(script);
    };

    Faye.ENV[callbackName] = function(replies) {
      cleanup();
      self._receive(replies);
    };

    script.type = 'text/javascript';
    script.src  = Faye.URI.stringify(endpoint);
    head.appendChild(script);

    script.onerror = function() {
      cleanup();
      self._handleError(messages);
    };

    return {abort: cleanup};
  }
}), {
  _cbCount: 0,

  getCallbackName: function() {
    this._cbCount += 1;
    return '__jsonp' + this._cbCount + '__';
  },

  isUsable: function(dispatcher, endpoint, callback, context) {
    callback.call(context, true);
  }
});

Faye.Transport.register('callback-polling', Faye.Transport.JSONP);

})();
(function(doc) {

  log = noop; // noOp, remove this line to enable debugging

  var coordinateSystemForElementFromPoint;

  function main(config) {
    config = config || {};

    coordinateSystemForElementFromPoint = navigator.userAgent.match(/OS [1-4](?:_\d+)+ like Mac/) ? "page" : "client";

    var div = doc.createElement('div');
    var dragDiv = 'draggable' in div;
    var evts = 'ondragstart' in div && 'ondrop' in div;

    var needsPatch = !(dragDiv || evts) || /iPad|iPhone|iPod|Android/.test(navigator.userAgent);
    log((needsPatch ? "" : "not ") + "patching html5 drag drop");

    if(!needsPatch) return;

    if(!config.enableEnterLeave) {
      DragDrop.prototype.synthesizeEnterLeave = noop;
    }

    doc.addEventListener("touchstart", touchstart);
  }

  function DragDrop(event, el) {

    this.dragData = {};
    this.dragDataTypes = [];
    this.dragImage = null;
    this.dragImageTransform = null;
    this.dragImageWebKitTransform = null;
    this.el = el || event.target

    log("dragstart");

    this.dispatchDragStart()
    this.createDragImage();

    this.listen()

  }

  DragDrop.prototype = {
    listen: function() {
      var move = onEvt(doc, "touchmove", this.move, this);
      var end = onEvt(doc, "touchend", ontouchend, this);
      var cancel = onEvt(doc, "touchcancel", cleanup, this);

      function ontouchend(event) {
        this.dragend(event, event.target);
        cleanup.call(this);
      }
      function cleanup() {
        log("cleanup");
        this.dragDataTypes = [];
        if (this.dragImage != null) {
          this.dragImage.parentNode.removeChild(this.dragImage);
          this.dragImage = null;
          this.dragImageTransform = null;
          this.dragImageWebKitTransform = null;
        }
        this.el = this.dragData = null;
        return [move, end, cancel].forEach(function(handler) {
          return handler.off();
        });
      }
    },
    move: function(event) {
      var pageXs = [], pageYs = [];
      [].forEach.call(event.changedTouches, function(touch, index) {
        pageXs.push(touch.pageX);
        pageYs.push(touch.pageY);
      });

      var x = average(pageXs) - (parseInt(this.dragImage.offsetWidth, 10) / 2);
      var y = average(pageYs) - (parseInt(this.dragImage.offsetHeight, 10) / 2);
      this.translateDragImage(x, y);

      this.synthesizeEnterLeave(event);
    },
    hideDragImage: function() {
      if (this.dragImage && this.dragImage.style["display"] != "none") {
        this.dragImageDisplay = this.dragImage.style["display"];
        this.dragImage.style["display"] = "none";
      }
    },
    showDragImage: function() {
      if (this.dragImage) {
        this.dragImage.style["display"] = this.dragImageDisplay ? this.dragImageDisplay : "block";
      }
    },
    // We use translate instead of top/left because of sub-pixel rendering and for the hope of better performance
    // http://www.paulirish.com/2012/why-moving-elements-with-translate-is-better-than-posabs-topleft/
    translateDragImage: function(x, y) {
      var translate = " translate(" + x + "px," + y + "px)";

      if (this.dragImageWebKitTransform !== null) {
        this.dragImage.style["-webkit-transform"] = this.dragImageWebKitTransform + translate;
      }
      if (this.dragImageTransform !== null) {
        this.dragImage.style["transform"] = this.dragImageTransform + translate;
      }
    },
    synthesizeEnterLeave: function(event) {
      this.hideDragImage();
      var target = elementFromTouchEvent(this.el,event)
      this.showDragImage();
      if (target != this.lastEnter) {
        if (this.lastEnter) {
          this.dispatchLeave(event);
        }
        this.lastEnter = target;
        if (this.lastEnter) {
          this.dispatchEnter(event);
        }
      }
      if (this.lastEnter) {
        this.dispatchOver(event);
      }
    },
    dragend: function(event) {

      // we'll dispatch drop if there's a target, then dragEnd.
      // drop comes first http://www.whatwg.org/specs/web-apps/current-work/multipage/dnd.html#drag-and-drop-processing-model
      log("dragend");

      if (this.lastEnter) {
        this.dispatchLeave(event);
      }

      this.hideDragImage();
      var target = elementFromTouchEvent(this.el,event)
      this.showDragImage();

      if (target) {
        log("found drop target " + target.tagName);
        this.dispatchDrop(target, event)
      } else {
        log("no drop target")
      }

      var dragendEvt = doc.createEvent("Event");
      dragendEvt.initEvent("dragend", true, true);
      this.el.dispatchEvent(dragendEvt);
    },
    dispatchDrop: function(target, event) {
      var dropEvt = doc.createEvent("Event");
      dropEvt.initEvent("drop", true, true);

      var touch = event.changedTouches[0];
      var x = touch[coordinateSystemForElementFromPoint + 'X'];
      var y = touch[coordinateSystemForElementFromPoint + 'Y'];
      dropEvt.offsetX = x - target.x;
      dropEvt.offsetY = y - target.y;

      dropEvt.dataTransfer = {
        types: this.dragDataTypes,
        getData: function(type) {
          return this.dragData[type];
        }.bind(this)
      };
      dropEvt.preventDefault = function() {
         // https://www.w3.org/Bugs/Public/show_bug.cgi?id=14638 - if we don't cancel it, we'll snap back
      }.bind(this);

      once(doc, "drop", function() {
        log("drop event not canceled");
      },this);

      target.dispatchEvent(dropEvt);
    },
    dispatchEnter: function(event) {

      var enterEvt = doc.createEvent("Event");
      enterEvt.initEvent("dragenter", true, true);
      enterEvt.dataTransfer = {
        types: this.dragDataTypes,
        getData: function(type) {
          return this.dragData[type];
        }.bind(this)
      };

      var touch = event.changedTouches[0];
      enterEvt.pageX = touch.pageX;
      enterEvt.pageY = touch.pageY;

      this.lastEnter.dispatchEvent(enterEvt);
    },
    dispatchOver: function(event) {

      var overEvt = doc.createEvent("Event");
      overEvt.initEvent("dragover", true, true);
      overEvt.dataTransfer = {
        types: this.dragDataTypes,
        getData: function(type) {
          return this.dragData[type];
        }.bind(this)
      };

      var touch = event.changedTouches[0];
      overEvt.pageX = touch.pageX;
      overEvt.pageY = touch.pageY;

      this.lastEnter.dispatchEvent(overEvt);
    },
    dispatchLeave: function(event) {

      var leaveEvt = doc.createEvent("Event");
      leaveEvt.initEvent("dragleave", true, true);
      leaveEvt.dataTransfer = {
        types: this.dragDataTypes,
        getData: function(type) {
          return this.dragData[type];
        }.bind(this)
      };

      var touch = event.changedTouches[0];
      leaveEvt.pageX = touch.pageX;
      leaveEvt.pageY = touch.pageY;

      this.lastEnter.dispatchEvent(leaveEvt);
      this.lastEnter = null;
    },
    dispatchDragStart: function() {
      var evt = doc.createEvent("Event");
      evt.initEvent("dragstart", true, true);
      evt.dataTransfer = {
        setData: function(type, val) {
          this.dragData[type] = val;
          if (this.dragDataTypes.indexOf(type) == -1) {
            this.dragDataTypes[this.dragDataTypes.length] = type;
          }
          return val;
        }.bind(this),
        dropEffect: "move"
      };
      this.el.dispatchEvent(evt);
    },
    createDragImage: function() {
      this.dragImage = this.el.cloneNode(true);
      
      duplicateStyle(this.el, this.dragImage);
      
      this.dragImage.style["opacity"] = "0.5";
      this.dragImage.style["position"] = "absolute";
      this.dragImage.style["left"] = "0px";
      this.dragImage.style["top"] = "0px";
      this.dragImage.style["z-index"] = "999999";
      this.dragImage.style["pointer-events"] = "none";

      var transform = this.dragImage.style["transform"];
      if (typeof transform !== "undefined") {
        this.dragImageTransform = "";
        if (transform != "none") {
          this.dragImageTransform = transform.replace(/translate\(\D*\d+[^,]*,\D*\d+[^,]*\)\s*/g, '');
        }
      }

      var webkitTransform = this.dragImage.style["-webkit-transform"];
      if (typeof webkitTransform !== "undefined") {
        this.dragImageWebKitTransform = "";
        if (webkitTransform != "none") {
          this.dragImageWebKitTransform = webkitTransform.replace(/translate\(\D*\d+[^,]*,\D*\d+[^,]*\)\s*/g, '');
        }
      }

      this.translateDragImage(-9999, -9999);

      doc.body.appendChild(this.dragImage);
    }
  }

  // event listeners
  function touchstart(evt) {
    var el = evt.target;
    do {
      if (el.draggable === true) {
        // If draggable isn't explicitly set for anchors, then simulate a click event.
        // Otherwise plain old vanilla links will stop working.
        // https://developer.mozilla.org/en-US/docs/Web/Guide/Events/Touch_events#Handling_clicks
        if (!el.hasAttribute("draggable") && el.tagName.toLowerCase() == "a") {
          var clickEvt = document.createEvent("MouseEvents");
          clickEvt.initMouseEvent("click", true, true, el.ownerDocument.defaultView, 1,
            evt.screenX, evt.screenY, evt.clientX, evt.clientY,
            evt.ctrlKey, evt.altKey, evt.shiftKey, evt.metaKey, 0, null);
          el.dispatchEvent(clickEvt);
          log("Simulating click to anchor");
        }
        evt.preventDefault();
        new DragDrop(evt,el);
      }
    } while((el = el.parentNode) && el !== doc.body)
  }

  // DOM helpers
  function elementFromTouchEvent(el,event) {
    var touch = event.changedTouches[0];
    var target = doc.elementFromPoint(
      touch[coordinateSystemForElementFromPoint + "X"],
      touch[coordinateSystemForElementFromPoint + "Y"]
    );
    return target
  }

  function onEvt(el, event, handler, context) {
    if(context) handler = handler.bind(context)
    el.addEventListener(event, handler);
    return {
      off: function() {
        return el.removeEventListener(event, handler);
      }
    };
  }

  function once(el, event, handler, context) {
    if(context) handler = handler.bind(context)
    function listener(evt) {
      handler(evt);
      return el.removeEventListener(event,listener);
    }
    return el.addEventListener(event,listener);
  }

  // duplicateStyle expects dstNode to be a clone of srcNode
  function duplicateStyle(srcNode, dstNode) {
    // Is this node an element?
    if (srcNode.nodeType == 1) {
      // Remove any potential conflict attributes
      dstNode.removeAttribute("id");
      dstNode.removeAttribute("class");
      dstNode.removeAttribute("style");
      dstNode.removeAttribute("draggable");

      // Clone the style
      var cs = window.getComputedStyle(srcNode);
      for (var i = 0; i < cs.length; i++) {
        var csName = cs[i];
        dstNode.style.setProperty(csName, cs.getPropertyValue(csName), cs.getPropertyPriority(csName));
      }
    }

    // Do the same for the children
    if (srcNode.hasChildNodes()) {
      for (var i = 0; i < srcNode.childNodes.length; i++) {
        duplicateStyle(srcNode.childNodes[i], dstNode.childNodes[i]);
      }
    }
  }

  // general helpers
  function log(msg) {
    console.log(msg);
  }

  function average(arr) {
    if (arr.length === 0) return 0;
    return arr.reduce((function(s, v) {
      return v + s;
    }), 0) / arr.length;
  }

  function noop() {}

  main(window.iosDragDropShim);


})(document);

/** PBS Kids Trampoline, HTML5
 *
 * Authored by Ivy Nye | inye@pbs.org
 * Designed by Chris Bishop
 * @copyright (c) 2016 PBS Kids Digital
 * @version 1.2.2
 *
 * @namespace Trampoline
 * @example Trampoline.init({app_name: test, parents: true});
 * 
 * TODO: Refactor Module Pattern - get with the times, better garbage collection 
 * TODO: Make a Trampoline 'Needs Refresh' boolean, in the event that something borked during startup
 */
var Trampoline = (function () {
  var _APP_NAME           = "PBS Kids Trampoline",
      _APP_VERSION        = "1.2.2",
      PBSKidsAppName,
      GUID,
      _didInit,
      _modulesDidInit,
      _debug              = false || window.location.hostname.match(/^(((?!www(\-tc)?|help\.).+)pbskids\.org)|(localhost:*)|(\d+\.\d+\.\d+\.\d+:8081*)/) || window.location.port.match(/^8081|(\.\d+:8081)/) ? true : false,
      _log                = function(message,args,type,force){if(Trampoline._debug===true||force===true){if( typeof message == "string" || !!args ){ message = _APP_NAME + " ver. " + _APP_VERSION + " | " + message;} else { args = message; message = _APP_NAME + " ver. " + _APP_VERSION + " | "; } if(typeof console!=="undefined"){if(type=="error")if(console.error){console.error(message,args);return;}else if(window.Error)throw new Error(message);if(type=="info"&&console.info){console.info(message,args);return;}if(type=="warn"&&console.warn){console.warn(message,args);return;}if(console.log)console.log(message,args);}else if(window.debug)window.debug.log.apply(message,args);}},
      _showBuildData      = false,
      _buttonArr          = [],
      _notifierArr        = [],
      _hasSplash          = true,
      _hasNav             = true,
      _hasNotifiers        = false,
      _hasMoreApps        = true,
      _superVisionConfig  = null, 
      _hasParents         = true,
      _hasPopups          = true,
	  _hasSettings 		  = true,
      _hasSettingsModule,
      _initConfig,
      _configUA,
      _devOnReady,
      _localize             = { auto: true },
      _path                 = './',
      _resourcePath         = '',
      _markupURL            = 'trampoline.html', //This location can be set in config 'path'
      _trampolineContainer  = document.getElementById('trampoline'),
      
      //EVENTS
      TRAMPOLINE_READY = new Event('TRAMPOLINE_READY'),
      TRAMPOLINE_REFRESH = new Event ('TRAMPOLINE_REFRESH'),
      TRAMPOLINE_ONLINE = new Event('TRAMPOLINE_ONLINE'),
      TRAMPOLINE_OFFLINE = new Event('TRAMPOLINE_OFFLINE'),
      TRAMPOLINE_STATE_CHANGE = new Event ('TRAMPOLINE_STATE_CHANGE'),
      TRAMPOLINE_STATE_BACK = new Event ('TRAMPOLINE_STATE_BACK'),
      
      //TODO: Add a custom function that creates a TRAMPOLINE_ERROR event with custom details
      
      /**
       * Initializes Trampoline from config object
       * @memberof Trampoline
       * @name Trampoline.init
       * @param {Object} configObj Configuration Details
       *                           
       * @todo TODO: allow init config to reference a file location for config json.
       */    
      init = function(configObj){
        if (!_didInit){
          Trampoline.state.change({to: 'init', from: 'uninit'});
          
          Trampoline.events.addEventListener('TRAMPOLINE_STATE_CHANGE', function(){
            _log('change state: ', Trampoline.state.current.name);
          });
            
          Trampoline.events.addEventListener('TRAMPOLINE_STATE_BACK', function(){
            _log('state back: ', Trampoline.state.current.name);
          });
          
          document.addEventListener('online', _onConnectionResume);
          document.addEventListener('offline', _onConnectionLost);

          _config(configObj);
          
          var _onTrampolineInject = function (){
            //Init analytics first, Splash page will need it.
            Trampoline.analytics.init(_configUA);
            Trampoline.analytics.dimension(1, 'NO_STATION');
            
            // !! returns _hasSplash true for both BOOL true and the 'manual-hide' string if config is set
            if (!!_hasSplash){
              Trampoline.state.change({to: 'splash', from: 'init'});
            }else{
              //TODO: Something wrong with no-splash mode 
              Trampoline.splash.remove();
              Trampoline.state.change({to: 'main', from: 'init'});
              Trampoline.onReady();
            }
            _didInit = true;
          };
          
          _buildMarkup(_onTrampolineInject);
          
          _trampolineContainer.addEventListener('touchmove', Trampoline.util.preventDefaultThis);
          
          // Show the build data in debug mode
          // TODO: move this into it's own thing, so it's out of core
          
          if (_showBuildData){
            if (!!Trampoline.util.buildData){
              Trampoline.util.displayBuildData();
            }else{
              Trampoline.events.addEventListener('BUILD_DATA_RETURNED', function(){
                Trampoline.util.displayBuildData();
              });
              Trampoline.util.getBuildData();
            }
          }
          
        }else{
          _log('Trampoline.init was already executed');
        }
      },
      
      /**
       * Holds and changes truthful States of the Trampoline Views
       * A simple state machine, probably shouldn't even call it that
       * @memberof Trampoline
       * @name Trampoline.state
       */
      state = (function(){
        var change =  function(stateObj){
              var stateObj = (stateObj) ? stateObj : {},
                  toState  = stateObj.to || Trampoline.state.last.name || 'init',
                  //TODO: Is a from state necessary? This is UI driven; is 'from' always the current state before change?
                  fromState = (stateObj.from) ?  stateObj.from : Trampoline.state.current.name;
              
              //If it can change state and has toggleView, do that
              if ( toState != Trampoline.state.current.name ){
                //Toggle current state
                if ( Trampoline.hasOwnProperty(Trampoline.state.current.name) && Trampoline[Trampoline.state.current.name].hasOwnProperty('toggleView') ){
                 Trampoline[Trampoline.state.current.name].toggleView();
                }
                //Toggle new state
                if ( Trampoline.hasOwnProperty(toState) && Trampoline[toState].hasOwnProperty('toggleView') ){
                  Trampoline[toState].toggleView();
                }                
                //Set current state, add previous state to state.last array
                Trampoline.state.last.push(Trampoline.state.current);
                if (!stateObj.hasOwnProperty('name')){
                  stateObj.name = toState;
                }
                Trampoline.state.current = stateObj;
                Trampoline.events.dispatchEvent(TRAMPOLINE_STATE_CHANGE);
              }else{ 
                _log('Cannot change to current state', toState);
              }
            },
            //TODO: Do I need a back state? can i just call .back if the stateObj is empty?
            back = function(){
              var toState =  Trampoline.state.last[Trampoline.state.last.length - 1].name,
                  popupsActive = _trampolineContainer.querySelector('section.popups').classList.contains('active'),
                  gateActive = _trampolineContainer.querySelector('section.gate').classList.contains('active');
              
              //If a popup / parent gate is open, _close_ it (these are not handled by state change, but need to be attached to back button).
              if (!!gateActive){
                Trampoline.gate.close();
              }else if (!!popupsActive){
                Trampoline.popups.close();
              }else if ( (toState != Trampoline.state.current.name) && (toState != 'uninit') && (toState != 'init') && (toState != 'splash')){
                //Toggle Last state
                if ( Trampoline.hasOwnProperty( toState ) && Trampoline[toState].hasOwnProperty('toggleView') ){
                  Trampoline[toState].toggleView();
                }
                //Toggle Current state. 
                if ( Trampoline.hasOwnProperty(Trampoline.state.current.name) && Trampoline[Trampoline.state.current.name].hasOwnProperty('toggleView') ){
                 Trampoline[Trampoline.state.current.name].toggleView();
                }
                //Remove last state from array
                Trampoline.state.current = Trampoline.state.last.pop();
                Trampoline.events.dispatchEvent(TRAMPOLINE_STATE_BACK);
                
              }else{
                _log('Cannot toggleView of toState', toState);
              }
            },
            current = {name: 'uninit'},
            last  = [];
          
        return {
          change : change,
          last : last,
          current : current,
          back : back
        };
      })(),
      
      /**
       * Configure Trampoline based on Parameters
       * @private
       * @memberof Trampoline
       * @param {Object} configObj - JSON configuration parameters
       */
      _config = function(configObj){
        if (typeof configObj === 'object') {      
          for (var obj in configObj){
            switch ( obj.toLowerCase() ){
              case 'app_name':
                if (typeof configObj[obj] === 'string'){
                  Trampoline.PBSKidsAppName = configObj[obj];
                }else{
                  _log('Could not set App Name', configObj[obj]);
                }
                break;
              /** Google Analytics Config (Object)
                * Recieves a UA String.
                */
              case 'guid':
                if (typeof configObj[obj] === 'string'){
                  Trampoline.GUID = configObj[obj];
                }else{
                  _log('Error in GUID config', configObj[obj]);
                }
                break;
              case 'ua':
                if (typeof configObj[obj] === 'string'){
                  _configUA = configObj[obj];
                  Trampoline.analytics.UA = configObj[obj];
                }else{
                  _log('Could not set Analytics UA', configObj[obj]);
                }
                break;              
              /** AddOn Config (Object)
                * Recieves a Button Object.
                * Obj parameters: Text, Icon, Callback
                */
              case 'addon':
                if (typeof configObj[obj] === 'object'){
                  _buttonArr.push(configObj[obj]);
                }else{
                  _log('Error in addon object config', configObj[obj]);
                }
                break;
              /** SuperVision Config (object)
                * Defaults to `null`, developer must enable
                */
              case 'supervision':
                if (typeof configObj[obj] === 'object'){
                  _superVisionConfig = configObj[obj];
                  
                  //TODO: This is basically markup and should designed as markup and not built in js?
                  _hasNotifiers = true;
                  _notifierArr.push({name : "supervision", icon : 'trophy', heading: "Hey parents!", instructions : "See what your kid is learning!", arrow : 'top-right'});
                  
                }else{
                  _log('Supervision config must return object', configObj[obj]); 
                }
                break;
              /** Splash Screen Config (Boolean)
                * Defulats to `true`
                */
              case 'splash':
                if (typeof configObj[obj] === 'boolean'){
                  _hasSplash = configObj[obj];
                }else if (configObj[obj].hasOwnProperty('hide') && typeof configObj[obj].hide == 'boolean'){
                  _hasSplash = 'auto-hide-' + configObj[obj].hide;
                  Trampoline.splash.config.hide = configObj[obj].hide;
                }else{
                  _log('Error in splash config', configObj[obj]); 
                }
                break;
              /** Parents Config
               *  Defaults to `false` 
               */
              case 'parents':
                if(typeof configObj[obj] === 'boolean'){
                  _hasParents = configObj[obj];
                }else
                if(typeof configObj[obj] === 'object'){
				  _hasParents = configObj[obj];
                }else{
                  _log('Parents config must return Boolean or Object', configObj[obj]);
                }
                break;
					
			case 'settings':
                if(typeof configObj[obj] === 'boolean' || typeof configObj[obj] === 'object'){
                  _hasSettings = configObj[obj];
                }else{
                  _log('Settings config must return Boolean or Object', configObj[obj]);
                }
                break;

              case 'popups':
                if(typeof configObj[obj] === 'boolean' || typeof configObj[obj] === 'object'){
                  _hasPopups = configObj[obj];
                }else{
                  _log('Popups config must return Boolean or Object', configObj[obj]);
                }
                break;

            /** onReady Callback
               *  Defaults to null. Can also listen for TRAMPOLINE_READY event on Trampoline.container 
               */
              case 'onready':
                if (typeof configObj[obj] === 'function'){
                  _devOnReady = configObj[obj];
                }else{
                  _log('onReady in config is not a function', configObj[obj]);
                }
                break;              
              
              case 'moreapps':
                if (typeof configObj[obj] === 'boolean'){
                  _hasMoreApps = configObj[obj];
                }else{
                  _log('MoreApps config must return Boolean', configObj[obj]); 
                }
                break;
                
              case 'nav':
                if (typeof configObj[obj] === 'boolean'){
                  _hasNav = configObj[obj];
                }else{
                  _log('nav config must return Boolean', configObj[obj]); 
                }
                break;
              
              /** Settings Module Config
                * This is a relic from a tooltip design requiring a settings module.
                * Settings is non-configurable as it will always include localization.
                */
              case 'settingsmodule':
                _hasSettingsModule = true;
                for (var key in configObj[obj]) {
                  Trampoline.settings.addSetting(configObj[obj][key]);
                }
                break;
              
              /** Button Config
                * Additional buttons can be added through config. 
                * Receives an object array, and adds buttons to the buttonObj and inject to the left of all other buttons.
                * [{text: 'button1', icon:'false', callback:function}, {text: 'button2', icon:'true', callback:function}]
                */
              case 'buttons':
                for (var key in configObj[obj]) {
                  _log('add buttons from config: '+configObj[obj][key]);
                  _buttonArr.push(configObj[obj][key]);
                }
                break;
              /** Set file path of Trampoline dist folder (must end in '/')
                */
              case 'path':
                // store path for resource loading.
                _path = configObj[obj];
                _markupURL = _path + 'trampoline.html';
                break;
                
              /** Set file path of resources, where img lives, dist folder (must end in '/')
              */
              case 'resources':
                _resourcePath = configObj[obj];
                break;
              
              case 'localize':
                // store path for resource loading.
                if(typeof configObj[obj] === 'object'){
                  _localize = configObj[obj];
                }else{
                  _log('Localize config must be an object with either auto or zipCode properties.', configObj[obj]);
                }
                break;
                
              case 'debug':
                if (typeof configObj[obj] === 'boolean'){
                  Trampoline._debug = configObj[obj];
                  if (Trampoline._debug){
                    _log = function(message,args,type,force){if( typeof message == "string" || !!args ){ message = _APP_NAME + " ver. " + _APP_VERSION + " | " + message;} else { args = message; message = _APP_NAME + " ver. " + _APP_VERSION + " | "; } if(typeof console!=="undefined"){if(type=="error")if(console.error){console.error(message,args);return;}else if(window.Error)throw new Error(message);if(type=="info"&&console.info){console.info(message,args);return;}if(type=="warn"&&console.warn){console.warn(message,args);return;}if(console.log)console.log(message,args);}else if(window.debug)window.debug.log.apply(message,args);};
                  }
                }else if (configObj[obj] === 'build-data'){
                  _showBuildData = true;
                  Trampoline._debug = true;
                  if (Trampoline._debug){
                    _log = function(message,args,type,force){if( typeof message == "string" || !!args ){ message = _APP_NAME + " ver. " + _APP_VERSION + " | " + message;} else { args = message; message = _APP_NAME + " ver. " + _APP_VERSION + " | "; } if(typeof console!=="undefined"){if(type=="error")if(console.error){console.error(message,args);return;}else if(window.Error)throw new Error(message);if(type=="info"&&console.info){console.info(message,args);return;}if(type=="warn"&&console.warn){console.warn(message,args);return;}if(console.log)console.log(message,args);}else if(window.debug)window.debug.log.apply(message,args);};
                  }
                }else{
                  _log('Error in debug config', configObj[obj]); 
                }
                _debug = true;
                break;
                
              case 'offline':
                if (typeof configObj[obj] === 'boolean'){
                  if (configObj[obj]){
                    _log('forcing offline mode');
                    Trampoline.util.isConnected = !configObj[obj]; 
                  }
                }else{
                  _log('Error in offline config', configObj[obj]); 
                }
                break;
                
              default :
                _log('Error in Config Settings', obj+' not found' , false);
            }
            
          }
        }else{
          _log('config settings not an object', configObj[obj]);
        }
      },
      
      /**
       * onReady builds main page after configuration and splash screen animation end. Calls configuration onReady callback when complete.
       * @name Trampoline.onReady
       * @memberof Trampoline
       * @todo TODO: Decouple onReady from splash screen end. Main page should build while splash screen is up.
       */      
      onReady = function(){
        _log('Trampoline.onReady called');

        //BackButton on Android
        if (Trampoline.util.isAndroid){
          document.addEventListener("backbutton", Trampoline.state.back, false);
        }
        
        if ( !!_hasNav && !_trampolineContainer.querySelector('.main nav') ){
          var trampolineMain = document.createElement('div'),
              trampolineNav = document.createElement('nav'),
              trampolineNavUL = document.createElement('ul');
          
          trampolineMain.classList.add('main');
          
          trampolineNav.appendChild(trampolineNavUL);
          trampolineMain.appendChild(trampolineNav);
          _trampolineContainer.appendChild(trampolineMain);

          _buildNav(_buttonArr);
          
          _buildNotifiers(_notifierArr);
        }
        
        var moduleInitQueue = [
          {
            prop: 'localize',
            options: {}
          },
          {
            prop: 'popups',
            options: _hasPopups, // this will only accept custom popups as we are always created
            alwaysCall: true
          },
          {
            prop: 'moreApps',
            options: { silent: !_hasNav, remove: !_hasMoreApps },
            needsInternet: true
          },
          {
            prop: 'supervision',
            options: _superVisionConfig
          },
          {
            prop: 'settings',
            options: _hasSettings
          },
          {
            prop: 'parents',
            options: _hasParents
          },
          {
            prop: 'gate',
            options: {}
          }
        ];
        
        var initLoader = buildModuleLoader(moduleInitQueue, 'init');

        initLoader.load( function( initLoader ){

          // call final ready on each module
          for( i = 0; i<moduleInitQueue.length; i++ ){
            mod = moduleInitQueue[i];
            if( mod.didInit && Trampoline[ mod.prop ].startup ){
              Trampoline[ mod.prop ].startup( initLoader.resources );
            }
          }
          
          //TODO: The modules should return an error if init failed. 
          _modulesDidInit = true;

          //Lastly, call config onReady.
          if (_devOnReady){ _devOnReady(); }
          Trampoline.events.dispatchEvent(TRAMPOLINE_READY);

        }.bind(this) );

      },

      /**
       * This is a handler for the public refresh call. Forces to reset online status and calls _onRefresh if online. 
       */
      refresh = function (){
		if (Trampoline.util.isConnected){
          _onRefresh();
		}else if (navigator.onLine){
          _onConnectionResume();
          _onRefresh();
        }else{
          document.addEventListener('online', _onRefresh);
	  	}
	  },
      
      /**
       * Runs the refresh method on modules that require a network connection, in order.
       */
      _onRefresh = function(){
        if (_didInit){
          var moduleRefreshQueue,
              refreshLoader;

          document.removeEventListener('online', _onRefresh);

          moduleRefreshQueue = [
            {
              prop: 'analytics',
              options: _configUA
            },  
            {
              prop: 'moreApps',
              options: { silent: !_hasNav },
              needsInternet: true
            },
            {
              prop: 'settings',
              options: _hasSettings,
            },
            {
              prop: 'parents',
              options: _hasParents
            }
          ];

          refreshLoader = buildModuleLoader(moduleRefreshQueue, 'refresh');

          refreshLoader.load( function( refreshLoader ){

//            for( i = 0; i<moduleRefreshQueue.length; i++ ){
//              mod = moduleRefreshQueue[i];
//              if( mod.didRefresh && mod.restart && Trampoline[ mod.prop ].startup ){
//                Trampoline[ mod.prop ].startup( refreshLoader.resources );
//              }
//            }
            //TODO: Make sure that all modules completed refresh
            Trampoline.events.dispatchEvent(TRAMPOLINE_REFRESH);
          }.bind(this) );
        }
      },      
      
      /**
       * Called when network connection is resumed, updates Modules accordingly
       */
      _onConnectionResume = function (){
        Trampoline.util.isConnected = window.navigator.onLine;
        if (_modulesDidInit){
          _log('on connection resume');
          var moduleConnectionResumeQueue,
              connectionResumeLoader;

          moduleConnectionResumeQueue = [
            {
              prop: 'supervision',
              options: _superVisionConfig
            },
            {
              prop: 'parents',
              options: _hasParents
            },            
            {
              prop: 'settings',
              options: _hasSettings
            }
          ];

          connectionResumeLoader = buildModuleLoader(moduleConnectionResumeQueue, 'connectionResume');

          connectionResumeLoader.load( function( connectionResumeLoader ){
            // Queue is done
            Trampoline.events.dispatchEvent(TRAMPOLINE_ONLINE);
          }.bind(this) );
        }
      },
      
      /**
       * Called when connection is lost, and updates Modules accordingly
       */
      _onConnectionLost = function (){
        Trampoline.util.isConnected = window.navigator.onLine;
        if (_modulesDidInit){
          _log('on connection lost');
          var moduleConnectionLostQueue,
              connectionLostLoader;

          moduleConnectionLostQueue = [
            {
              prop: 'supervision',
              options: _superVisionConfig
            },
            {
              prop: 'parents',
              options: {}
            },
            {
              prop: 'settings',
              options: _hasSettings
            }
          ];

          connectionLostLoader = buildModuleLoader(moduleConnectionLostQueue, 'connectionLost');

          connectionLostLoader.load( function( connectionLostLoader ){
            Trampoline.events.dispatchEvent(TRAMPOLINE_OFFLINE);
          }.bind(this) );
        }
      },
      
      /**
       * Builds a module loader from queue obj to run a method (i.e. init, resume, connectionLost) on all modules depending on connectivity / config.
       * @param   {Object} moduleQueue The queue of modules, in order
       * @param   {String} method      The method we want to run. When run: puts a didMethod on the module
       * @returns {Object} The loader, see onReady for implementation
       */
      buildModuleLoader = function (moduleQueue, method){
        _log('building a module loader for: '+ method, moduleQueue);
        var i,mod,method,
            //did + camelCase the method.
            didMethod = 'did' + (method.charAt(0).toUpperCase() + method.slice(1));
        
        // Checks to see if module has the method, and calls it with options
        // unless there is no network connection <- but still will if alwaysCall = true
        for( i = 0; i<moduleQueue.length; i++ ){
          mod = moduleQueue[i];
          hasMethod = true;
          if( mod.needsInternet && !Trampoline.util.isConnected ){
            hasMethod = false;
          }
          if( ( mod.options && hasMethod ) || mod.alwaysCall ){
            // execute method on the module
            Trampoline[ mod.prop ][method] ? Trampoline[ mod.prop ][method](mod.options) : null;
            mod[didMethod] = true;
          }else{
            // remove the module
            Trampoline[ mod.prop ].remove ? Trampoline[ mod.prop ].remove() : null;
            mod[didMethod] = false;
          }
        }

        // Fetch any resources the module needs by querying the module for a list.
        var loader = new Trampoline.classes.ResourceLoader();
        // TODO : Need to check for multiple resource requests.
        // Could handle this here - instead of checking for duplicates in templating.
        for( i = 0; i<moduleQueue.length; i++ ){
          mod = moduleQueue[i];
          if( mod[didMethod] && Trampoline[ mod.prop ].listResources ){
            loader.add( Trampoline[ mod.prop ].listResources() );
          }
        }
        return loader;
      },
  
      /**
       * Injects trampoline.html into the DOM.
       * @private
       * @memberof Trampoline
       * @param {function} callback Callback, returns nothing.
       */
      _buildMarkup = function (callback){
        _log('Trying to inject trempoline...');
        var _trampolineContainer = document.getElementById('trampoline'),
            _onSuccess = function (trampolineMarkup){
              _trampolineContainer.insertAdjacentHTML('beforeend', trampolineMarkup);
              _log('...Trampoline injected!');
              callback();
            },
            _onFail = function( err ){
              _log('...Trampoline failed to inject! :(' , err);
            };

        Trampoline.util.fetch(_markupURL, _onSuccess,_onFail);
      },
      
      /**
       * Builds Nav from Button Array in reverse order.
       * @private
       * @memberof Trampoline
       * @param {Array} _buttonArr array of Button Objects; see addButton
       */
      _buildNav = function (_buttonArr){
        // Build Defaults
        if (_hasParents){
          _buttonArr.push({
            text: 'Grownups',
            icon: 'parents',
            state: 'parents'
          });
        }
        
        //Settings Not Always True (that amazon kiddos thing or whatever)
	  	if (_hasSettings){
			_buttonArr.push({
			  text: 'Settings',
			  icon: 'settings',
			  state: 'settings'
			});
        }
                
        if (_superVisionConfig !== null) {
          _buttonArr.push({
            text: false,
            icon: 'supervision',
            state: 'supervision'
          });
        }
        
        for (var i=_buttonArr.length-1; i>=0; i--){
          _addButton(_buttonArr[i]);
        }

      },
      
      /**
       * Adds a buttons to the nav. 
       * @private
       * @memberof Trampoline
       * @param {Object} buttonObj Named object with text, icon(optional) and callback(required)
       */
      _addButton = function(buttonObj){
        var _buttonContainer = document.querySelector('.main nav ul'),
            _newButton = document.createElement('li'),
            _newContent = '';
        
        //Define Icon
        if (buttonObj.icon){
          if (buttonObj.text === false){
            _newButton.classList.add('button', 'no-text', buttonObj.icon.toLowerCase());
          }else{
            _newButton.classList.add(buttonObj.icon.toLowerCase(), 'button');
          }
          _newContent += '<div class="icon"></div>';
        }else{
          _newButton.classList.add('add-on', 'button');
          if (!buttonObj.hasOwnProperty('icon')){
            _newContent += '<div class="icon"></div>';
          }
        }
                
        //Set Text
        if (buttonObj.text){
          _newContent += '<h2>'+buttonObj.text+'</h2>';
        }
        
        _newButton.innerHTML = _newContent;
        
        //Add Event Listener
        _log('Adding button:', buttonObj);
        _newButton.addEventListener(Trampoline.util.tapEvent, function(e){
          e.preventDefault();
          Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit');
          if (buttonObj.hasOwnProperty('callback') ){
            buttonObj.callback();
          }else if ( buttonObj.hasOwnProperty('state') && (Trampoline.state.current.name == 'main') ){
            Trampoline.state.change({to: buttonObj.state});
          }else{
            _log(buttonObj[0]+' : neither Callback nor State defined');
          }
        });
        
        _buttonContainer.insertBefore(_newButton, _buttonContainer.firstChild);
        
      },
      
      /**
       * Builds notifiers from array
       * @param {Array} _notifierArr All the notifiers
       */
      _buildNotifiers = function(_notifierArr){
        if (_hasNotifiers){
          for (var i=0; i<_notifierArr.length; i++){
            _addNotifier(_notifierArr[i]);
          }
        }
      },
      
      /**
       * Creates a notifier and inserts it into #Trampoline.main
       * @param {Object} notifierObj The notifier, with properties. (icon, heading, instruction, arrow)
       */
      _addNotifier = function(notifierObj){
        var _newContent = '';

        if (notifierObj.hasOwnProperty('icon')){
          _newContent += '<div class="icon"><img src="' + _resourcePath + 'img/icons/badges/badge-' + notifierObj.icon + '.svg" /></div>'
        }
        if (notifierObj.hasOwnProperty('heading')){
          _newContent += '<div class="heading"><h2>' + notifierObj.heading + '</h2></div>'
        }
        
        if (notifierObj.hasOwnProperty('instructions')){
          _newContent += '<div class="instructions"><h3>' + notifierObj.instructions + '</h3></div>'
        }
        if (notifierObj.hasOwnProperty('arrow')){
          _newContent += '<div class="arrow"><img src="' + _resourcePath + 'img/icons/arrow-' + notifierObj.arrow + '.svg" /></div>'
        }
        
        // Create the notifier and into .main if there is content (but before the nav).
        if (!!_newContent){
          var _trampolineMain = _trampolineContainer.querySelector(".main"),
              _newNotifier = document.createElement('div');
          
          _newNotifier.classList.add('notifier');
          _newNotifier.setAttribute('id', notifierObj.name+"-notifier")
          _newNotifier.innerHTML = '<div class="notifier-container">' + _newContent + '</div>';
          _trampolineMain.insertBefore(_newNotifier, _trampolineMain.firstChild);
        }
      },      
      
      /**
       * Injects contents of .svg into DOM Node with xhr request, so that it can be manipulated with CSS and JS.
       * @private
       * @memberof Trampoline
       * @param {object}   node   DOM Node
       * @param {string}   url    SVG Location
       * @param {function} onFail Callback in the event that the SVG is unreachable
       *                          
       */
      _injectSVG = function (node, url, onFail){
        var svgContent = Trampoline.util.fetch(url, _addSVG),
            _addSVG = function(svgData){
              node.innerHTML(svgData);
            };
      };
  
      // Public Façade
      var facade = {
        init            : init,
        state           : state,
        onReady         : onReady,
        refresh         : refresh,
        /**
         * DOM Node containing #Trampoline
         * @memberof Trampoline
         */
        container       : _trampolineContainer,

        /**
         * Throwing in a façade which attaches events to Trampoline.container
         * (So we can change things later without breaking the API.)
         */
        events          : _trampolineContainer,
        _log            : _log,
        _debug          : _debug,
        /**
         * Name of Property App
         * @memberof Trampoline 
         */
        PBSKidsAppName  : PBSKidsAppName,
        GUID            : GUID,
        _resourcePath   : _resourcePath
      };

      // define these properties as getters as value
      // can change from config at init.
      Object.defineProperties( facade, {
        _path: {
          get: function(){
            return _path;
          }
        },
        _localize: {
          get: function(){
            return _localize;
          }
        },
        _resourcePath: {
          get: function(){
            return _resourcePath;
          }
        }
        
      });

      return facade;
    
}
());

if ((typeof module != 'undefined') && !!module.exports) {
  module.exports = Trampoline;
}
(
  /**
   * Analytics Module for Trampoline
   * @memberof Trampoline
   * @namespace analytics
   */
  function(Trampoline){
  Trampoline.analytics = (function (){
    var _log      = Trampoline._log,
        _didInit  = false,
        _source   = null,
        _queue    = [],
        UA,
    
        /**
         * Initializes Analytics if connected. Checks to see if ga obj exists, and initates new tracker for trampoline.
         * @memberof Trampoline.analytics
         * @param {String} configUA App Specific Google Analytics UA Code
         */
        init = function (configUA){
          if (Trampoline.util.isConnected && !!Trampoline.analytics.UA){
            if (!!configUA){
              Trampoline.analytics.UA = configUA;
            }
            _log('Initializing Analytics...', Trampoline.analytics.UA);
            if (window.ga){
              _source = 'window';
            }
            if(typeof ga == 'undefined'){
              (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
              (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
              m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
              })(window,document,'script','https://www.google-analytics.com/analytics.js','ga');
              _source = 'script';
            }
            if (_source == 'window'){
              window.ga.startTrackerWithId(Trampoline.analytics.UA)
              _log('Created GA tracker instance through universal plugin: window.ga');
            }else{
              ga('create', Trampoline.analytics.UA, 'auto'); 
              _log('Created GA tracker instance through js script');
            }
            _didInit = true;
          }else{
            _log('Analytics not available', 'UA Code: ' + Trampoline.analytics.UA);
          }
        },
        refresh = function (options) {
          if (!_didInit){
            init(options);
          }else{
            _log('analytics.init already completed');
          }
        },
        /**
         * Send a Trampoline event to Google Analytics
         * @memberof Trampoline.analytics
         * @param {String} action   Name of event trigger
         * @param {String} label    Event (open, closed, pass) or Trigger Detail (MoreApps App Name + Badge)
         * @param {String} category Trampoline section *(not used in root level events)
         */
        event = function(action, label, category){
          if (_didInit && Trampoline.util.isConnected){

            var _propertyAppTitle = Trampoline.PBSKidsAppName.replace(/ /g,"_"),
                _appCategory = 'Trampoline_'+_propertyAppTitle;
                
                if (typeof category != 'undefined'){
                  _appCategory += '_'+category;
                }
            
            if (_source === 'script'){
               ga('send', {
                'hitType': 'event',          // Required.
                'eventCategory': _appCategory,   // Required.
                'eventAction': action,      // Required.
                'eventLabel': label,
                'eventValue': 4
              }); 
            }else if (_source === 'window'){
              window.ga.trackEvent(_appCategory, action, label);
            }
            
            _log('send, event, '+_appCategory+', '+action+', '+label);            
          }else{
            _queue.push({type: 'event', category: category, action: action, label: label});
          }
        },
		/**
		 * Set Custom Dimension 
		 * @param {number} index     The dimension index/key (1 for stations, leaving configurable for the future)
		 * @param {string} dimension The name value of the dimension 
		 */
		dimension = function (index, dimension){
          if (_didInit && Trampoline.util.isConnected){
            if (_source == 'script'){
              ga('set', 'dimension'+index, dimension.toString() );
              
            }else if (_source == 'window'){
              window.ga.addCustomDimension(index, dimension);
            }
            _log('Set ga dimension, '+ index + " , "+dimension);
          } else {
            _queue.push({type: 'dimension', index: index, dimension: dimension});
          }
		},
        //TODO: Clean this up, we don't actually need it
        _runQueue = function(){
          _log('Running analytics queue');
          for (var i=0; i<_queue.length; i++){
            item = _queue.pop;
            if (item.type = 'event'){
              Trampoline.analytics.event(item.category, item.event, item.label);
            }else if (item.type = 'dimension'){
              Trampoline.analytics.dimension(item.index, item.dimension);
            }
          }
        };
      
    return {
      init      : init,
      refresh   : refresh,
      event     : event,
      dimension : dimension,
      UA        : UA
    };
  })();
            
  return Trampoline;
  
}(Trampoline || {}));
(
  /**
   * Station Localization Module for Trampoline
   * (*this has nothing to do with language localization)
   * @memberof Trampoline
   * @namespace localize
   * 
   * @todo TODO: Remove all these callbacks from localization?
   */
  function(Trampoline){
  Trampoline.localize = (function (){
    var didLocalize = false,
        _log        = Trampoline._log,
        _stationServicesEndpoint = 'http://api.pbskids.org/trampoline/stations/',
        //_stationServicesEndpoint = 'https://station.services.pbs.org/api/public/v1/stations/',
        stationList = {},
        localStation,
        stationListCount,
        LOCALIZED = new Event('LOCALIZED'),
        STATION_LIST_POPULATED = new Event('STATION_LIST_POPULATED'),
        STATION_IMAGES_POPULATED = new Event('STATION_IMAGES_POPULATED');
    
        /**
         * Hit the PBS localization service AUTO endpoint --> then Sets Localization (
         * @memberof Trampoline.localize
         * @param {Function} callback Callback, returns stationList if successful
         * @param {Function} failCallback In the event of failure...
         */
    
        init = function(){
          _log('init localization');
          //Setup one-time event listeners
          Trampoline.events.addEventListener('STATION_IMAGES_POPULATED', _countStationManagerImageCollection);
        },
        
        auto = function(callback, failCallback){
          var _localStation;
          
          //check for persistent localization data
          try{
            _localStation = Trampoline.util.getLocalStorage('trampolineLocalStation');
            //Try to parse it now, since this can throw errors later...
            JSON.parse(_localStation);
          }catch(e){
            _log(e);
          }
          
//          if (!localStation){
//            setLocalizationFromLocalStorage();
//          }             

          var _onSuccess = function(data){
                if (!callback){ callback = null };
                _stationListFromAutoLocalizeData(data);
              },
              _onFail = function(data){
                _log('Failed to autolocalize' );
                if (typeof failCallback === 'function'){
                  _log('Resorting to fail callback', data.error);
                  failCallback();
                }
              };
          Trampoline.util.isConnected ? Trampoline.util.fetch('http://localization.services.pbs.org/localize/auto/', _onSuccess, _onFail) : _onFail();
        },
        
        /**
         * Hit the PBS localization service ZIPCODE endpoint -> then Sets Localization
         * @memberof Trampoline.localize
         * @param {String}   zip          The Zipcode
         * @param {Function} callback     Callback that gets passed through several functions, ends with setting the stationList obj if succesful
         * @param {Function} failCallback In the event of failure...
         */
        zipcode = function(zip, callback, failCallback){
          var _onSuccess = function(data){
                if (!callback){ callback = null };
                _stationListFromZipcodeLocalizeData(data, callback);
              },
              _onFail = function(data){
              _log('Failed to localize from zipcode' );
                if (typeof failCallback === 'function'){
                  _log('Resorting to fail callback', data.error);
                  failCallback();
                }
              };
          Trampoline.util.isConnected ? Trampoline.util.fetch('http://localization.services.pbs.org/localize/zipcode/'+zip+'/', _onSuccess, _onFail) : _onFail();
        },
        /**
         * Takes Auto Localization Data and hits Station Manager Endpoint to return more Station specific data.
         * @private
         * @memberof Trampoline.localize
         * @param {String}   localizedStr AutoLocalization JSON
         * @param {function} callback     Callback passed in by autolocalization
         */
        _stationListFromAutoLocalizeData = function(localizedStr, callback){
          Trampoline._log('station callsigns retrieved by auto localization', callback);
          _setAutoLocalizationCallback = function(data){
            _handleStationManagerData(data, callback);
          };
          var localizedJSON = JSON.parse([localizedStr]),
              stationsObj   = Object.keys(localizedJSON.stations);
          
          if (stationsObj.length > 0){
            Trampoline.localize.didLocalize = 'auto';
            //Clear the current stationlist
            Trampoline.localize.stationList = {};
            stationListCount = 0;
            for (i=0; i<stationsObj.length; i++){
              var stationCallsign = stationsObj[i].toUpperCase();
              
              Trampoline.localize.stationList[stationCallsign] = {};
              
              //Get station image from station manager data
              _getStationManagerDataFromCallSign(stationCallsign, _setAutoLocalizationCallback);
              
            }
          }
        },
        
        //TODO: Merge this method with previous and require a parameter to differntiate between Auto and Zipcode data
        _stationListFromZipcodeLocalizeData = function(localizedStr, callback){
          _log('station callsigns retrieved from zipcode');

          _setZipcodeLocalizationCallback = function (data){    
            _handleStationManagerData(data, callback);
          };
          
          var localizedJSON = JSON.parse([localizedStr]),
              stationsObj   = localizedJSON.stations;
                    
          if (stationsObj.length > 0){
            Trampoline.localize.didLocalize = 'zipcode';

            //Clear the current stationlist
            Trampoline.localize.stationList = {};
            stationListCount = 0;
            
            //Go through the stationObject and add add each station to the stationList obj.
            for (i=0; i<stationsObj.length; i++){
              var stationCallsign = stationsObj[i].callsign.toUpperCase();
              Trampoline.localize.stationList[stationCallsign] = {};
              //Asynchronousely fetch detailed station data from the Station Manager
              _getStationManagerDataFromCallSign(stationCallsign, _setZipcodeLocalizationCallback);
            }
          }
        },
        /**
         * Retrieves station details from station manager 
         * @private
         * @memberof localize
         * @param {String}   callSign  Accepts a list of call signs
         * @param {function} onSuccess Success Callback
         * @param {function} onFail    Fail Callback
         */
        _getStationManagerDataFromCallSign = function(callSign, onSuccess, onFail){
          function handleSucces(stationManagerStr){
            var stationData = JSON.parse(stationManagerStr);

            if (stationData.data.length <= 0 ){
              _log('Station Manager returned empty', callSign);
              delete Trampoline.localize.stationList[callSign];
            }else{
              onSuccess(stationManagerStr); 
            }
          };
          
          Trampoline.util.fetch(_stationServicesEndpoint, handleSucces, onFail, {data : {call_sign : callSign } });
        },
        
        _countStationManagerImageCollection = function(){
          stationListCount++;
          if (stationListCount >= Object.keys(Trampoline.localize.stationList).length){
            Trampoline.events.dispatchEvent(STATION_LIST_POPULATED);
          }
        },
        /**
         * Parses station manager data and keys it to the stationList
         * @private
         * @memberof Trampoline.localize
         * @param {String}   stationManagerStr Data returned from Station Manager API
         * @param {function} callback          Callback, returns Station Image
         */
        _handleStationManagerData = function(stationManagerStr, callback){
          var stationJSON = JSON.parse(stationManagerStr),
              stationData = stationJSON.data.length > 0 ? stationJSON.data[0].attributes : null;
                    
          function _handleStationManagerImages(imageData){
            Trampoline.localize.stationList[stationData.call_sign].images = imageData;
            Trampoline.events.dispatchEvent(STATION_IMAGES_POPULATED);
          }
          
          if( !!stationData ){
            if (!!Trampoline.localize.stationList[stationData.call_sign]){
              Trampoline.localize.stationList[stationData.call_sign].donate_url = stationData.donate_url;
              Trampoline.localize.stationList[stationData.call_sign].tvss_url = stationData.tvss_url;
              Trampoline.localize.stationList[stationData.call_sign].full_common_name = stationData.full_common_name;
              Trampoline.localize.stationList[stationData.call_sign].callsign = stationData.call_sign;
              _parseStationManagerImages(stationData, _handleStationManagerImages);

              //Fire localized event if this is the first station in the list
              if ( stationData.call_sign == Object.keys(Trampoline.localize.stationList)[0] ){
                var _autoLocalStation = Trampoline.localize.stationList[stationData.call_sign];
                Trampoline.util.setLocalStorage('trampolineAutoLocalStation', JSON.stringify(_autoLocalStation) );
                Trampoline.events.dispatchEvent(LOCALIZED);
              }
            }
          }else{
            _log('No station data :', stationJSON );
            //TODO: remove this from the stationlist.
            //StationList.indexOf[]
          }

        },
        
        /**
         * Parse Station Specific Array of Image data from Station Manager Response
         * @private
         * @memberof Trampoline.localize
         * @param   {String} stationManagerStr Station Manager Response Data
         * @returns {Object} parsed and keyed Station Images 
         */
        _parseStationManagerImages = function(stationData, callback){
          var stationImagesObj = stationData.images,
              stationImages = {};
          
          for (var i=0; i<stationImagesObj.length; i++){
            stationImages[stationImagesObj[i].profile] = stationImagesObj[i].url;
          }
          callback(stationImages);
        },
        
        /**
         * Fetches the station image from the callsign specific data stored in the stationList obj.
         * Throws event on success/fail for error handling.
         * @param   {Object}   callsgn  Station Callsign
         * @param   {string}   logoType Which logo (color-logo, black-logo, white-logo)
         * @param   {function} callback A Callback
         * @returns {obj}      Loaded image object 
         * @event GET_STATION_IMAGE# {status: true/false, data: stationListItem}
         */
        getStationImage = function(stationListItem, logoType, callback, resizeWidth, resizeHeight){
          var stationImageURL = stationListItem.images[logoType],
              GET_STATION_IMAGE;
          resizeWidth = resizeWidth || null;
          resizeHeight = resizeHeight || null;
                    
          if (stationImageURL){
            if (resizeWidth || resizeHeight){
              _log('resizing station image');
              stationImageURL = Trampoline.util.resizeITS(stationImageURL, resizeWidth, resizeHeight);
            }
            Trampoline.localize.localStation = stationListItem;
            var stationLogo = new Image();
            stationLogo.setAttribute('data-callsign', stationListItem.callsign);
            stationLogo.src = stationImageURL;
            stationLogo.onload = function(){
              callback(stationLogo);
              GET_STATION_IMAGE = new CustomEvent('GET_STATION_IMAGE', {'detail': { 'status': true, 'data': stationListItem }});
              Trampoline.events.dispatchEvent(GET_STATION_IMAGE);
            }
          }else if (logoType != 'color-logo'){
            _log('Could not retrieve specified station image, trying color-logo');
            getStationImage(stationListItem, 'color-logo', callback);
          }else{
            _log('Could not retrieve specified station image');
            GET_STATION_IMAGE = new CustomEvent('GET_STATION_IMAGE', {'detail': { 'status': false, 'data': stationListItem }});
            Trampoline.events.dispatchEvent(GET_STATION_IMAGE);
          }

        },
        setLocalStation = function(callsign){
/* 
          //Moving this process From Settings.js
          //The original blocks:
          
          //updateStationList: //persist the local station
          Trampoline.localize.localStation = Trampoline.localize.stationList[e.target.dataset.callsign];
          Trampoline.localize.localStation.callsign = e.target.dataset.callsign;
          Trampoline.util.setLocalStorage('trampolineLocalStation', JSON.stringify(Trampoline.localize.localStation) );

          //from: updateLocalStation
          Trampoline.util.setLocalStorage('trampolineLocalStation', _localStation);

          Trampoline.localize.localStation = _localStation;
          Trampoline.events.dispatchEvent(LOCALIZED);
 */         
          //The New blocks:
          var _localStation = Trampoline.localize.stationList[callsign];          
          
          //Set everything to the new sation.
          Trampoline.localize.localStation = Trampoline.localize.stationList[callsign];
          Trampoline.localize.localStation.callsign = callsign;
          Trampoline.util.setLocalStorage('trampolineLocalStation', JSON.stringify(_localStation) );
          
          //Update the Analytics Dimension
          Trampoline.analytics.dimension(1, callsign);
          
          Trampoline.localize.localStation = _localStation;
          Trampoline.events.dispatchEvent(LOCALIZED);
          
          //TODO: Save the station logo to localstorage
        };
    
    return {
      init: init,
      auto: auto,
      zipcode: zipcode,
      didLocalize : didLocalize,
      /**
       * Array of station data based on localization
       * @memberof Trampoline.localize
       */
      stationList: stationList,
      localStation: localStation,
      getStationImage : getStationImage,
      setLocalStation : setLocalStation
    };
  })();
            
  return Trampoline;
  
}(Trampoline || {}));
(
  /**
   * Sponsorship Module for Trampoline
   * @memberof Trampoline
   * @namespace sponsorship
   */
  function(Trampoline){
  Trampoline.sponsor = (function (){
    var _trampolineContainer,
        _log = Trampoline._log,
        
        /**
         * Initialize Sponsorship module
         * @memberof Trampoline.sponsor
         */
        init  =  function(sponsorObj){
          _trampolineContainer = Trampoline.container || document.getElementById('trampoline');
          _log('init sponsorship');
        }
    
    return {
      init              : init,
    };
  })();
            
  return Trampoline;
  
}(Trampoline || {}));
(
/**
 * Templating Module for Trampoline
 * @memberof Trampoline
 * @namespace templating
 */
  function(Trampoline){
    Trampoline.templating = (function (){

      var _log = Trampoline._log,

        _convertToPath = function( template ){
          return Trampoline._path + 'templates/' + template + '.html';
        },

        _traverseElement = function( element, operator ){

          operator( element );

          for( var i = 0; i<element.children.length; i++ ){
            _traverseElement( element.children[i], operator );
          }

        },

        /**
         * Lists resources for template
         * @param   {Object} templateConfig   configObj
         * @param   {Object} defaultTemplates List of defaults
         * @returns {Object}  templates        All Templates to be rendered with configs
         */
        _listResourcesForConfig = function( templateConfig, defaultTemplates ){

          // Fetch the list of templates in the default templates.
          var templates = defaultTemplates.map( function( obj ){
            return obj.template;
          });

          // check supplied custom config and add additional and remove defaults if needed.
          if( typeof templateConfig === 'object' ){
            if( !templateConfig.includeDefaults ){
              templates.splice(0);
            }
            if( templateConfig.customPanels ){
              templates = templates.concat(
                templateConfig.customPanels.map( function( obj ){
                  return obj.template;
                }) );
            }
          }

          // convert this list of template to a list of file paths
          // of html template files to load.
          var hash = {};
          templates.forEach( function( str ){
            hash[ str ] = str;
          } );

          templates.splice(0);
          for( var key in hash ){
            templates.push( _convertToPath(key) );
          }

          return templates;

        },

        /**
         * Takes a set of template configs and default templates along with
         * loaded html template files ( resources ) and returns a list
         * of DOM elements customised and ready to insert into the DOM.
         * @returns {Object} DOM Fragments (Elements)
         */
        _buildTemplates = function( templateConfig, defaultTemplates, resources ){

          var templates = defaultTemplates.slice(0);
          if( typeof templateConfig === 'object' ){
            if( !templateConfig.includeDefaults ){
              templates.splice(0);
            }
            if( templateConfig.customPanels ){
              templates = templates.concat( templateConfig.customPanels );
            }
          }

          var elements = [];
          var template,resource;
          for( var i = 0; i<templates.length; i++ ){
            template = templates[i];

            // fetch html template resource.
            resource = resources[ _convertToPath( template.template ) ];

            if( !resource ){
              throw new Error( 'No resource found for template :', template.template );
            }

            // build template.
            elements.push( _buildTemplate( ('000' + i ).slice(-3), resource, template ) );
          }

          return elements;
        },


        /**
         * [[Description]]
         */
        _buildTemplate  = function( id, resource, config ){

          var fragment = document.createDocumentFragment();
          var parser = new DOMParser();
          var insert = parser.parseFromString( resource.xhr.responseText, 'text/html' );
          var root = insert.body.firstChild;

          if( !root ){
            console.error( 'Problem using template with id : ', config.template );
            throw new Error( 'Template error with template id :' + config.template );
            return;
          }
          // target elements ( those with ids )
          var elements = [];
          var hash = {};

          _traverseElement( root, function( ele ){
            if( ele.id ){
              elements.push( ele );
              hash[ ele.id ] = ele;
            }
          });

          // assign labels
          var key,ele;
          var labels = config.labels || {};
          for( key in labels ){
            ele = hash[key];
            if( ele ){
              ele.innerText = labels[key];
            }else{
              console.error( 'Problem assigning label to template element with id :', key, ' - ' + config.template );
            }
          }

          // add css classes
          var classes = config.classList || {};
          var cls;
          for( key in classes ){
            ele = hash[key];
            if( ele ){
              cls = classes[key];
              if( typeof cls === 'string' ){
                ele.classList.add( cls );
              }else
              if( cls instanceof Array ){
                ele.classList.add.apply( ele.classList, cls );
              }else{
                console.error( 'Problem assigning supplied css class values to element id :', key, ' - ' + config.template );
              }

            }else{
              console.error( 'Problem assigning css classes to template element with id :', key, ' - ' + config.template );
            }
          }

          // bind events
          var events = config.events || {};
          var args,eventType,targetId;
          for( key in events ){
            args = key.split( '_' );
            targetId = args[0];
            eventType = args[1];
            ele = hash[targetId];

            if( ele ){
              if( eventType === 'pointer' ){
                // TODO : 'pointer' event type
                // handles both desktop and mobile pointer support??
              }else{
                ele.addEventListener( eventType, events[key] );
              }
            }else{
              console.error( 'Problem assigning event to template element with id :', targetId, ' - ' + config.template );
            }
          }

          // finally, make ids unique before
          // inserting into main document.

          for( var i = 0; i<elements.length; i++ ){
            ele = elements[i];
            ele.id = ele.id + '_' + id;
          }

          fragment.appendChild( root );

          // assign the id to the root element.
          if( config.id ){
            root.id = config.id;
          }

          if( config.onBuildComplete ){
            var contentAreas = root.querySelectorAll( '*[data-content]' );
            config.onBuildComplete( root, contentAreas, '_' + id );
          }

          return fragment;

        };

      return {

        listResourcesForConfig: _listResourcesForConfig,
        buildTemplates: _buildTemplates,
        buildTemplate : _buildTemplate

      };

    })();

    return Trampoline;

  }(Trampoline || {}));
(
  /**
   * @memberof Trampoline
   * @namespace util
   */
  function(Trampoline){
  Trampoline.util = (
    function (){

    var showDataEndpoint = 'http://pbskids.org/api/trampoline/v1/shows',
        allShowsData,
        //showDataEndpoint = 'http://cms.pbskids.org/api/catalog/shows',
        //TODO: Update this Query to include sponsorship information
        //showDataQuery    = '?with=short_description&with=min_age&with=max_age&with=google_analytics_id&with=tier_group&include=curriculum',
        _log = Trampoline._log,
        buildData,
        currentTouches = [],
            
        BUILD_DATA_RETURNED = new Event('BUILD_DATA_RETURNED'),
        
        isConnected = (
        /**
         * Self Executing function which checks for internet availability (on load only).
         * @memberof Trampoline.util
         * @returns {Boolean}
         */
          function(checkURL){
            if (!!window.cordova && (typeof navigator.connection != 'undefined') && (typeof navigator.connection.type != 'undefined') ){
              var cordovaOnline = (navigator.connection.type == 'Connection.NONE') ? false : true;
              
              return cordovaOnline;
            }else if (!!window.navigator.onLine){
              return window.navigator.onLine;
//            }else{
//              _log('Signs point to offline, checking truthiness with xhr...')
//              var checkXhr = new XMLHttpRequest(),
//                  fetchURL = (typeof checkURL != 'undefined') ? checkURL : 'https://api.pbskids.org/trampoline/v1;
//
//              checkXhr.onload = function(){
//                // Running Connection Resume since this was an async method
//                _log('Connection resolved true');
//                Trampoline.connectionResume();
//                return true;
//              };
//
//              checkXhr.onerror = function(){
//                _log('Connection did not resolve, setting to offline mode');
//                return false;
//              };          
//
//              checkXhr.open('GET', fetchURL, true);
//              checkXhr.send();
            }
            
        })(),
        
        /**
         * XHR Data Fetch
         * @memberof Trampoline.util
         * @param {String}   url       Data URI
         * @param {function} onSuccess Success callback
         * @param {function} onFail    Fail callback
         * @param {Object}   options   'header: {key, value}', 'timeout: time in ms', 'data: { key, value }, 'method': 'GET or POST'
         */
        fetch = function(url, onSuccess, onFail, options){
          options = options || {};

          //TODO: Finish writing local storage?.
          var maxWaitTime = options.timeout || 2000,
              headers = options.headers || {},
              data = options.data || {},
              method = options.method ? options.method.toUpperCase() : 'GET',
              xhr = new window.XMLHttpRequest(),
              noResponseTimer = window.setTimeout(function() {
                xhr.abort();
                var localStorageData = getLocalStorage(url);
                if ( !!localStorageData && typeof(onFail) == 'function') {
                  onFail(localStorageData);
                  //onFail(xhr.responseText);
                  return;
                }
              }, maxWaitTime);
          
          // headers['Access-Control-Allow-Origin'] = '*.pbskids.org/*, pbskids.org/**, */google-analytics.com/**, *.pbs.org/*';

          xhr.onreadystatechange = function() {
            if (xhr.readyState != 4) {
              return;
            }

            if (xhr.status == 200 || ( xhr.status === 0 && xhr.responseText ) ) {
              window.clearTimeout(noResponseTimer);
              window.localStorage[url] = xhr.responseText;
              onSuccess(xhr.responseText);
            }
            else {
              if (!!window.localStorage[url]) {
                _log('Fetch failed. Local storage succeeded!');
                onSuccess(window.localStorage[url]);
                return;
            }else{
                if (typeof onFail === 'function'){
                  onFail(xhr);
                }else{
                  throw new Error('Unhandled fetch failure.', xhr );
                }
              }
            }
          };

          //TODO: clean up these dangling Vars
          
          // build a query string for the end
          var queryString = Object.keys(data).map(function(key) {
            return encodeURIComponent(key) + '=' + encodeURIComponent(data[key]);
          }).join('&');
          
          
          var getURL = url;
          if (queryString !== ''){ 
            getURL = url + '?' + queryString;
          }
          
          if(method === 'GET') {
            xhr.open('GET', getURL);
          } else if(method === 'POST') {
            xhr.open('POST', url);
            headers['Content-type'] = 'application/x-www-form-urlencoded';
          }

          // Attach headers
          Object.keys(headers).forEach(function(key) {
            _log('setting header: ' + key + ' : ' + headers[key]); 
            xhr.setRequestHeader(key, headers[key]);
          });

          if(method === 'GET') {
            xhr.send();
          } else if(method === 'POST') {
            xhr.send(queryString);
          }
        },
        /**
         * Set to Local Storage
         * @memberof util
         * @param {string} key        Named key for local storage
         * @param {object} dataObject Data to store
         */
        setLocalStorage = function(key, dataObject){
          if (window.hasOwnProperty('localStorage') && window.localStorage !== null) {
            window.localStorage.setItem(key, dataObject);
          } else {
           _log('Error: Local Storage Not Available');
          }
        },
        /**
         * Get from Local Storage
         * @memberof! util
         * @param   {string} key Named key of local storage
         * @returns {Object} Data retrieved from local storage
         */
        getLocalStorage = function(key){
          if (window.hasOwnProperty('localStorage') && window.localStorage !== null) {
            return window.localStorage.getItem(key);
            } else {
            _log('Error: Local Storage Not Available');
          }
        },
        /**
         * Returns a show from the show data endpoint.
         * @param   {String} showGUID The GUID of the show
         * @param   {String} callback A function which accepts the data as an argument
         * @event SHOW_DATA_RETURNED #The show data is returned to e.target.data
         */
        getShowData = function (showGUID, callback){
          var allShowsDataJSON = getLocalStorage('trampoline_allShowsData') || {},
              showDataTimestamp = (getLocalStorage('trampoline_showDataDate') ? new Date(getLocalStorage('trampoline_showDataDate')) : ''),
              showDataExpiration = new Date(new Date() - 1000*60*60*24*7),
              /**
               * Get data for all the shows
               * @private
               * @returns {Object}   Single Show Data as an object
               */
              _fetchShowData = function (){
                var showDataURI = showDataEndpoint;

                function _onShowDataSuccess(data){
                  var ALL_SHOWS_DATA_RETURNED = new Event('ALL_SHOWS_DATA_RETURNED');
                  if (!!data){
                    Trampoline.util.setLocalStorage('trampoline_allShowsData', data);
                    Trampoline.util.setLocalStorage('trampoline_showDataDate', new Date())
                  }else{
                    _log('error getting shows data');
                  }
                }

                if (isConnected){
                  fetch(showDataURI, _onShowDataSuccess);
                }else{
                  _log('no connection');
                  return {};
                }          
              },
              
              /**
               * Return a single show data obj
               * @private
               * @param   {Object} allShowDataJSON The data in json form
               * @returns {Object} The show object
               */
              _returnShowData = function (allShowData){
                var SHOW_DATA_RETURNED,
                    i;

                if (showGUID){
                  for (i = 0; i < allShowData.length; i++) {
                    if (allShowData[i].guid == showGUID){
                      SHOW_DATA_RETURNED = new Event('SHOW_DATA_RETURNED', { 'data': allShowData[i] });
                      _log('show data : ', allShowData[i]);
                      Trampoline.events.dispatchEvent(SHOW_DATA_RETURNED);
                      if( callback ){
                        callback( allShowData[i] );
                      }
                      return allShowData[i];
                    }
                    _log('Error retrieving Show ID');
                    SHOW_DATA_RETURNED = new Event('SHOW_DATA_RETURNED', { 'data': null });
                    if( callback ){
                      callback(null);
                    }
                    Trampoline.events.dispatchEvent(SHOW_DATA_RETURNED);
                  }
                }
              };

          //TODO: This should probably be a try catch
          //Make sure that the data is an Object
          allShowsData = (typeof allShowsDataJSON == 'string' ? JSON.parse(allShowsDataJSON) : allShowsDataJSON);
          
          if ( allShowsData[0] && showDataTimestamp && (showDataTimestamp > showDataExpiration) ){
            if (allShowsData[0] && allShowsData[0].hasOwnProperty('guid')){
              _returnShowData(allShowsData);
            }else{
              _log('show data not ready, setting up listener');
              Trampoline.events.addEventListener('ALL_SHOWS_DATA_RETURNED', _returnShowData);
            }
          }else{
            _log('No show data or data expired. Setting up listener and fetching all shows');
            Trampoline.events.addEventListener('ALL_SHOWS_DATA_RETURNED', _returnShowData);
            _fetchShowData();
          }
        },
        /**
         * Add sticky class on scroll
         * @memberof! Trampoline.util
         * @param {Object} element         DOM Node to make sticky
         * @param {Object} scrollingParent Parent object that will scroll
         */
        makeSticky = function (element, scrollingParent){
          var top = element.offsetTop;
          scrollingParent.onscroll = function() {
              var y = scrollingParent.scrollTop;
              if (y > top) {
                  element.classList.add('sticky');
              }
              else {
                  element.classList.remove('sticky');
              }
          };

        },
        /**
         * Scroll to top based on window height and scroll distance
         * @memberof! Trampoline.util
         * @param   {Object} element DOM Node to scroll
         * @param   {Number} time    Time in MS
         * @returns {object} I honestly don't remember what this is for
         */
        scrollToTop = function(element, time){
            var _currentSpeed,
                _animateFrame = (function(){
                  var action = window.requestAnimationFrame ||
                    window.webkitRequestAnimationFrame ||
                    window.mozRequestAnimationFrame ||
                    window.oRequestAnimationFrame ||
                    window.msRequestAnimationFrame ||
                    function (callback) {
                        window.setTimeout(callback, 1000 / 60);
                    };
                  return function(runner){
                    action.call(window, runner);
                  };
                }()),
                _scrollTop = function(element, nextFrame){
                  if(nextFrame === null) {
                    var isNull;
                    isNull = element.scrollY !== null ? element.scrollY : element.scrollTop;
                    return isNull;
                  } else if(nextFrame <= 0) {
                    element.scrollTo ? element.scrollTo(0, 0) : element.scrollTop = 0 ;
                    return 0;
                  } else {
                    element.scrollTo ? element.scrollTo(0, nextFrame) : element.scrollTop = nextFrame;
                    return nextFrame;
                  }
                },
                _speedConduct = function(originSpeed, time, cur, total){
                  if(total === 0) {
                    return 0;
                  }
                  var _INIT_SPEED = 2;
                  return originSpeed * Math.sin(Math.PI * (total-cur)/total) + _INIT_SPEED;
                },
                _originY = _scrollTop(element),
                _currentY = _originY,
                _originSpeed = _originY / (time / 60);
          
            (function _operate(){
                _currentSpeed = _speedConduct(_originSpeed, time, _currentY, _originY);
                _currentY -= _currentSpeed;
                if(_scrollTop(element, _currentY) !== 0) {
                    _animateFrame(_operate);
                }
            }());
        },

        /**
         * Replace line breaks and spaces with underscores
         * @memberof! Trampoline.util
         * @param   {String} string String to run regex on
         * @returns {string} Returns empty string if a string is not supplied
         */
        underscore = function(string){
          return (typeof string === 'string') ? string.replace(/\n| /g, "_") : '';
        },
        /**
         * Creates base 64 string from an image via canvas(for local storage)
         * @param   {Object} image The image element
         * @returns {String} Base64 String
         */
        //TODO: This should be changed so it doesn't require canvas.
        getBase64Img = function(img) {
            
            var canvas = document.createElement("canvas"),
                ctx,
                dataURL;
          
            canvas.width = img.width;
            canvas.height = img.height;

            ctx = canvas.getContext("2d");
            ctx.drawImage(img, 0, 0);

            dataURL = canvas.toDataURL("image/png");
            
            delete canvas.instance; 
            return dataURL.replace(/^data:image\/(png|jpg);base64,/, "");
        },
        /**
         * Gets image Base64 from LocalStorage and returns img element
         * @param   {String}   key          The localstorage key, usually the image name
         * @param   {Function} loadCallback Function to call when the image has loaded
         * @returns {Object}   img element that has loaded the data from local storage
         */
        getImageFromLocalStorage = function(key, loadCallback){
          var dataImage = getLocalStorage('key'),
              imgElement = document.createElement('img');
          
          imgElement.src = "data:image/png;base64," + dataImage;
          if (typeof callback === 'function'){
           imgElement.onload = function (){
             loadCallback();
            }; 
          }
          return imgElement;
        },
        /**
         * Changes URI to return resized image.pbs.org images with the Image Transformation Service
         * @memberof! Trampoline.util
         * @param   {String} imageURI The image Location
         * @param   {String} resizeW  New Width
         * @param   {String} resizeH  New Height
         * @returns {String} Configured URI, or original if conditions not met.
         */
        resizeITS = function (imageURI, resizeW, resizeH, newExt){
          _log('trying to resize image with ITS');
          if ( !!imageURI && (imageURI.indexOf('image.pbs.org') > -1) ){
			  
            var resizedURI = imageURI + '.resize.',
                extension = newExt || (/(?:\.([^.]+))?$/).exec(imageURI)[0];
            
            if (!!resizeW || !!resizeH){
              if (!!resizeW){
                resizedURI += resizeW + 'x';
                if (!!resizeH){
                  resizedURI += resizeH;
                }
              }else if (!!resizeH){
                resizedURI += 'x' + resizeH;
              }
              return resizedURI += extension;
            } else {
              _log('Neither resize Width or Height Specified');
              return imageURI;
            }           
          } else {
            _log('Image Transormation Service not available on given URI');
            return imageURI;
          }

        },
        /**
         * Check to see if Cordova App
         * @memberof Trampoline.util
         * @name isCordova
         * @returns {bool}
         */        
        isCordova = function() {
            var namespace = !!window.cordova;
            return namespace;
        },
        
        /**
         * Function to see if this an iOS device
         * @memberof! Trampoline.util
         * @returns {Boolean} Returns true only if iOS
         */        
        isiOS = function(){ 
            var iosUA = ( !!isCordova.call() ) ? /iOS|iPhone|iPad|iPod|apple-tablet/i.test(window.device.platform) : /iPhone|iPad|iPod|apple-tablet/i.test(window.navigator.userAgent);
            return ( iosUA || false);
        },
        
        /**
         * Function to see if this an Amazon device
         * @memberof! Trampoline.util
         * @returns {Boolean} Returns true only if Amazon / Silky
         */
        isAmazon = function(){
          var silkyUA =  ( !!isCordova.call() ) ? /Amazon/i.test(window.device.platform) : /silk|kindlefire/gi.test(window.navigator.userAgent);
          return ( silkyUA || false);
        },

        /**
         * Function to see if this an Android device
         * @memberof! Trampoline.util
         * @returns {Boolean} Returns true only if Android
         */        
        isAndroid = function(){
            var androidUA = ( !!isCordova.call() ) ? /Android/i.test(window.device.platform) : /Android/i.test(window.navigator.userAgent);
            return (androidUA || false);
        },

        /**
         * Random number generator
         * @memberof Trampoline.util
         * @param   {Number} min Lower threshold
         * @param   {Number} max Upper threshold
         * @returns {Number} Random integer between min and max
         */
        randomInt = function (min, max) {
          return Math.floor(Math.random() * (max - min + 1)) + min;
        },
        
        getBuildData = function(){
          var _onSuccess = function(buildString){
                Trampoline.util.buildData = JSON.parse(buildString);
                _log('Build data returned');
                Trampoline.events.dispatchEvent(BUILD_DATA_RETURNED);
              },
              _onFail = function(){
                _log('Could not set build data');
              };
          
          fetch('build-data.json', _onSuccess, _onFail);
        },

        displayBuildData = function(){
          //TODO: Inject the styles here, not in the LESS. 
          var _buildData = Trampoline.util.buildData,
              _buildDataContainer  = document.createElement('section');
          
          _buildDataContainer.setAttribute('id', 'trampoline-debug-data');
          _buildDataContainer.innerHTML = '<div class="build-data">' 
                                          + 'PBS Kids Trampoline ' + _buildData.tag
                                          + '<br>' + _buildData.branch + '  #' + _buildData.hash
                                          + '<br>' + _buildData.date + ' ' + _buildData.time
                                          +'</div>';
          
          document.getElementById('trampoline').appendChild(_buildDataContainer);                                     
        },
        /**
         * Random selection of items from an array
         * @param   {Array}  array    The original array reference
         * @param   {number} numItems Number of items to retun
         * @returns {Array}     An array of randomly pop'd items from the array (no duplicates).
         */
        randomArraySelection = function(array, numItems){
          var _itemArray = [],
              _randomArray = [];

          Array.prototype.push.apply(_itemArray, array);

          while(_itemArray.length > 0 && _randomArray.length < numItems){
            var _randomItemIndex = Trampoline.util.randomInt(0, _itemArray.length-1),
                _randomItem = _itemArray[_randomItemIndex];

            _randomArray.push(_itemArray[_randomItemIndex]);
            _itemArray.splice(_randomItemIndex, 1);
          }

          return _randomArray;
        },
        /**
         * Add an animation class to an element, then remove the class when animation is complete
         * @param {Object} node      The element
         * @param {Str}    animation The animation CSS class (See animation.less)
         */
        toggleAnimation = function(node, startAnimation, exitAnimation){
          //TODO: move functions out of if block
          function whichAnimationEvent(){
            var t,
                el = document.createElement("fakeelement"),
                animations = {
                  "animation"         : "animationend",
                  "WebkitAnimation"   : "webkitAnimationEnd"
                };
            for (t in animations){
              if (el.style[t] !== undefined){
                return animations[t];
              }
            }
          }
          
         function exitAnimEvent(e){
            e.target.removeEventListener(animationEvent, exitAnimEvent);
            if ( e.target.classList.contains(startAnimation) ){
              e.target.classList.remove(startAnimation);
              //Add the new exit event listener and animation
              e.target.addEventListener(animationEvent, removeAnimEvent, false);
              node.classList.add(exitAnimation);
            }
          }

          function removeAnimEvent(e){
            e.target.removeEventListener(animationEvent, removeAnimEvent);
            if ( e.target.classList.contains(startAnimation) ){
              e.target.classList.remove(startAnimation);
            } 
            else if (e.target.classList.contains(exitAnimation) ){
              e.target.classList.remove(exitAnimation);
            }
          }
          
          if (!node.classList.contains(startAnimation) && !node.classList.contains(exitAnimation)){

            var animationEvent = whichAnimationEvent();

            //Black Magic to trigger a repaint if restarting the animation (ﾉ◕ヮ◕)ﾉ*:･ﾟ✧
            //node.offsetWidth = node.offsetWidth;

            if (!!exitAnimation){
              node.addEventListener(animationEvent, exitAnimEvent, false);
            }else{
              node.addEventListener(animationEvent, removeAnimEvent, false);
            }

            node.classList.add(startAnimation);
          }
        },
      
        /**
         * Injects contents of .svg into DOM Node with xhr request, so that it can be manipulated with CSS and JS.
         * @private
         * @memberof Trampoline
         * @todo TODO: Finish inject SVG function
         * @param {object}   node   DOM Node
         * @param {string}   url    SVG Location
         * @param {function} onFail Callback in the event that the SVG is unreachable
         *                          
         */
        injectSVG = function (node, url, onFail){
          var _addSVG = function(svgData){
                node.innerHTML = svgData;
              },
              _onFail = function(){
                _log('Failed to inject SVG');
                if (typeof onFail == 'function'){
                  onFail();
                }
              };
          
          Trampoline.util.fetch(url, _addSVG, _onFail);
        },
        
        mobileCheck = function() {
            var check = false;
            (function(a){if(/(android|ipad|playbook|silk|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i.test(a)||/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0,4)))check = true})(navigator.userAgent||navigator.vendor||window.opera);
          return check;
        },
        tapEvent = (function(){
          //return mobileCheck() ? 'touchstart' : 'click';
          return document.documentElement.hasOwnProperty('ontouchstart') ? 'touchstart' : 'click';
        }()),
        tapEnterExit = function(node, onTapEnter, onTapExit){
          //TODO: make this check more robust
          var startEvent = mobileCheck() ? 'touchstart' : 'mousedown',
              endEvent   = mobileCheck() ? 'touchend' : 'mouseup'; 
          
          function handleStartEvent(e){
            e.target.addEventListener(endEvent, onTapExit);
          }
                    
          //handle the start tap
          node.addEventListener(startEvent, handleStartEvent);
          
        },
        styleSheet = (function (){
          var stylesheets = document.styleSheets[(document.styleSheets.length - 1)],
              trampolineStyleSheet;

          for( var i = 0; i < document.styleSheets.length; i++ ){
              if( document.styleSheets[i].href && document.styleSheets[i].href.indexOf("trampoline.css") ) {
                  trampolineStyleSheet = document.styleSheets[i];
                  break;
              }
          }
          return trampolineStyleSheet;
        }()),
        
        handleSwipeTouchStart = function (startEvent){
          var touchstartX = startEvent.changedTouches[0].clientX,
              touchstartY = startEvent.changedTouches[0].clientY;
                
          function swipeTouchEndCallback (endEvent){
            startEvent.target.removeEventListener('touchend', swipeTouchEndCallback);
            handleSwipeTouchEnd(endEvent, touchstartX, touchstartY);
          }
          
          startEvent.target.addEventListener('touchend', swipeTouchEndCallback);          
        },
        
        handleSwipeTouchEnd = function (endEvent, touchstartX, touchstartY){
          var swipeEvent,
              dX = touchstartX - endEvent.changedTouches[0].clientX,
              dY = touchstartY - endEvent.changedTouches[0].clientY,
              acceptableDistanceX = screen.width * 0.05,
              acceptableDistanceY = screen.height * 0.10;
                    
          //old version of distance that's relative to screen diagonal
          //acceptableDistance = 0.3 * Math.sqrt(screen.width*screen.width + screen.height*screen.height);

          
          if ( ( Math.abs(dY) > Math.abs(dX) ) && ( Math.abs(dX)/Math.abs(dY) < 1 ) ){
            //Swiped Vert
            if (Math.abs(dY) > acceptableDistanceY){
              swipeEvent = (dY > 0) ? 'TRAMPOLINE_SWIPE_UP' : 'TRAMPOLINE_SWIPE_DOWN';

            }
          }else if ( ( Math.abs(dY) < Math.abs(dX) ) && ( Math.abs(dY)/Math.abs(dX) < 1 ) ){
            //Swiped Horz
            if (Math.abs(dX) > acceptableDistanceX){
              swipeEvent = (dX > 0) ? 'TRAMPOLINE_SWIPE_LEFT' : 'TRAMPOLINE_SWIPE_RIGHT';
            }
          }

          if (!!swipeEvent){
            var SWIPE_EVENT = new CustomEvent(swipeEvent, {'detail': { 'touchEndEvent': endEvent, 'dX' : dX, 'dY' : dY }});
            
            Trampoline.events.dispatchEvent(SWIPE_EVENT);
          }
          
        },

        /**
         * Copies the touch event
         * @param   {object} touch touchevent
         * @returns {object} object with params from touchevent 
         */
        copyTouchEvent = function(touch){
          return { identifier: touch.identifier, pageX: touch.pageX, pageY: touch.pageY, clientX: touch.clientX, clientY: touch.clientY  };
        },
        
        /**
         * Touchevent pull down from to - panel will close, translate3D attached to touchmove for UX feedback 
         * TODO: Write in conditions to accept dX value 
         * @param   {object}         pullElement the panel to move
         * @param   {object}         options {dY, dX, scrollParent, parentElement}
         */
        addSwipePull = function(pullElement, options){
          var startTouch = {},
              triggerDistanceY = screen.height * 0.00,
              direction = (parseFloat(options.dY) < 0) ? -1 : 1,
              acceptableDistanceY = (!!options.dY) ? screen.height * options.dY : false,
              acceptableDistanceX = (!!options.dX) ? screen.width * options.dX : false,
              _scrollParent = (!!options.scrollParent) ? options.scrollParent : false,
              _parentElement = (!!options.parentElement) ? options.parentElement : pullElement,
              startTransformMatrix;
          
          pullElement.addEventListener('touchstart', _handlePullTouchStart, false);
          
          function _handlePullTouchStart(e){
            if (!!_scrollParent && _scrollParent.scrollTop > 0){
              // Quit it. Touchstart on scroll that is not at the top
              return;
            }
            startTransformMatrixStr = Trampoline.util.matrixStrToArr(window.getComputedStyle(_parentElement).transform);
            
            //TODO: capture more than just the first touch
            startTouch = Trampoline.util.copyTouchEvent(e.changedTouches[0]);
            pullElement.addEventListener('touchmove', _handlePullTouchMove, false);
            pullElement.addEventListener('touchend', _handlePullTouchEnd, false);
          }
          
          function _handlePullTouchMove(e){
            var changedTouches = e.changedTouches;
          
            for (var i = 0; i < changedTouches.length; i++) {
              if (changedTouches[i].identifier == startTouch.identifier){
                //You got the touch! (You got the power!)
                var move_dY = startTouch.clientY - changedTouches[i].clientY,
                    updatedTransformMatrix = startTransformMatrixStr.slice();
                
                if ( (direction < 0) ? (move_dY < triggerDistanceY) : (move_dY > triggerDistanceY) ){
                  if (!!_scrollParent){
                    if ( !_scrollParent.classList.contains('pulled') ){
                      _scrollParent.classList.add('pulled');
                    }
                    _scrollParent.scrollTop = 0;
                  }
                  //Update the Transformation based on any Transform already applied to this
                  //TODO: add dX to this.
                  updatedTransformMatrix[5] +=  -move_dY;                
                  _parentElement.style.transform = 'matrix(' + updatedTransformMatrix.toString() + ')';                  
                }
                
                if ( (direction < 0) ? (move_dY < 2*acceptableDistanceY ) : (move_dY > 2*acceptableDistanceY) ){
                  _handlePullTouchEnd(e);
                }
              }
            }
          }
          
          function _handlePullTouchEnd(e){
            var endTouch = Trampoline.util.copyTouchEvent(e.changedTouches[0]),
                dX = startTouch.clientX - endTouch.clientX,
                dY = startTouch.clientY - endTouch.clientY;
                                                
            if (startTouch.identifier == endTouch.identifier){
              // Remove the listeners
              pullElement.removeEventListener('touchmove', _handlePullTouchMove, false);
              pullElement.removeEventListener('touchend', _handlePullTouchEnd, false);
              if (!!_scrollParent) { _scrollParent.classList.remove('pulled'); }
                        
              _parentElement.style.transform = '';
              // The drag is over and meets acceptable distance in correct direction
              // and double checks that the scroll is at the top toggle the view, if scrollParent is provided.
              //TODO: Make this accept more than just a state change.

              if ( Math.abs(dX/dY) < 1 ){
                //Vertical Swipe -- check scroll top only on swipe down
                if ( !!acceptableDistanceY && ((direction < 0) ? ( (dY <= acceptableDistanceY) && !!_scrollParent && (_scrollParent.scrollTop <= 0) ) : (dY >= acceptableDistanceY)) ){
                  if (!!options.callback && typeof(options.callback) == 'function'){
                    _log('Swipe Pull ' + (direction>0 ? 'Up' : 'Down') + ', Callback');
                    options.callback.call();
                  }else {
                    //No callback function, change state
                    _log('Swipe Pull ' + (direction>0 ? 'Up' : 'Down') + ', To state: ' + options.state);
                    (!!options.state && options.state !== 'back') ? Trampoline.state.change( {to: options.state } ) : Trampoline.state.back();
                  }
                }
              }else{
                //Horizontal Swipe
                if ( !!acceptableDistanceX && ( (direction < 0) ? (dX <= acceptableDistanceX) : (dX >= acceptableDistanceX)) ){
                  if (!!options.callback && typeof(options.callback) == 'function'){
                    _log('Swipe Pull ' + (direction>0 ? 'Right' : 'Left') + ', Callback');
                    options.callback.call();
                  }else {
                    //No callback function, change state
                    _log('Swipe Pull ' + (direction>0 ? 'Right' : 'Left') + ', To state: ' + options.state);
                    (!!options.state && options.state !== 'back') ? Trampoline.state.change( {to: options.state } ) : Trampoline.state.back();
                  }
                }
              }
            }
          }
        },
        matrixStrToArr = function(matrixStr){
          var matrixArr = matrixStr.replace(/matrix\(|\)| /g, '').split(',').map(Number);
          return matrixArr;
        },
        
        preventDefaultThis = function(e){
          if (e.target == this){
            e.preventDefault();
            return false;
          }
        };

    return {
      isConnected           : isConnected,
      fetch                 : fetch,
      buildData             : buildData,
      getBuildData          : getBuildData,
      displayBuildData      : displayBuildData, 
      setLocalStorage       : setLocalStorage,
      getLocalStorage       : getLocalStorage,
      resizeITS             : resizeITS,
      getShowData           : getShowData,
      makeSticky            : makeSticky,
      scrollToTop           : scrollToTop,
      underscore            : underscore,
      isiOS                 : isiOS,
      isAmazon              : isAmazon,
      isAndroid             : isAndroid,
      randomInt             : randomInt,
      randomArraySelection  : randomArraySelection,
      toggleAnimation       : toggleAnimation,
      injectSVG             : injectSVG,
      tapEvent              : tapEvent,
      preventDefaultThis    : preventDefaultThis,
      currentTouches        : currentTouches,
      copyTouchEvent        : copyTouchEvent,
      handleSwipeTouchStart : handleSwipeTouchStart,
      styleSheet            : styleSheet,
      addSwipePull          : addSwipePull,
      matrixStrToArr        : matrixStrToArr
    };
  }());

  /**
  * @memberof Trampoline.util
  */
  return Trampoline;
  
}(Trampoline || {}));

(
/** 
 * Trampoline Parent Gate
 * @memberof Trampoline
 * @namespace gate
 * 
 */  
  function(Trampoline){
  Trampoline.gate = (function (){
    var _trampolineContainer,
        _gateContainer,
        _gateActive,
        _gateLanguage,
        _log = Trampoline._log,
        tapEvent = Trampoline.util.tapEvent,
        _gateShapeList = [{english: 'circle', espanol: 'círculo'}, {english: 'square', espanol: 'cuadrado'}, {english: 'triangle', espanol : 'triángulo'}],
        _gateNumberList = [{english: 'one', espanol: 'uno', text: '1'}, {english: 'two', espanol: 'dos', text: '2'}, {english: 'three', espanol: 'tres', text: '3'}, {english: 'four', espanol: 'quatro', text: '4'}, {english: 'five', espanol: 'cinco', text: '5'}, {english: 'six', espanol: 'seis', text: '6'}, {english: 'seven', espanol: 'siete', text: '7'}, {english: 'eight', espanol: 'ocho', text: '8'}, {english: 'nine', espanol: 'nueve', text: '9'}]
        
        //EVENTS
        GATE_OPEN   = new Event('GATE_OPEN'),
        GATE_CLOSE  = new Event('GATE_CLOSE'),
        GATE_PASS   = new Event('GATE_PASS'),
        GATE_FAIL   = new Event('GATE_FAIL'),
        
        /**
         * Initializes Parent Gate DOM element
         * @memberof Trampoline.gate
         **/
        init = function(){
          // Waiting until Init to grab these Elements, for memory management, ya know.
          _trampolineContainer = Trampoline.container || document.getElementById('trampoline');
          _gateContainer = _trampolineContainer.querySelector('section.gate');
          _gateContainer.querySelector('.gate-ui').addEventListener('touchmove', Trampoline.util.preventDefaultThis, false);
          
          //Setup language switch
          _gateLanguage = 'english';
          _gateContainer.dataset.language  = _gateLanguage;
          _gateContainer.querySelector('.language-switch').addEventListener(tapEvent, function(e){
            e.preventDefault();
            toggleLanguage();
          });
          _log('Initiating Parent Gate...');
        },
        /**
         * Toggles the perent gate.
         * @param {Function} callback Required callback when calling the parent gate to open.
         * @memberof Trampoline.gate
         */
        
        toggleView = function (callback){
          if (!callback && !!_gateActive){
            _close();
          }else if (!_gateActive){
            _open(callback);
          } else {
            return;
          }
        },
        _open = function (callback){ 
          _gateActive = true;
          _buildGateUI(_gateShapeList, _gateNumberList, callback);
          _gateContainer.classList.add('active');
          
          Trampoline.analytics.event('Parent_Gate', 'open');
          Trampoline.events.dispatchEvent(GATE_OPEN);
        },
        _close = function (){
          function _doClose(){
            if (!_gateActive){
              _clearUI();
              _gateContainer.removeEventListener('transitionend', _doClose);
              Trampoline.events.dispatchEvent(GATE_CLOSE);
            }
          }
          //TODO: Remove drop & lameDrop event listeners, they are stacking up
          
          _gateContainer.addEventListener('transitionend', _doClose);
          _gateActive = false;
          _gateContainer.classList.remove('active');
          _gateContainer.scrollTop = 0;
          return false;
        },
        _pass = function (callback){
          _close();
          Trampoline.analytics.event('Parent_Gate', 'pass');
          Trampoline.events.dispatchEvent(GATE_PASS);
          if (typeof callback === 'function'){
            callback();
          }
        },
        _fail = function (callback){
          //Shake for UI feedback or Close Gate
          var _gateUI = _gateContainer.querySelector('.gate-ui'),
              _doFail = function(){
                _gateContainer.querySelector('.gate-ui').classList.remove('shaking');
                _clearUI(function(){
                  _buildGateUI(_gateShapeList, _gateNumberList, callback);
                });
                Trampoline.analytics.event('Parent_Gate', 'fail');
                Trampoline.events.dispatchEvent(GATE_FAIL);
              };
          
          _gateUI.classList.add('shaking');
          _gateUI.addEventListener('animationend', _doFail);
        },
        /**
         * Build the UI from a list of shapes.
         * @param {Array}    shapeList A list of possible shapes
         * @param {Number} numShapes Int, Number of shapes to display
         */
        _buildGateUI = function (shapeList, numberList, callback){
          //Setup the stuff:
          var _gateInstructionsEnglish = document.createElement('h1'),
              _gateInstructionsEspanol = document.createElement('h1'),
              _randomShapeList = Trampoline.util.randomArraySelection(shapeList, 3),
              _randomNumberList = Trampoline.util.randomArraySelection(numberList, 3),
              _nextItemList = [],
              _gateUI = _gateContainer.querySelector('.gate-ui'),
              _gateWrapper = _gateContainer.querySelector('.gate-wrapper'),
              _gateItemWrapper = _gateContainer.querySelector('.gate-item-wrapper'),
              _targetShape = _randomShapeList[Trampoline.util.randomInt(0, _randomShapeList.length-1)],
              _targetNumber = _randomNumberList[Trampoline.util.randomInt(0, _randomNumberList.length-1)],
              _addGateItem = function(gateItem){
                //TODO: add random offset             
                var _gateElement = document.createElement('div');              
                _gateElement.classList.add('gate-item');
                _gateElement.dataset.name = gateItem.english;
                _gateElement.setAttribute('draggable', true);
                _gateElement.innerHTML = (gateItem.hasOwnProperty('text') ? gateItem.text : '');
                _gateItemWrapper.appendChild(_gateElement);
                //Add drag to everything
                _gateElement.addEventListener('dragstart', _handleDragStart, false);
                _gateElement.addEventListener('dragleave', _handleDragLeave, false);
                _gateElement.addEventListener('dragend', _handleDragEnd, false);
                _gateElement.addEventListener('touchcancel', _lameDropCallback, false);
                
                //Add drop to everything
                _gateElement.addEventListener('drop', _dropCallback, false);
                _gateElement.addEventListener('dragenter', _handleDragEnter, false);
                _gateElement.addEventListener('dragover', _handleDragOver, false);
                _gateElement.addEventListener('dragleave', _handleDragLeave, false);
              },
              _setTarget = function(gateTarget, gateKey){
                var _gateTarget = _gateItemWrapper.querySelector('div[data-name="'+gateTarget+'"]');
                _gateTarget.classList.add('gate-target');
                _gateTarget.dataset.key = gateKey;
              },
              _dropCallback = function(e){
                _handleDrop(e, callback);
              },
              _lameDropCallback = function(e){
                _handleLameDrop(e, callback);
              };
          
          //Re-Setup Close (listeners are wiped away on clearUI)
          _gateContainer.querySelector('button.close').addEventListener(tapEvent, function(e){
            e.preventDefault();
            _close(e);
          });
          _gateWrapper.addEventListener(tapEvent, function(e){
            //e.preventDefault();
            _handleOutsideClose(e);
          });
          _gateWrapper.addEventListener('touchmove', function(e){
            e.preventDefault();
            _handleOutsideClose(e);
          }, false);
          
          //Alternate shapes and numbers and add them randomly from each list.
          for (var i=0; i<6; i++){
            var _nextItemList =  (i % 2) ? (_randomShapeList.length ? _randomShapeList : _randomNumberList) : (_randomNumberList.length ? _randomNumberList : _randomShapeList),
                _randomItemIndex = Trampoline.util.randomInt(0, _nextItemList.length-1),
                _nextItem = _nextItemList[_randomItemIndex];

            _addGateItem(_nextItem);

            if (_nextItem == _targetShape){
              _setTarget(_nextItem.english, _targetNumber.english);
            }
            //Removes item from respective list (shape or number). _nextItemList is just a reference. 
            _nextItemList.splice(_randomItemIndex, 1);
          }
          
          _gateContainer.querySelector('.gate-wrapper:not(.gate-target)').addEventListener('dragover', _handleDragOver, false);
          _gateContainer.querySelector('.gate-wrapper:not(.gate-target)').addEventListener('drop', _lameDropCallback, false);

          //Add the instructions in English
          _gateInstructionsEnglish.innerHTML = 'Drag the number <strong>'+_targetNumber.english+'</strong> to the <strong>'+_targetShape.english+'</strong>';
          _gateInstructionsEnglish.classList.add('gate-instructions', 'english');
          _gateUI.appendChild(_gateInstructionsEnglish);

          //Add the instructions en Español
          _gateInstructionsEspanol.innerHTML = 'Arrastre el número <strong>'+_targetNumber.espanol+'</strong> en la forma de <strong>'+_targetShape.espanol+'</strong>';
          _gateInstructionsEspanol.classList.add('gate-instructions', 'espanol');
          _gateUI.appendChild(_gateInstructionsEspanol);
        },
        
        /**
         * Clears out the entire Gate Wrapper, replaces it with a clone that has no listeners.
         * @private
         */
        _clearUI = function (callback){          
          var _gateUI =  _gateContainer.querySelector('.gate-ui'),
              _gateWrapper = _gateContainer.querySelector('.gate-wrapper'),
              _gateWrapperClone,
              _gateInstructionsEnglish = _gateContainer.querySelector('.gate-instructions.english'),
              _gateInstructionsEspanol = _gateContainer.querySelector('.gate-instructions.espanol');

          _gateContainer.querySelector('.gate-item-wrapper').innerHTML = '';
          _gateUI.removeChild(_gateInstructionsEnglish);
          _gateUI.removeChild(_gateInstructionsEspanol);
                   
          //replace the wrapper with clone to remove all lingering listeners
          _gateWrapperClone = _gateWrapper.cloneNode(true);
          _gateWrapper.parentNode.replaceChild(_gateWrapperClone, _gateWrapper);
          
          if (typeof callback == 'function'){
            callback();
          }
        },

        _handleDragStart = function(e) {
          var _isDragging = _gateContainer.querySelector('.dragging');
          if ( !_isDragging ){
            e.target.classList.add('dragging');
            //Listener does not need to be removed; e.target will be deleted.
            //e.dataTransfer.setDragImage(dragIcon, -10, -10);
            e.dataTransfer.effectAllowed = "copy";
            e.dataTransfer.setData("text/plain", e.target.dataset.name);
          }else{
            return false;
          }
        },

        _handleDragEnter = function(e) {
          var DraggedShape = e.target;
          DraggedShape.classList.add('over');
        },
        _handleDragOver = function(e){
          e.preventDefault();
        },

        _handleLameDrop = function(e, callback){
          var droppedShape = e.dataTransfer.getData("text/plain");        
          
          if ( !e.target.dataset.target == droppedShape ){
           _fail(callback);
          }
        },
        
        _handleDragLeave = function() {
          this.classList.remove('over');
        },
        
        _handleDragEnd = function() {
          this.classList.remove('dragging');
        },
        
        _handleDrop = function (e, callback){
          //element dropped
          e.preventDefault();

          var droppedShape = e.dataTransfer.getData("text/plain"),
              droppedNode = _gateContainer.querySelector('div[data-name="'+droppedShape+'"]');
        
          droppedNode.classList.add('dropped');

          if ( e.target.dataset.key == droppedShape ){
            e.target.classList.add('dropped');
            _pass(callback);
              
          }else{
            e.target.classList.remove('over');
            _fail(callback);
          }
          
        },
        
        _handleOutsideClose = function (e){
          if (e.target != _gateContainer.querySelector('.gate-wrapper') ){
            return;
          }
          _close();
        },
        
        toggleLanguage = function (){
          if (_gateContainer.dataset.language != 'english'){
            _gateContainer.dataset.language = _gateLanguage = 'english';
          }else {
            _gateContainer.dataset.language = _gateLanguage = 'espanol';
          }
        };
    
    return {
      init            : init,
      toggleLanguage  : toggleLanguage,
      toggleView      : toggleView,
      close           : _close
    };
  })();
            
  return Trampoline;
  
}(Trampoline || {}));
(
/** 
 * Trampoline More Apps
 * @memberof Trampoline
 * @namespace moreApps
 * 
 */  
  function(Trampoline){
  Trampoline.moreApps = (function (){
    
    //Inherintly private (_) methods
    var ready = false,
        feature = {},
		largeFeatures = [],
        _log = Trampoline._log,
        _didInit,
        _didBuild,
        _trampolineContainer,
        _moreAppsContainer,
        _moreAppsWrapper,
        _moreAppsContentContainer,
        _panelActive,
        _silentMode = false,
		_removeMoreApps = false,
        _scrollParent,
        tapEvent = Trampoline.util.tapEvent,
        
        //_moreAppsDataEndpoint = 'http://api.pbskids.org/trampoline/moreapps/',
        //New Endpoint for Tortuga, must resolve to Default / App-Slug
        _moreAppsDataEndpoint = 'http://pbskids.org/api/trampoline/v2/moreapps/',
        
        //EVENTS
        MOREAPPS_READY                = new Event('MOREAPPS_READY'),
        MOREAPPS_BUTTON_READY         = new Event('MOREAPPS_BUTTON_READY'),
        MOREAPPS_FEATURE_READY        = new Event('MOREAPPS_FEATURE_READY'),
        MOREAPPS_LARGE_FEATURES_READY = new Event('MOREAPPS_LARGE_FEATURES_READY'),
        MOREAPPS_OPEN                 = new Event('MOREAPPS_OPEN'),
        MOREAPPS_CLOSE                = new Event('MOREAPPS_CLOSE'),
        
        /**
         * Initializes the More App section by connecting to trampoline/moreapps/{{deviceOS}} endpoint and
         * building out the more apps DOM element.
         * @memberof Trampoline.moreApps
         * @param {Object} options moreApps init option (silent mode)
         */
        init = function(options){      
          _trampolineContainer      = Trampoline.container || document.getElementById('trampoline');
          _moreAppsContainer        = _trampolineContainer.querySelector('section.more-apps');
          _moreAppsWrapper          = _moreAppsContainer.querySelector('.more-apps-wrapper');
          _moreAppsContentContainer = _moreAppsContainer.querySelector('.more-apps-wrapper .content');
          _scrollParent             = _moreAppsContainer.querySelector('.more-apps-wrapper .content-wrapper')
          
          _moreAppsContainer.addEventListener('touchmove', function(e){
            if (_moreAppsWrapper.scrollTop < 0){
              _moreAppsWrapper.scrollY = 0;
            }
          }, false);

          if (!!options){
            if (!!options.silent){
              _silentMode = true;
            }
		  	
			if (!!options.remove){
              _removeMoreApps = true;
            }
          }
          
          Trampoline.util.addSwipePull(_moreAppsContainer, {dY: -0.10, state: 'back', scrollParent : _scrollParent});
          
          _didInit = true;
        },
        
        /**
         * Exposes the resources needed for startup to the module loader
         * @memberof Trampoline.moreApps
         * @public
         * @returns {Array} List of resources
         */
        listResources = function(){
          return[ _getMoreAppsUri(Trampoline.GUID)];
        },

        /**
         * Builds out the section after init. Run by the module loader in core.js.
         * @param {Object} resources Config options
         */
        startup = function( resources ){
		  if (_removeMoreApps){
			  return;
		  }
          var moreappsResource = resources[ _getMoreAppsUri(Trampoline.GUID) ];
          
          if( moreappsResource){
            _log('Received More Apps Data through GUID');
            _buildMoreApps( moreappsResource.data );
            
          }else if (!!Trampoline.GUID){
            _log('More Apps Data Failed for App GUID, fetching default data...');
            var moreAppsUri = _getMoreAppsUri(Trampoline.GUID);
            Trampoline.util.fetch( moreAppsUri, _buildMoreApps, _log );
            
          }else {
            _log('More Apps Data Failed');
          }
          
        },
        
        /**
         * Refreshes More Apps panel and exposed objects based on init / startup success.
         * @param {Object} options The same options from startup, just because
         */
        refresh = function (options) {          
          if (!_didInit){
            
            Trampoline.moreApps.init(options);
            _moreAppsRestart(options);
              
          }else if (!_didBuild){
            _log('moreApps.init already completed, build failed, rebuilding');  
            _moreAppsRestart(options)
            
          }else{
            _log('moreApps already built, refreshing data');
            var moreAppsUri = _getMoreAppsUri(Trampoline.GUID);
            _handleMoreAppsData( moreAppsUri, _refreshMoreAppsData );
          }
        },
        
        /**
         * Builds a mini loader to run startup outside of Trampoline.init
         * @memberof Trampoline.moreApps
         * @private
         * @param {Object} options The same options from startup
         */
        _moreAppsRestart = function (options){
          var moreAppsRefreshLoader = new Trampoline.classes.ResourceLoader();

          if( _didInit && Trampoline.moreApps.listResources ){
            moreAppsRefreshLoader.add( Trampoline.moreApps.listResources() );
          }

          moreAppsRefreshLoader.load( function( moreAppsRefreshLoader ){
            Trampoline.moreApps.startup( moreAppsRefreshLoader.resources );
          });
        },
        
        /**
         * Refreshes moreapps data only
         * @memberof Trampoline.moreApps
         * @private
         * @param {Object} moreAppsData The data from Tortuga
         */
        _refreshMoreAppsData = function (moreAppsData){
          _log('Refreshing More Apps data only...');
          var moreAppsDataObj = JSON.parse(moreAppsData);
          
          _styleMoreApps(moreAppsDataObj.settings);
                              
          //Build the MoreApps Button if needed, inject when ready
          _log( 'HAS MORE APPS' + !Trampoline.container.classList.contains('has-more-apps') )
          if (!_trampolineContainer.classList.contains('has-more-apps')){
            if (!_silentMode && moreAppsDataObj.settings){
               Trampoline.events.addEventListener('MOREAPPS_BUTTON_READY', _addMoreAppsButton); 
              _buildMoreAppsButton(moreAppsDataObj.settings);
            }
          }
          
          (!!moreAppsDataObj.featurePanel) ? _exposeFeatureData(moreAppsDataObj.featurePanel) : _log('No feature data to expose');
          
        },
        
        /**
         * Return MoreApps JSON data and build MoreApps Panel on success
         * @memberof Trampoline.moreApps
         * @param {String} dataURI Endpoint for Moreapps Data
         */
        _handleMoreAppsData = function(dataURI, onSuccess){
          _log('Getting more apps data...');
          Trampoline.util.fetch( dataURI, onSuccess, _log );
        },

        /**
         * Returns the proper More Apps API URI for the device with links to that App Store
         * @memberof Trampoline.moreApps
         * @private
         * @param   {String} endpointSlug The GUID of the app.
         * @returns {String} The More Apps endpoint
         */
        _getMoreAppsUri = function(endpointSlug){

          var moreappsURI = (!!endpointSlug) ? _moreAppsDataEndpoint + endpointSlug + '/' : _moreAppsDataEndpoint + 'default/';
          
          if (Trampoline.util.isiOS.call()){
            moreappsURI += 'ios';
          }else if (Trampoline.util.isAmazon.call()) {
            moreappsURI += 'amazon';
          }else{
            moreappsURI += 'android';
          }
          return moreappsURI;
        },
        
        /**
         * Populates more apps panel with newest available data from json
         * @memberof Trampoline.moreApps
         * @private
         * @param   {String}  moreAppsData JSON String returned from endpoint
         */
        _buildMoreApps = function (moreAppsData){
          _log('Building More Apps...');
          var moreAppsDataObj = JSON.parse(moreAppsData),
              moreAppsObj = moreAppsDataObj.moreApps,
              largeFeatureArray = [];
          
          _styleMoreApps(moreAppsDataObj.settings);
                    
          //Build the MoreApps Button, inject when ready
          if (!_silentMode && moreAppsDataObj.settings){
             Trampoline.events.addEventListener('MOREAPPS_BUTTON_READY', _addMoreAppsButton); 
            _buildMoreAppsButton(moreAppsDataObj.settings);
          }
          
          //TODO: Is this the right place to expose the feature data?
          (!!moreAppsDataObj.featurePanel) ? _exposeFeatureData(moreAppsDataObj.featurePanel) : _log('No feature data to expose');

          
          
          /**
           * TODO: Create some sort of object for reference so that after two panels are built, wait for all the images to load and then show the more-apps button.             
           * 
           * TODO: Create a function here that makes an image object, downloads the images and returns a callback.
           * 
           * TODO: Have each panel animate in when all of its images have loaded
           */        
          //Build out the MoreApps Section Panels
          for (var count=0; count<moreAppsObj.blocks.length; count++){
            var panelData = moreAppsObj.blocks[count],
                moreAppsPanel = document.createElement('div');
              
            switch(panelData.type){
                
              case 'feature_large':
                var featureData = panelData;
                var panelHeading, panelImage, featureButton,
                featureObject = {
                  featureGraphic : featureData.featureGraphic,
                  storeUrl: featureData.apps.storeUrl,
                  buttonText: featureData.buttonText,
                  buttonColor: featureData.buttonColor,
                  type: featureData.type,
                  appIcon: featureData.apps.appIcon,
                  appIcon2x: featureData.apps.appIcon2x,
                  title: featureData.apps.title,
                  badge: featureData.apps.badge,
                };
					
                largeFeatureArray.push(featureObject);
                                
                //Classes
                moreAppsPanel.classList.add('feature');
                featureButton = _buildFeatureButton(featureObject, moreAppsPanel);
                
                //Heading
                panelHeading = document.createElement('h2');
                panelHeading.setAttribute('backgroundColor', featureData.buttonColor);
                panelHeading.innerHTML = featureData.buttonText;
                moreAppsPanel.appendChild(panelHeading);
                
                break;
                
              case 'split_feature':
                featureData = panelData;
                var panelLeftHeading, panelLeftImage, panelLeftFeature, 
                    panelRightHeading, panelRightImage, panelRightFeature;
                
                moreAppsPanel.classList.add('split-feature');
                
                //Left Feature
                panelLeftFeature = document.createElement('a');
                panelLeftFeature.addEventListener('click', function(e){
                  //TODO: Add some UI animation to this.
                  //Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit'); 
                  Trampoline.gate.toggleView( function(){
                    window.open(featureData.apps[0].storeUrl, '_system');
                  });
                  Trampoline.analytics.event('Selected_in_'+featureData.type, Trampoline.util.underscore(featureData.apps[0].title), 'MoreApps');
                });
                panelLeftImage = new Image(); 
                panelLeftImage.src = featureData.apps[0].featureGraphic;
                panelLeftImage.onload = function(){
                };
                panelLeftFeature.appendChild(panelLeftImage);
                moreAppsPanel.appendChild(panelLeftFeature);
                
                //Right Feature
                panelRightFeature = document.createElement('a');
                panelRightFeature.addEventListener('click', function(e){
                  Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit'); 
                  Trampoline.gate.toggleView( function(){
                    window.open(featureData.featureData.apps[1].storeUrl, '_system');
                  });
                  Trampoline.analytics.event('Selected_in_'+featureData.type, Trampoline.util.underscore(featureData.apps[1].title), 'MoreApps');
                });              
                panelRightImage = new Image(); 
                panelRightImage.src = featureData.apps[1].featureGraphic;
                panelRightImage.onload = function(){
                };
                panelRightFeature.appendChild(panelRightImage);
                moreAppsPanel.appendChild(panelRightFeature);
                break;
                  
              case 'panel_single_row':
                var panelHeading, panelDescription, panelApps;
                //Classes
                moreAppsPanel.classList.add('panel', 'carousel', 'pref-single');
                //Headings
                  panelHeading = document.createElement('h2');
                  panelHeading.innerHTML = panelData.headerText;
                  moreAppsPanel.appendChild(panelHeading);
                
                  panelDescription = document.createElement('h3');
                  panelDescription.innerHTML = panelData.description;
                  moreAppsPanel.appendChild(panelDescription);
                
                panelApps = document.createElement('ul');
                for (var i=0; i<panelData.apps.length; i++){
                  var _newApp = _buildAppIcon(panelData.apps[i], panelData.headerText, panelData.type);
                  panelApps.appendChild(_newApp);
                  //TODO: Keep Track of App Image Download Progress?
                }
                
                moreAppsPanel.appendChild(panelApps);
                break;
                                
              case 'panel_prefer_two_rows':
                var panelHeading, panelDescription, panelApps;
                //Classes
                moreAppsPanel.classList.add('panel', 'carousel', 'pref-double');
                //Headings
                  panelHeading = document.createElement('h2');
                  panelHeading.innerHTML = panelData.headerText;
                  moreAppsPanel.appendChild(panelHeading);
                
                  panelDescription = document.createElement('h3');
                  panelDescription.innerHTML = panelData.description;
                  moreAppsPanel.appendChild(panelDescription);
                
                panelApps = document.createElement('ul');
                for (var i=0; i<panelData.apps.length; i++){
                  var _newApp = _buildAppIcon(panelData.apps[i], panelData.headerText, panelData.type);
                  panelApps.appendChild(_newApp);
                  //TODO: Keep Track of App Image Download Progress?
                }
                
                moreAppsPanel.appendChild(panelApps);
                break;
                
              case 'panel_large_icons':
                var panelHeading, panelDescription, panelApps;
                //Classes
                moreAppsPanel.classList.add('panel', 'list', 'large');
                //Headings
                  panelHeading = document.createElement('h2');
                  panelHeading.innerHTML = panelData.headerText;
                  moreAppsPanel.appendChild(panelHeading);
                
                  panelDescription = document.createElement('h3');
                  panelDescription.innerHTML = panelData.description;
                  moreAppsPanel.appendChild(panelDescription);
                
                panelApps = document.createElement('ul');
                for (var i=0; i<panelData.apps.length; i++){
                  var _newApp = _buildAppIcon(panelData.apps[i], panelData.headerText, panelData.type);
                  panelApps.appendChild(_newApp);
                  //TODO: Keep Track of App Image Download Progress?
                }
                
                moreAppsPanel.appendChild(panelApps);
                break;
              default: 
                _log('More Apps panel Type not supported');
            }
            
            _moreAppsContentContainer.appendChild(moreAppsPanel);
            
          }
          
          if (largeFeatureArray.length > 0){
            _exposeLargeFeaturesData(largeFeatureArray);
          }

          _didBuild = true;
          Trampoline.events.dispatchEvent(MOREAPPS_READY);
          
        },

        /**
         * Builds the more Apps button for the nav (little guy)
         * @memberof Trampoline.moreApps
         * @private
         * @param {object} settingsObj The settings obj from the more apps data
         */
        _buildMoreAppsButton = function (settingsObj){
            _log('getting moreapps button');                  
            var moreAppsButtonImage = new Image(),
                moreAppsButtonBadge = new Image();
                        
            moreAppsButtonImage.src = (window.devicePixelRatio >= 2) ? settingsObj.trampolineIcon2x : settingsObj.trampolineIcon;

            moreAppsButtonImage.onload = function(){
              var moreAppsButton = document.createElement('button'),
                  moreAppsButtonIcon = document.createElement('div');
              
              moreAppsButton.classList.add('more-apps');
              moreAppsButton.appendChild(moreAppsButtonIcon);
              moreAppsButtonIcon.classList.add('icon');
              moreAppsButtonIcon.appendChild(moreAppsButtonImage);
              
              if (settingsObj.trampolineBadge){
                moreAppsButtonBadge.src = settingsObj.trampolineBadge;
                moreAppsButtonBadge.onload = function(){
                  moreAppsButtonBadge.classList.add('badge');
                  moreAppsButton.appendChild(moreAppsButtonBadge);
                };
              }

              _trampolineContainer.querySelector('.main').appendChild(moreAppsButton);
              Trampoline.events.dispatchEvent(MOREAPPS_BUTTON_READY);
            };
          
          },

        /**
         * Adds the more-apps button to the main nav bar
         * @memberof Trampoline.moreApps
         * @private
         */
        _addMoreAppsButton = function(){
          Trampoline._log("Adding More Apps Button...");
          var moreAppsButton = _trampolineContainer.querySelector('button.more-apps');
          _trampolineContainer.classList.add('has-more-apps');
          Trampoline.util.toggleAnimation(moreAppsButton, 'bounceInDown');
          moreAppsButton.style.opacity = 1;
          moreAppsButton.addEventListener(Trampoline.util.tapEvent, function(e){
            e.preventDefault();
            Trampoline.util.toggleAnimation(moreAppsButton, 'tapped', 'tap-exit');
            Trampoline.state.change({to: 'moreApps', from: 'init'});
          });
        },        

        /**
         * Builds an app icon with title
         * @memberof Trampoline.moreApps
         * @private
         * @param   {object} appObject     The app obj from moreapps data
         * @param   {String} appPanelTitle The app title of the panel (for analytics on the link)
         * @param   {String} appPanelType  The panel type (for analytics on the link)
         * @returns {Object} A div container element that can be injected into a panel / carousel
         */
        _buildAppIcon = function (appObject, appPanelTitle, appPanelType){
          var appContainer = document.createElement('li'),
              appIcon = document.createElement('div'),
              appBadge,
              appName = document.createElement('h4'),
              appLabel =  Trampoline.util.underscore(appObject.title),
              appPanelTitle = Trampoline.util.underscore(appPanelTitle);

          appIcon.classList.add('icon');
          //TODO: Use fetch to get images?

          appIcon.style.backgroundImage = (window.devicePixelRatio >= 2) ? 'url('+appObject.appIcon2x+')' : 'url('+appObject.appIcon+')';

          //Badge the icon if it has one
          if (!!appObject.badge){
            appBadge = document.createElement('div');
            appBadge.classList.add('badge');
            appBadge.style.backgroundImage = 'url('+appObject.badge+')';
            appIcon.appendChild(appBadge);
          }
          appContainer.appendChild(appIcon);

          //Parse name because it has \n for breaks
          appName.innerHTML = appObject.title.replace(/\r?\n|\r/g, ' ');
          appContainer.appendChild(appName);

          appContainer.addEventListener('click', function(e){
            //TODO: Update this to get badge.name from new json model (specifically GA)
            Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit');                    
            if (appObject.badge){
              var re = /[^/]+(?=\..{3,4}$)/g;
              appLabel += '_'+re.exec(appObject.badge);
            }
            Trampoline.analytics.event('Selected_in_'+appPanelTitle+'_'+appPanelType, appLabel, 'MoreApps');
            Trampoline.gate.toggleView( function(){
              window.open(appObject.storeUrl, '_system');
            });

          }, false);

          return appContainer;
        },
                
        _buildFeatureButton = function(featureObject, parentPanel){      
          //Image
          var featureUrl = featureObject.storeUrl,
              featureText = featureObject.buttonText,
              panelType = featureObject.type,
              featureButton = document.createElement('img');

          featureButton.setAttribute('src', featureObject.featureGraphic);
          parentPanel.appendChild(featureButton);
        
          //Click
          parentPanel.addEventListener('click', function(e){
            //TODO: Add some UI animation to this.
            //Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit');
            Trampoline.analytics.event('Selected_in_'+panelType, Trampoline.util.underscore(featureText), 'MoreApps');
            Trampoline.gate.toggleView( function(){
              window.open(featureUrl, '_system');
            });
          });
          
          return featureButton;
        },

        /**
         * Expose the feature Data (i.e. for grownups panels in gamesapp)
         * @memberof Trampoline.moreApps
         * @private
         * @param {Obj} featureData The moreapps feature data
         */
        _exposeFeatureData = function (featureData){
          //Expose the feature data
          if (featureData){
            Trampoline.moreApps.feature = featureData;
            Trampoline.moreApps.ready = true;
            Trampoline.events.dispatchEvent(MOREAPPS_FEATURE_READY);
          }
        },
		
        _exposeLargeFeaturesData = function(largeFeatures){
		  Trampoline.moreApps.largeFeatures = largeFeatures;
          Trampoline.events.dispatchEvent(MOREAPPS_LARGE_FEATURES_READY);
        },
        
        /**
         * Injects new styles for the content of the moreApps panel
         * @memberof Trampoline.moreApps
         * @private
         * @param {object} settingsObj The settings obj from the More Apps Data
         */
        _styleMoreApps = function(settingsObj){
          //More Apps Styling from JSON Data
          if (settingsObj.hasOwnProperty('headerColor')){
            Trampoline.util.styleSheet.insertRule('#trampoline section.more-apps nav{ background-color: ' + settingsObj.headerColor + ' !important; }', 1);
          }
          if (settingsObj.panelTitleColor){
            Trampoline.util.styleSheet.insertRule('#trampoline section.more-apps .content-wrapper .content .panel h2 { color: ' + settingsObj.panelTitleColor + ' !important; }', 1);
          }          
          if (settingsObj.backgroundColor){
            Trampoline.util.styleSheet.insertRule('#trampoline section.more-apps .more-apps-wrapper{ background-color: ' + settingsObj.backgroundColor + ' !important; }', 1);
            Trampoline.util.styleSheet.insertRule('#trampoline section.more-apps.active{ background: linear-gradient(to bottom, transparent, transparent 80px, ' + settingsObj.backgroundColor + ' 81px, ' + settingsObj.backgroundColor + ') !important; }', 1);
			Trampoline.util.styleSheet.insertRule('#trampoline section.more-apps.active{ background: -webkit-linear-gradient(to bottom, transparent, transparent 80px, ' + settingsObj.backgroundColor + ' 81px, ' + settingsObj.backgroundColor + ') !important; }', 1);
          }             
        },

        /**
         * Shows the More Apps Panel
         * @memberof Trampoline.moreApps
         * @param {Function} callback Fires callback when shown
         */
        _open = function(callback){
          _moreAppsContainer.classList.add("active");
          Trampoline.analytics.event('MoreApps', 'open');
          Trampoline.events.dispatchEvent(MOREAPPS_OPEN);
          _trampolineContainer.querySelector('.more-apps nav button.close').addEventListener(tapEvent, function(e){
            e.preventDefault();
            Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit'); 
            Trampoline.state.back();
          });
          _panelActive = true;  
          if (typeof callback === 'function'){
            callback();
          }
        },

        /**
         * Hide MoreApps view
         * @private
         * @memberof Trampoline.moreApps
         * @param {Function} callback callback
         */
        _close = function(callback){
          var _scrollContainer = _moreAppsContainer.querySelector('.more-apps-wrapper'),
              _moreAppsScrollTop = _scrollContainer.scrollTop,
              _removeActiveClass = function(){
                _moreAppsContainer.classList.remove("active");
                Trampoline.events.dispatchEvent(MOREAPPS_CLOSE);
                if (typeof callback === 'function'){
                  callback();
                }
                _panelActive = false;
              }
              if (_moreAppsScrollTop <= 0){
                _log('Not scrolling node to top, just removing active');
                _removeActiveClass();
              }else{
                // Dude, I made the scroll time relevant to the scroll distance if < window height (it's capped at 0.15s). How cool is that? 
                // More obvious in portrait mode.
                var _scrollTime = 150,
                    _containerHeight = _scrollContainer.offsetHeight;
                if (_moreAppsScrollTop <= _containerHeight){
                  _scrollTime = _scrollTime * (_moreAppsScrollTop / _containerHeight);
                }
                setTimeout(_removeActiveClass, _scrollTime);
                Trampoline.util.scrollToTop(_moreAppsContainer, _scrollTime);
              }
        },

        /**
         * Toggle More Apps View
         * @memberof Trampoline.moreApps
         * @param {function} callback Provides callback method.
         */
        toggleView = function(callback){
          if (!callback){
            callback = null;
          }
          if (!!_panelActive){
            _close(callback);
          }else{
            _open(callback);
          }
        };
      
    return {
      /**
       * More Apps has connected to endpoint and bugun building
       * @memberof Trampoline.moreApps
       */
      ready         : ready,
      toggleView    : toggleView,
      listResources : listResources,
      startup       : startup,
      init          : init,
      refresh       : refresh,
      feature       : feature,
      largeFeatures : largeFeatures

    };
  })();
            
  return Trampoline;
  
}(Trampoline || {}));
(
  /**
   * Parents View Module for Trampoline 
   * @memberof Trampolone
   * @namespace parents
   * 
   * @event PARENTS_OPEN
   * @event PARENTS_CLOSE
   */
  function(Trampoline){
  Trampoline.parents = (function (){
    var _didInit = false,
		_didBuild = false,
        _trampolineContainer,
        _parentsContainer,
        _parentsContentContainer,
        _panelActive,
        _scrollParent,
        _log = Trampoline._log,    
        _config = null,
        tapEvent = Trampoline.util.tapEvent,

        //Events
        PARENTS_CLOSE  = new Event('PARENTS_CLOSE'),
        PARENTS_OPEN   = new Event('PARENTS_OPEN'),
		
        // Default Template Panels
        _defaultPanels = [
        ],

        /**
         * Initialize Parents Section
         * Adds event triggers to the Parents Container
         * @memberof Trampoline.parents
         */
        init = function( config ){
          _config = config;
          _log('Initiating Parents Section...');
          _trampolineContainer = Trampoline.container || document.getElementById('trampoline');
          _parentsContainer    = _trampolineContainer.querySelector('section.parents');
          _parentsContentContainer = _trampolineContainer.querySelector('.parents-wrapper .content');
          _scrollParent = _parentsContainer.querySelector('.parents-wrapper .content-wrapper');
          
          Trampoline.util.makeSticky(_parentsContainer.querySelector('nav'), _parentsContainer.querySelector('.parents-wrapper') );
          
          _addParentsSection();
                    
          _parentsContainer.addEventListener('touchmove', Trampoline.util.preventDefaultThis);
		  
          if (!!config.grownupsTab) { _addGrownupsTab(); }
          
          //Connect swipe down to close
          Trampoline.util.addSwipePull(_parentsContentContainer, {dY: '-0.10', state:'back', scrollParent: _scrollParent, parentElement: _parentsContainer});
          
          _didInit = true;
        },

        /**
         * Returns template source files of custom and default panels (if configured true)
         * @returns {Object} 
         */
        listResources = function(){
          return Trampoline.templating.listResourcesForConfig( _config, _defaultPanels );
        },

        
        /**
         * Startup runs after init. Modules are run in a specif order through Trampoline core init. 
         * @memberof Trampoline.parents
         * @param {Obj} resources The resource list.
         */
        startup = function( resources ){
		  if( _didBuild ){
			  return;
		  }
		  _didBuild = true;
          // build the DOM elements from the templates now we have the loaded template files.
          var elements = Trampoline.templating.buildTemplates( _config, _defaultPanels, resources );

          // add document fragments to container.
          for( var i = 0; i<elements.length; i++ ){
            _parentsContentContainer.appendChild( elements[i] );
          }
        },
        refresh = function (options){
          if (!_didInit){
            init(options);
          }
          if (options.customPanels && options.customPanels.length > 0){
            for (var i = 0; i<options.customPanels.length; i++){
              if ( options.customPanels[i].hasOwnProperty('refresh') ){
                options.customPanels[i].refresh();
              }
            }
          }
        },

        /**
         * Toggles Parent Bar DOM view with animation
         * @memberof Trampoline.parents
         */
        toggleView = function(){
          if (!!_panelActive){
            _close();
          }else{
            _open();
          }
        },
        _open = function (){
          _panelActive = true;
          _parentsContainer.classList.add('active');
          if (!Trampoline.util.isConnected){
            _parentsContainer.classList.add('offline');
          }else if( _parentsContainer.classList.contains('offline') ){
            _parentsContainer.classList.remove('offline');
          }          
          Trampoline.analytics.event('Parents', 'open');
          Trampoline.events.dispatchEvent(PARENTS_OPEN);
        },
        _close = function (){
          _panelActive = false;
          _parentsContainer.classList.remove('active');
          Trampoline.events.dispatchEvent(PARENTS_CLOSE);
        },
        _addParentsSection = function(){
          _trampolineContainer.classList.add('has-parents');
          _trampolineContainer.querySelector('.parents nav .close').addEventListener(tapEvent, function(e){
            e.preventDefault();
            Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit');
            Trampoline.state.back();
          });
        },
        _addGrownupsTab = function(){
          var grownupsTabMarkup = document.createElement('div');
          grownupsTabMarkup.classList.add('parents-pull-tab');
          grownupsTabMarkup.innerHTML = '<h1>Grownups</h1>';
          
          _parentsContainer.insertBefore(grownupsTabMarkup, _parentsContainer.querySelector('.parents-wrapper') );
          
          Trampoline.util.addSwipePull(grownupsTabMarkup, {dY: '0.20', state: 'parents', parentElement: _parentsContainer});
        },
        connectionLost = function (){
          if (_panelActive && !Trampoline.util.isConnected){
            _parentsContainer.classList.add('offline');
          }
        },
        connectionResume = function (){
          if (_parentsContainer.classList.contains('offline')){
            _parentsContainer.classList.remove('offline');
          }
        },
        /**
         * Removes Parents Area from DOM
         */
        remove = function (){
          if (!_parentsContainer){
            _parentsContainer = document.getElementById('trampoline').querySelector('section.parents');
          }
          _parentsContainer.parentElement.removeChild(_parentsContainer);
        };

    return {
      init              : init,
      startup           : startup,
      refresh           : refresh,
      listResources     : listResources,
      toggleView        : toggleView,
      connectionResume  : connectionResume,
      connectionLost    : connectionLost,
      remove            : remove
    };
  })();
            
  return Trampoline;
  
}(Trampoline || {}));

(
/**
 * Popups View Module for Trampoline
 * @memberof Trampolone
 * @namespace parents
 *
 * @event POPUPS_OPEN
 * @event POPUPS_CLOSE
 */
  function(Trampoline){
    Trampoline.popups = (function (){
      var _trampolineContainer,
        _popupsContainer,
        _popupsContentContainer,
        _popupsById = {},
        _currentPopupId,
        _currentPoupupElement,
        _defaultPopupId,
        _panelActive,
        _log = Trampoline._log,
        _config = null,

      //Events
        POPUPS_CLOSE  = new Event('POPUPS_CLOSE'),
        POPUPS_OPEN   = new Event('POPUPS_OPEN'),

      // Default Template Panels
        _defaultPanels = [
        ],

        /**
         * Initialize Popups section.
         * Adds event triggers to the Parents Container
         * @memberof Trampoline.parents
         */
        init = function( config ){
          _config = config;
          _log('Initiating Popups Section...', config );
          _trampolineContainer = Trampoline.container || document.getElementById('trampoline');
          _popupsContainer    = _trampolineContainer.querySelector('section.popups');
          _popupsContentContainer = _trampolineContainer.querySelector('.popups-wrapper .content');
          _addPopupsSection();
        },

        listResources = function(){
          return Trampoline.templating.listResourcesForConfig( _config, _defaultPanels );
        },

        startup = function( resources ){

          // build the DOM elements from the templates now we have the loaded template files.
          var elements = Trampoline.templating.buildTemplates( _config, _defaultPanels, resources );

          if( elements.length === 0 ){
            return;
          }

          // don't add all popups immediately.
          // build a map of all popups by id and wait for a popup open request
          // to happen.
          var frag,ele;
          for( var i = 0; i<elements.length; i++ ){
            frag = elements[i];
            ele = frag.firstChild;
            _popupsById[ ele.id ] = ele;
          }

          _defaultPopupId = elements[0].firstChild.id;

        },

        /**
         * Toggles Parent Bar DOM view with animation
         * @memberof Trampoline.parents
         */
        toggleView = function(){
          if (!!_panelActive){
            _close();
          }else{
            _open();
          }
        },

        _open = function( popupId ){

          var id = popupId || _defaultPopupId;
          var popup = _popupsById[ id ];

          if( !popup ){
            return;
          }

          _popupsContainer.classList.add( 'active' );

          if( _currentPoupupElement ){
            _popupsContentContainer.removeChild( _currentPoupupElement );
          }

          _currentPopupId = id;
          _currentPoupupElement = popup;

          _popupsContentContainer.appendChild( _currentPoupupElement );

          _panelActive = true;
          //_popupsContainer.classList.add('active');
          //Trampoline.analytics.event('Parents', 'open');
          Trampoline.events.dispatchEvent(POPUPS_OPEN, _currentPopupId, _currentPoupupElement );
        },

        _close = function(){

          if( _currentPoupupElement ){
            _popupsContentContainer.removeChild( _currentPoupupElement );
          }

          _popupsContainer.classList.remove( 'active' );

          var ele = _currentPoupupElement;
          var id = _currentPopupId;

          _currentPoupupElement = null;
          _currentPopupId = null;

          _panelActive = false;
          //_popupsContainer.classList.remove('active');
          Trampoline.events.dispatchEvent(POPUPS_CLOSE, id, ele );
        },
        _addPopupsSection = function(){
          //_trampolineContainer.classList.add('has-parents');
          //_trampolineContainer.querySelector('.parents nav .close').addEventListener('click', function(e){
          //  e.preventDefault();
          //  _close();
          //});
        },
        /**
         * Removes Parents Area from DOM
         */
        remove = function (){
          if (!_popupsContainer){
            _popupsContainer = document.getElementById('trampoline').querySelector('section.popups');
          }
          _popupsContainer.parentElement.removeChild(_popupsContainer);
        };

      return {
        init          : init,
        startup       : startup,
        listResources : listResources,
        toggleView    : toggleView,
        remove        : remove,
        open          : _open,
        close         : _close
      };
    })();

    return Trampoline;

  }(Trampoline || {}));
(
  /**
   * Settings View Module for Trampoline
   * @memberof Trampoline
   * @namespace settings
   */
  function(Trampoline){
  Trampoline.settings = (function (){
    var _didInit = false,
        _trampolineContainer,
        _settingsButton,
        _settingsModule,
        _settingsContainer,
        _panelActive,
        _scrollParent,
        _log = Trampoline._log,
        tapEvent = Trampoline.util.tapEvent,
        
        //Events
        SETTINGS_CLOSE  = new Event('SETTINGS_CLOSE'),
        SETTINGS_OPEN   = new Event('SETTINGS_OPEN'),
		LOCALIZED   = new Event('LOCALIZED'),
                
        /**
         * Initialize Settings DOM element
         * @memberof Trampoline.settings
         */
        init  =  function(config){
          _trampolineContainer = Trampoline.container || document.getElementById('trampoline');
          _settingsButton      = document.querySelector('.settings.button');
          _settingsModule      = document.querySelector('#settings-module');
          _settingsContainer   = _trampolineContainer.querySelector('section.settings');
          _scrollParent        = _settingsContainer.querySelector('.content-wrapper');

		  if (config == false){
			  return;
		  }else{
			  var _localStation,
              _autoLocalStation = JSON.parse(Trampoline.util.getLocalStorage('trampolineAutoLocalStation'));
          
			  try{ _localStation = JSON.parse(Trampoline.util.getLocalStorage('trampolineLocalStation'));
			  }catch(e){ _log(e); }

			  Trampoline.util.makeSticky(_settingsContainer.querySelector('nav'), _settingsContainer.querySelector('.settings-wrapper'));
			  _log('init settings');
			  _addSettings();

			  _settingsContainer.addEventListener('touchmove', Trampoline.util.preventDefaultThis);

			  if (!!_localStation){
				_log('Local Station is in Local Storage, set it and forget it!')
				Trampoline.localize.getStationImage(_localStation, 'color-logo', stationPicker.setStationLogo);
			  }else if(!!_autoLocalStation){
				Trampoline.localize.getStationImage(_autoLocalStation, 'color-logo', stationPicker.setStationLogo);
			  }
			  stationPicker.init();
            
              Trampoline.util.addSwipePull(_settingsContainer, {dY: -0.10, state: 'back', scrollParent: _scrollParent});
            
			  _didInit = true;
		  }
        },
        refresh = function(options){
          if (!_didInit){
            init(options);
          }else{
            //Something bonked. Let's reinit the station picker
            stationPicker.init();
          }
        },
        /**
         * Add a module to the settings view
         * @param {object} settingsObj TBD
         * @memberof Trampoline.settings
         */
        addSettingsModule = function (settingsObj){
          //TODO: Add settings to module from settingsObject.
          _log('add setting');
          if (typeof settingsObj === 'object'){
             
          }else{
            _log('error in settings object');
          }
        },
        stationPicker = {
          //TODO: if there is no station data (network exists, but connection error) we need show that things got messed up
          init : function (){
            if (Trampoline.util.isConnected){
              _log('Building Station Picker', ': connected');              
              Trampoline.events.addEventListener('STATION_LIST_POPULATED', stationPicker.build);

              if (!Trampoline.localize.didLocalize || !Trampoline.localize.stationList[0]){
                _log('No data for Station Picker, attempting auto localization'); 
                Trampoline.localize.auto(); 
              }else{
                stationPicker.build();
              }
              
              //document.querySelector('.zipcode-input').addEventListener(tapEvent, stationPicker.showZipcodeEntry);
              
              didInit = true;
            }else{
              // Error, No internet connection fallbacks
              _log('Trying to build Station Picker', ': network error');
              connectionLost();
            }
            //Set to non-localized view. Try to auto localize again?          
          },
          build : function(){
            var _stationList = Trampoline.localize.stationList;
            _log('Localized, Building Station Picker', _stationList);
            stationPicker.updateStationList();
            
          },
          submitZipcode : function(e){
            var zipcodeValue = document.querySelector('.zipcode-input').value;
            
            Trampoline.util.toggleAnimation(e.target, 'tap-pulse-button');
                        
            if (zipcodeValue == null || zipcodeValue == "" || zipcodeValue.toString().length < 5 || isNaN(zipcodeValue) ) {
                //TODO: Improve Zipcode UX
                return false;
            }else{
              Trampoline.localize.zipcode(zipcodeValue, null, stationPicker.inputError);
            }
          },
          
          updateStationList : function(){
            var _stationList = Trampoline.localize.stationList,
                _stationsButtonContainer = _settingsContainer.querySelector('.station-picker ul.station-buttons'),
                _localStationFromStorage = JSON.parse(Trampoline.util.getLocalStorage('trampolineLocalStation'));
            
            _stationsButtonContainer.innerHTML = '';

            for (var i = 0; i < Object.keys(Trampoline.localize.stationList).length; i++){
              var stationCallsign = Object.keys(Trampoline.localize.stationList)[i],
                  stationData = Trampoline.localize.stationList[stationCallsign],
                  stationButton = document.createElement('li'),
                  stationName = document.createTextNode(stationData.full_common_name),
                  stationImage = new Image();
              
              if ((!!_localStationFromStorage) && (_localStationFromStorage.callsign == stationCallsign)){
                stationButton.classList.add('selected');   
              }
                  
              stationButton.dataset.callsign = stationCallsign;
              //TODO: make tap event on station list global/general
              stationButton.addEventListener(tapEvent, function(e){
                e.preventDefault();
                if ( !e.target.classList.contains('selected') ){              
                  var i,
                      stationListItems = _stationsButtonContainer.querySelectorAll('li');
                                    
                  //Persist the Localstation 
                  stationPicker.updateLocalStation(e.target.dataset.callsign);
//                  Trampoline.localize.localStation = Trampoline.localize.stationList[e.target.dataset.callsign];
//                  Trampoline.localize.localStation.callsign = e.target.dataset.callsign;
//                  Trampoline.util.setLocalStorage('trampolineLocalStation', JSON.stringify(Trampoline.localize.localStation) );                  
                  
                  if (!!Trampoline.localize.didLocalize){
                    Trampoline.analytics.event('Settings', Trampoline.localize.didLocalize);
                    _log('Settings, ' + Trampoline.localize.didLocalize);
                  }
                  
                  Trampoline.util.toggleAnimation(e.target, 'tap-pulse');
                  for (i = 0; i < stationListItems.length; i++){
                    if (stationListItems[i].classList.contains('selected') ){
                      stationListItems[i].classList.remove('selected');
                    }
                  }
                  e.target.classList.add('selected');
                }
              });
              
              //TODO: add .selected to station if it's also set as the Selected Station

              stationImage.src = stationData.images['color-logo'];
              stationButton.appendChild(stationImage);
              stationButton.appendChild(stationName);
              _stationsButtonContainer.appendChild(stationButton);
            }
            _log('station list buttons updated');
          },
          inputError : function(){
            stationPicker.clearStationButtons();
            var _stationsButtonContainer = _settingsContainer.querySelector("ul.station-buttons");
            _stationsButtonContainer.innerHTML = '<li class="no-data"><h2>Oops!</h2>No Local PBS Station</br>found for that zip code</li>';
            Trampoline.util.toggleAnimation(_stationsButtonContainer, 'pulse');
            Trampoline.container.querySelector('section.settings input.zipcode-input').value = null;
          },
          updateLocalStation : function(callsign){
            _log('update local station', callsign);
                      
            var _localStation = Trampoline.localize.stationList[callsign];
            Trampoline.localize.getStationImage(_localStation, 'color-logo', stationPicker.setStationLogo);
//            Trampoline.util.setLocalStorage('trampolineLocalStation', _localStation);
//
//            Trampoline.localize.localStation = _localStation;
//            Trampoline.events.dispatchEvent(LOCALIZED);
            Trampoline.localize.setLocalStation(callsign);
          },
          clearStationButtons : function(){
            var _stationsButtonContainer = _settingsContainer.querySelector("ul.station-buttons");
            //stationsButtons.parentNode.removeChild(stationsButtons);
            _stationsButtonContainer.innerHTML = '';
          },
          setStationLogo : function(stationLogoObj){
            var _localStationImg = _settingsContainer.querySelector('.station-picker .selected-station img.station-logo');
            stationLogoObj.classList.add('station-logo', 'localized');
            Trampoline.util.toggleAnimation(stationLogoObj, 'pulse');

            _localStationImg = _localStationImg.parentNode.replaceChild(stationLogoObj, _localStationImg);
          },
          showZipcodeEntry : function(e){
            var zipcodeDisplay = e.target;
            _settingsContainer.classList.add('zipcode-entry');
          }
        },
        _addSettings = function (){
          _trampolineContainer.classList.add('has-settings');
          _trampolineContainer.querySelector('.settings nav .close').addEventListener(tapEvent, function(e){
            e.preventDefault();
            Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit');
            Trampoline.state.back();
          });
          //Add a real thing to the zipcode button
          _settingsContainer.querySelector('.zipcode-submit').addEventListener(tapEvent, stationPicker.submitZipcode);
        },
        connectionLost = function (){
          if (_panelActive && !Trampoline.util.isConnected){
            _settingsContainer.classList.add('offline');
          }
        },
        connectionResume = function (){
          if (_settingsContainer.classList.contains('offline')){
            _settingsContainer.classList.remove('offline');
          }
        },
        /**
         * Removes the Settings DOM element
         * @memberof Trampoline.settings
         */
        remove = function (){
          _log('config' , 'remove settings module');
          if (!!_settingsModule){
            _settingsModule.parentElement.removeChild(_settingsModule);        	
          }
        },
        /**
         * Toggles the Settings View with animation
         * @memberof Trampoline.settings
         */
        toggleView = function (){
          if (!!_panelActive){
            _close();
          }else{
            _open();
          }
        },
        _open = function (){
          _panelActive = true;
          _settingsContainer.classList.add('active');
          if (!Trampoline.util.isConnected){
            _settingsContainer.classList.add('offline');
          }else if( _settingsContainer.classList.contains('offline') ){
            _settingsContainer.classList.remove('offline');
          }
          Trampoline.analytics.event('Settings', 'open');
          Trampoline.events.dispatchEvent(SETTINGS_OPEN);
        },
        _close = function (){
          _panelActive = false;
          _settingsContainer.classList.remove('active');
          _settingsContainer.scrollTop = 0;
          Trampoline.events.dispatchEvent(SETTINGS_CLOSE);
        },
        //Relics from the original setting modules
        _openSettingsModule = function (){
          _log('show settings module');
          _settingsButton.removeEventListener(tapEvent, function(e){
            e.preventDefault();
            _openSettingsModule();
          });
          _settingsButton.addEventListener(tapEvent, function(e){
            e.preventDefault();
            _closeSettingsModule();
          });
          _settingsModule.classList.add('active');
        },
        _closeSettingsModule = function (){
          _settingsButton.removeEventListener(tapEvent, function(e){
            e.preventDefault();
            _closeSettingsModule();
          });
          _settingsButton.addEventListener(tapEvent, function(e){
            e.preventDefault();
            _openSettingsModule();
          });
          _settingsModule.classList.remove('active');
        };
      
    return {
      init              : init,
      refresh           : refresh,
      addSettingsModule : addSettingsModule,
      remove            : remove,
      toggleView        : toggleView,
      connectionLost    : connectionLost,
      connectionResume  : connectionResume
    };
  })();
            
  return Trampoline;
  
}(Trampoline || {}));
(
  /**
   * Splash Screen Module for Trampoline 
   * @memberof Trampoline
   * @namespace localize
   */
  function(Trampoline){
  Trampoline.splash = (function (){
    var _trampolineContainer,
        _splashContainer,
        _stationLogo,
        _isLocalized = false,
        _splashActive = false,
        _log = Trampoline._log,
        _localizeTimer,
        _splashSpinner,
        
        config = {},
        
        //EVENTS
        SPLASH_HIDE = new Event('SPLASH_HIDE'),
        SPLASH_DONE = new Event('SPLASH_DONE'),
        
        
        /**
         * Initializes the Splash screen, attempts to localize, fades out.
         * @memberof Trampoline.splash
         */
        init = function(){
          _trampolineContainer = Trampoline.container || document.getElementById('trampoline');
          _splashContainer     = _trampolineContainer.querySelector('.splash');
          _stationLogo         =  _splashContainer.querySelector('.station-logo');
                        
          _splashContainer.classList.remove('hidden');
          _splashActive = true;
        
          
          _splashContainer.addEventListener('touchmove', Trampoline.util.preventDefaultThis);
          
          // Remove Cordova Splash Screen if exists, (short timeout prevents flicker?)
          if ( navigator.hasOwnProperty('splashscreen') ){
//            if (Trampoline.util.isiOS){
//              setTimeout(function(){
//                navigator.splashscreen.hide();
//              }, 250 );              
//            }else{
                navigator.splashscreen.hide();
//            }
          }
          
          //reset the HTML bgcolor which has been set by css to suppress a white flash when the webview is loading
          document.body.style.backgroundColor = 'initial';
          
          //Don't localize if config `splash: {localize: false}`
          if ( Trampoline.splash.config.hasOwnProperty('localize') && (Trampoline.splash.config.localize == false) ){
            setTimeout(function(){
            	_log('Trampoline config set to NOT localize splash.');
				_splashDone();
            }, 2500);
          }else if (Trampoline.util.isConnected){
            _localizeSplashScreen();
          }else{
            setTimeout(function(){
            _log('Trampoline is not connected, cannot localize station.');
              if (!_isLocalized){
                _splashDone();
              }
            }, 2500);
            //Check to see if there is a persistent localStation data.
            //TODO: Write this once localStorage Images are available
            
            //stationLogo = _getPersistentStationImage();
            //Otherwise, The splash screen is over, give the kids logo a little time to read before calling _splashDone
          }
        },
        _getPersistentStationImage = function(){
          var localStation = Trampoline.util.getLocalStorage('trampolineLocalStation');
          
          if (localStation && localStation.images['white-logo']){
            var localStationImage = localStation.images['white-logo'];
            return localStationImage;
          }else{
            return null;
          }
        },
        /**
         * TODO: Check for persistent localized data before auto-localizing
         * Starts timer, (checks for local storage, if none,) attempts to autolocalize.
         * Shows station logo if StationLogo is returned before timeout.
         */
        _localizeSplashScreen = function(){
          var _localStation;
          
          //check for persistent localization data
          try{
            _localStation = JSON.parse(Trampoline.util.getLocalStorage('trampolineLocalStation'));
          }catch(e){
            _log(e);
          }
          
          _startLocalizeTimer(2500);
          
          //TODO: Clean this up... things got weird merging 1.1 into 1.2.
          function _handleLocalizedListener(e){
            if (!_isLocalized && !!Object.keys(Trampoline.localize.stationList)[0]){
              Trampoline.localize.getStationImage(Trampoline.localize.stationList[Object.keys(Trampoline.localize.stationList)[0]],'white-logo', _showStationLogo, 550);
            }
            e.target.removeEventListener('LOCALIZED', _handleLocalizedListener);
          }
          
          Trampoline.events.addEventListener('LOCALIZED', _handleLocalizedListener);

          if (!!_localStation){
            _log('Localizing from local storage');
            Trampoline.localize.getStationImage(_localStation, 'white-logo', _showStationLogo, 550);
          }else if( Trampoline._localize.auto ){
            Trampoline.localize.auto(_showStationLogo);
          }else if( Trampoline._localize.zipCode ){
            Trampoline.localize.zipcode(Trampoline._localize.zipCode,_showStationLogo, Trampoline.analytics.dimension(1, 'NO_STATION'));
          }

        },
        /**
         * Simple UX timer for splash data retrieval. 
         * @param {Number} time Time in MS
         */
        _startLocalizeTimer = function(time){
          _localizeTimer = setTimeout(function(){ 
            if (!_isLocalized){
              _log('Splash screen localization time out');
              _splashDone();
            }
          }, time);
        },
        /**
         * Append station logo as DOM Node and animate PBS Kids Logo
         * @param {Object} stationLogoObj Image DOM node
         */
        _showStationLogo = function(stationLogoObj){
          //We have a station logo! Set it and forget it.
          _isLocalized = true;
          clearTimeout(_localizeTimer);
          
          //Just a few fallbacks on the callsign...
          var _callsign = ((typeof stationLogoObj.dataset.callsign !== "undefined") ? stationLogoObj.dataset.callsign : 0) || ((typeof Trampoline.localize.localStation.callsign !== "undefined") ? Trampoline.localize.localStation.callsign : 0) || ((typeof Object.keys(Trampoline.localize.stationList)[0] !== "undefined") ? Object.keys(Trampoline.localize.stationList)[0] : 0) || false;
          
          if (!!_callsign){          
            //We have a definite station, set the dimension (this can change again through the station-picker).
            Trampoline.analytics.dimension(1, _callsign);
          }
          
          //Set the station logo.
          _stationLogo.appendChild(stationLogoObj);
          _splashContainer.classList.add('localized');
          setTimeout(function(){
            _splashDone();
          }, 3000);
        },
        /**
         * Whenever the splash screen is finished, regardless of connection state, we want to run the same block
         * So that if the developer wants to hang, while they load the rest of the joint, we can do that.
         * This also triggers the Trampoline.onReady
         * 
         * @event SPLASH_DONE The splash screen is done, if splash.hide = true we will stall with a loader until state is changed to 'main'
         */
        _splashDone = function(){          
          if ( Trampoline.splash.config.hasOwnProperty('hide') && (Trampoline.splash.config.hide == false) ){
            _log('Splash screen auto-hide set to false, waiting for Trampoline.state.change...');
            
            //Inject the loading spinner (previously part of markup)
            _splashSpinner = document.createElement('div');
            _splashSpinner.classList.add('splash-spinner');
            _splashSpinner.innerHTML = '<div class="splash-loader-text">Loading</div>';   
            _splashContainer.querySelector('.container').appendChild(_splashSpinner);
            
            Trampoline.events.dispatchEvent(SPLASH_DONE);
            Trampoline.util.toggleAnimation(_splashSpinner, 'pulse-in');
            _splashSpinner.classList.add('active');
          }else{
            Trampoline.events.dispatchEvent(SPLASH_DONE);
            Trampoline.state.change({to: 'main', from:'splash'});
          }
          Trampoline.onReady();
        },
        /**
         * Hides the Splash screen once transition has ended, if there is one.
         * @param {Function} callback Callback, no return
         */
        _hide = function(callback){
          _splashContainer.addEventListener('transitionend', function(){
            if (_splashActive){
              Trampoline.events.dispatchEvent(SPLASH_HIDE);
              remove();
              document.body.style.backgroundColor = 'initial';
              _splashActive = false;
            }
          });
          _splashContainer.classList.remove("active");
          if (typeof callback === 'function'){
            callback();
          }
        },
        
        remove = function(){
          if (!_splashContainer){
            _splashContainer = document.getElementById('trampoline').querySelector('.splash');
          }
          document.body.style.backgroundColor = 'initial';
          _splashContainer.parentElement.removeChild(_splashContainer);
        },
        toggleView = function(){
          if (!_splashActive){
            init();
          }else{
            if (!!_splashSpinner){
              Trampoline.util.toggleAnimation(_splashSpinner, 'pulse-out', 'hidden');
              window.setTimeout(function(){
                _hide();
              //slightly shorter than animation
              }, 200);
            }else{
              _hide();
            }
          }
          
        };
      
    return {
      config  : config,
      remove  : remove,
      toggleView : toggleView
    };
  })();
            
  return Trampoline;
  
}(Trampoline || {}));
(
  /**
   * PBS Kids Super Vision Module for Trampoline 
   * @memberof Trampoline
   * @namespace supervision
   */
  function ( Trampoline ) {
  Trampoline.supervision = (function (){
    var _trampolineContainer,
        _supervisionButton,
        _supervisionContainer,
        _syncCodeContainer,
        _playTimerContainer,
        _connectingContainer,
        _log = Trampoline._log,
        _panelActive = false,
        _fayeClient = null,
        _currentSessionId = null,
        _currentPlayTimerTimeout = null,
        _closePanelTimeout = null,
        _isPaired = false,
        _previousSubscription = null,
        _options = {},
        tapEvent = Trampoline.util.tapEvent,
        sync_state = null,
        
        //EVENTS
        SUPER_VISION_OPEN          = new Event('SUPER_VISION_OPEN'),
        SUPER_VISION_CLOSE         = new Event('SUPER_VISION_CLOSE'),
        SUPER_VISION_PAIR          = new Event('SUPER_VISION_PAIR'),
        SUPER_VISION_UNPAIR        = new Event('SUPER_VISION_UNPAIR'),
        PLAYTIMER_SHOW             = new Event('PLAYTIMER_SHOW'),
        PLAYTIMER_HIDE             = new Event('PLAYTIMER_HIDE'),

        // Environment configuration
        //TODO: Remove these other devs for Prod release.
        _environmentConfiguration = {
          dev: {
            faye_root: 'http://super-vision-dev.pbskids.org:9000/faye',
            broker_url: 'http://super-vision-dev.pbskids.org:9000'
          },
          chip: {
            faye_root: 'http://chip.pbskids.org:8080/faye',
            broker_url: 'http://chip.pbskids.org:8080'
          },
          stage: {
            faye_root: 'http://sv-broker-stg.pbskids.org:9100/faye',
            broker_url: 'http://sv-broker-stg.pbskids.org:9100',
          },
          production_v3: {
            faye_root: 'https://supervision.pbskids.org:9300/faye',
            broker_url: 'https://supervision.pbskids.org:9300'
          }
        },
        
        /**
         * Initialize Supervision Module
         * @memberof Trampoline.supervision
         * @param {Object|null} options super vision init options
         */
        init = function (options) {
          _log('Supervision init...');

          _injectPlaytimer();
          
          // provide some default options
          _options = options || {};
          _options.origin = options.origin || 'org.pbskids.WebSite';
          _options.environment = options.environment || 'dev';

          // A few DOM Elements we need
          _trampolineContainer   = Trampoline.container || document.getElementById('trampoline');
          _supervisionButton     = _trampolineContainer.querySelector('nav .button.supervision');
          _supervisionContainer  = _trampolineContainer.querySelector('section.supervision');
          _connectingContainer   = _supervisionContainer.querySelector('.connecting-wrapper');
          _syncCodeContainer     = _supervisionContainer.querySelector('.sync-code');

          _playTimerContainer    = _trampolineContainer.querySelector('section.play-timer');
          
          _supervisionContainer.addEventListener('touchmove', Trampoline.util.preventDefaultThis);
          
          // Inject the Onboarding SVGs so we can manipulate them with CSS, and leave them out of the markup.
          var svgPath = (!!Trampoline._resourcePath) ? Trampoline._resourcePath : '';
          
          Trampoline.util.injectSVG(_supervisionContainer.querySelector('.tablet-user'), svgPath +'img/supervision/tablet-user.svg');
          Trampoline.util.injectSVG(_supervisionContainer.querySelector('.mobile-hand'), svgPath +'img/supervision/mobile-hand.svg');
                                            
          // Hook in some events
          _supervisionContainer.querySelector('nav .close').addEventListener(tapEvent, function(e){
            e.preventDefault();
            Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit');
            Trampoline.state.back();
          });
          
          Trampoline.util.addSwipePull(_supervisionContainer, {dY: -0.10, state: 'back', scrollParent : _connectingContainer});
          
          var connectingPagination = _supervisionContainer.querySelectorAll('.dot');
          for (var i = 0; i < connectingPagination.length; i++ ){
            connectingPagination[i].addEventListener(tapEvent, _setOnboardingScreen);
          }
          
          _supervisionContainer.querySelector('.disconnect').addEventListener(tapEvent, function(e){
            e.preventDefault();
            _showDisconnectDialogue();
          });
          _supervisionContainer.querySelector('.okay').addEventListener(tapEvent, function(e){
            e.preventDefault();
            //Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit');
            _hideDisconnectDialogue();
            disconnect();
          });
          //TODO: add tap animations that don't suck
          _supervisionContainer.querySelector('.cancel').addEventListener(tapEvent, function(e){
            e.preventDefault();
            //Trampoline.util.toggleAnimation(e.target, 'tap-exit', 'tap-exit');
            _hideDisconnectDialogue();
          });   
          _supervisionContainer.querySelector('.done').addEventListener(tapEvent, function(e){
            e.preventDefault();
            Trampoline.util.toggleAnimation(e.target, 'tapped', 'tap-exit');
            window.setTimeout(function(){
              Trampoline.state.back();
            }, 350);
            
          });
          
          function _showSupervisionNotifier(){
            var _supervisionNotifier = _trampolineContainer.querySelector('.main #supervision-notifier');
            if (!!_supervisionNotifier){
              window.setTimeout(function(){
                Trampoline.util.toggleAnimation(_supervisionNotifier, 'bounceDown', null);
                _supervisionNotifier.classList.add('active');
                _supervisionNotifier.querySelector('.notifier-container').addEventListener(Trampoline.util.tapEvent, function(){
                  clearTimeout(); 
                  Trampoline.util.toggleAnimation(_supervisionNotifier, 'tapped');
                  Trampoline.util.toggleAnimation(_supervisionNotifier, 'bounceDownReturn');
                  window.setTimeout(function(){
                    _supervisionNotifier.classList.remove('active');
                  }, 600);
                  Trampoline.state.change({to: 'supervision', from: 'main'});
                });
                window.setTimeout(function(){
                  Trampoline.util.toggleAnimation(_supervisionNotifier, 'bounceDownReturn');
                  window.setTimeout(function(){
                    _supervisionNotifier.classList.remove('active');
                  }, 600);
                }, 30000);
                // 5 Second delay on notifier
              }, 2000);
            }
          }
          
          function _addSupervisionNotifierOnMain(e){
              if (Trampoline.state.current.name == 'main'){
                e.target.removeEventListener('TRAMPOLINE_STATE_CHANGE', _addSupervisionNotifierOnMain);
                _showSupervisionNotifier();
              }
          }
          
          // Show the tooltip on state.main
          if (Trampoline.state.current.hasOwnProperty('name') && (Trampoline.state.current.name == 'main')){
            _showSupervisionNotifier();
          }else{
            Trampoline.events.addEventListener('TRAMPOLINE_STATE_CHANGE', _addSupervisionNotifierOnMain);
          }
          
          //Connect swipe events to Onboarding Screens
          _connectingContainer.addEventListener('touchstart', Trampoline.util.handleSwipeTouchStart, false);
          Trampoline.events.addEventListener('TRAMPOLINE_SWIPE_RIGHT', _regressOnboardingScreen);
          Trampoline.events.addEventListener('TRAMPOLINE_SWIPE_LEFT', _advanceOnboardingScreen);
          
          //Cancel Event Bubbling through Play-Timer screens
          if (!!_playTimerContainer){
            _playTimerContainer.addEventListener(tapEvent, function(e){ e.preventDefault(); });
          }

          // Whenever the app closes, hide the super vision panel, so we don't get stale sync codes
          document.addEventListener('pause', _closePanel);
          document.addEventListener('resign', _closePanel); // ios specific 'pause' event: http://cordova.apache.org/docs/en/5.4.0/cordova/events/events.pause.html#ios-quirks
          
          // Create the initial connection
          reconnectToExistingChannel();
        },
        
        connectionResume = function (){
          if (_supervisionContainer.classList.contains('offline')){
            _supervisionContainer.classList.remove('offline');
          }
          if(!_isPaired) {
            //Make sure the UI state is reset
            _syncCodeContainer.dataset.status = 'getting-code';
            connectToNewChannel();

            // Set the panel to close after inactivity. That way, the parent doesn't accidentally try to sync with an expired code
            window.clearTimeout(_closePanelTimeout);
            _closePanelTimeout = window.setTimeout(_closePanel, 4 * 60 * 1000);
          }
        },
        
        connectionLost = function () {
          _log('supervision connection lost'); 
          if (_panelActive){
            _supervisionContainer.classList.add('offline');
          }
        },

        _closePanel = function () {
          if(_panelActive) {
            Trampoline.state.back();
          }
        },

        _setOnboardingScreen = function (e){
          var currentScreen = parseInt(_connectingContainer.dataset.screen),
              toScreen = e.target.dataset.screen;
          if ( toScreen != currentScreen && toScreen > 0 && toScreen <= 3 ){
            e.preventDefault();
            _log('setting onboarding screen to: ' + toScreen);
            _connectingContainer.dataset.screen = toScreen.toString();
          }
        },

        _advanceOnboardingScreen = function (e){
          var currentScreen = parseInt(_connectingContainer.dataset.screen),
              wasChildNodeEvent = _supervisionContainer.contains(e.detail.touchEndEvent.target);
          
          if (wasChildNodeEvent && (currentScreen < 3)){
            currentScreen++;
            _connectingContainer.dataset.screen = currentScreen.toString();
          }
        },
        
        _regressOnboardingScreen = function (e){
          var currentScreen = parseInt(_connectingContainer.dataset.screen),
              wasChildNodeEvent = _supervisionContainer.contains(e.detail.touchEndEvent.target);
          
          if (wasChildNodeEvent && (currentScreen > 1)){
            currentScreen--;
            _connectingContainer.dataset.screen = currentScreen.toString();
          }
        },
        
        
        _injectPlaytimer = function(){
          var _playtimerContainer = document.createElement('div'),
              _onSuccess = function (playtimerMarkup){
                _playtimerContainer.setAttribute('id', 'trampoline-play-timer');
                _playtimerContainer.insertAdjacentHTML('beforeend', playtimerMarkup);
                document.body.appendChild(_playtimerContainer);
                _log('...Playtimer injected into body!');
              },
              _onFail = function( err ){
                _log('...Playtimer failed to inject! :(' , err);
              };
          
          Trampoline.util.fetch(Trampoline._path + 'play-timer.html', _onSuccess, _onFail);
        },
        
        _showDisconnectDialogue = function(){
          _supervisionContainer.querySelector('.disconnect-alert').classList.add('active');
        },
        
        _hideDisconnectDialogue = function(){
          var _disconnectContainer = _supervisionContainer.querySelector('.disconnect-alert');
          Trampoline.util.toggleAnimation(_disconnectContainer, 'bounceDownReturn');
          _disconnectContainer.classList.remove('active');
        },
        
        /**
         * Retrieve a Super Vision sync code form the Faye client
         * @memberof Trampoline.supervision
         * @param {function} onSuccess Success callback
         */
        getSyncCode = function(onSuccess, numTries){
          var syncCodeUrl = _environmentConfiguration[_options.environment].broker_url + '/channel/create',
              ajaxOptions = {
                method: 'POST',
                data: { origin: _options.origin }
              },
                            
              _fetchSyncCode = function(numFetchTries) {            
                // if we run out of tries, we'll need to do something...                
                var _fetchSyncCodeSuccess = onSuccess || function() { },

                    _fetchSyncCodeFail = function(numFetchTries) {
                      _log('Sync code failed. ' + (numFetchTries - 1) + ' tries left');
                      setTimeout(function() { _fetchSyncCodeSuccess(numFetchTries - 1); }, 2000);

                    };
                
                //TODO: Add in timeout for 'try-again' and attach a tap event handler to reconnect.  
                if(numTries <= 0) {
                  _syncCodeContainer.dataset.status = 'try-later';
                  _clearSyncCode();
                  return;
                }
                
                // If the fetch call fails, try again in a second or two, until we have no more tries left
                Trampoline.util.fetch(syncCodeUrl, onSuccess, _fetchSyncCodeFail, ajaxOptions);
              };
            
          //Try to get a sync code (5) times
          _fetchSyncCode(numTries);
        },
        
        _clearSyncCode = function(){
          var _syncCodeCharacters = _syncCodeContainer.querySelectorAll('.character');
          
          for (var i = 0; i < _syncCodeCharacters.length; i++) {
            _syncCodeCharacters[i].innerHTML = '';
          }
        },
        

        /**
         * Sets up the faye client, using a singleton pattern so we don't get duplicate events
         * @memberof Trampoline.supervision
         * @return {Faye.Client} A Faye client instance
         */
        initializeFayeClient = function() {
          if(_fayeClient !== null) {
            return _fayeClient;
          }

          _fayeClient = new Faye.Client(_environmentConfiguration[_options.environment].faye_root);

          // Add some Faye middleware to detect when the super vision app pairs and disconnects with this running
          // instance of trampoline
          var monitorConnectionState = function(message, callback) {
            var messageData = message.data || {};

            // If it's not a connection status event, ignore it
            if(messageData.type !== 'status') {
              return callback(message);
            }

            // Detect an "unpair" event
            if(messageData.value === 'unpaired') {
              return disconnect();
            }

            // notify listeners if another client (a super vision app instance) subscribes
            if(messageData.value === 'paired') {
              Trampoline.util.setLocalStorage('supervision_channel_id', message.channel.replace('/', ''));
              Trampoline.events.dispatchEvent(SUPER_VISION_PAIR);
              _supervisionContainer.classList.add('connected');
              _trampolineContainer.classList.add('supervision-connected');
              _isPaired = true;

              // also, clear the "auto-close" timeout, since they successfully connected
              window.clearTimeout(_closePanelTimeout);
            }

            callback(message);
          };
          _fayeClient.addExtension({ incoming: monitorConnectionState });

          // Add Faye middleware for detecting timer changes
          var monitorTimers = function(message, callback) {
            var messageData = message.data || {};
            
            // If we just subscribed, and there's an existing timer, we'll need to attach a new timer
            if(message.channel == '/meta/subscribe' && message._channel && message._channel.timer) {
              setPlayTimer(message._channel.timer);
              return callback(message);
            }

            // If it's not a play timer event, just break out early
            if(messageData._meta === undefined || messageData._meta.type !== 'play_timer') {
              return callback(message);
            }

            // If we receive a play timer event, process it the same way
            if(messageData.name === 'sleep') {
              setPlayTimer(messageData);
            }

            // If we receive a cancellation, cancel
            if(messageData.name === 'cancel_sleep') {
              clearPlayTimer(message);
            }

            return callback(message);
          };
          _fayeClient.addExtension({ incoming: monitorTimers });

          // Add Faye middleware for detecting kid label reassignments
          var monitorKidLabels = function(message, callback) {
            var messageData = message.data || {};
            
            // If we just subscribed be sure to attach sync state
            if(message.channel == '/meta/subscribe' && message._channel) {
              Trampoline.supervision.sync_state = message._channel;
            }

            // ignore everything but an update kid label message
            if(messageData._meta === undefined || messageData._meta.type != 'update_kid_label') {
              return callback(message);
            }

            // make sure that sync_state is indeed defined
            Trampoline.supervision.sync_state = Trampoline.supervision.sync_state || {};

            // if there's a save slot, treat sync_state's default kid label as an object. Otherwise, it should be flat
            if(messageData.save_slot !== undefined) {
              Trampoline.supervision.sync_state.default_kid_label = Trampoline.supervision.sync_state.default_kid_label || {};
              Trampoline.supervision.sync_state.default_kid_label[messageData.save_slot] = messageData.kid_label_guid;
            } else {
              Trampoline.supervision.sync_state.default_kid_label = messageData.kid_label_guid;
            }
            
            return callback(message);
          };
          _fayeClient.addExtension({ incoming: monitorKidLabels });

          return _fayeClient;
        },

        /**
         * Checks if this trampoline instance has a cached connection to super vision and reconnects. However, if there
         * is no connection, this function is a no-op
         * @memberof Trampoline.supervision
         * @return {Boolean} Whether or not a connection was able to be established
         */
        reconnectToExistingChannel = function() {
          initializeFayeClient();

          // Determine if the storage has anything
          var _cachedChannelId = Trampoline.util.getLocalStorage('supervision_channel_id');
          if(_cachedChannelId !== null && _cachedChannelId !== '' && _cachedChannelId !== undefined) {
            // reconnect
            var _onSuccessfulReconnect = function() {
              _supervisionContainer.classList.add('connected');
              _trampolineContainer.classList.add('supervision-connected');
              _isPaired = true;
            };

            var _onFailedReconnect = function() {
              Trampoline.util.setLocalStorage('supervision_channel_id', '');
            };

            _previousSubscription = _fayeClient.subscribe('/' + _cachedChannelId);
            _previousSubscription.then(_onSuccessfulReconnect, _onFailedReconnect);
            return true;
          }

          return false;
        },

        /**
         * Establishes a connection to a new channel
         * @memberof Trampoline.supervision
         */
        connectToNewChannel = function() {
          // A callback that initiates the Faye connection on a channel when a new (or existing) channel is created
          // (or retrieved from localstorage)
          var onGetChannel = function(channel) {
            // update the UI
            displaySyncCode(channel.sync_code);

            // now subscribe to the channel (the middleware will handle the rest)
            _previousSubscription = _fayeClient.subscribe('/' + channel.channel_id);
          };

          getSyncCode(function(response) {
            onGetChannel(JSON.parse(response));
          });
        },

        /**
         * Sends data to super vision over a websocket connection
         * @memberof Trampoline.supervision
         * @param {object} message The data to send to supervision
         * @param {Function} callback A callback for when the message is has been received and confirmed by the server
         * @return {Boolean} Whether or not a message was sent
         */
        send = function(message, callback) {
          // If we're not paired, this is a no-op
          if(!_isPaired) {
            return false;
          }

          // create a new session id if this is a new activity
          if(message._meta && message._meta.type == 'activity') {
            _currentSessionId = _sessionId();
          }

          // make sure to attach session id
          message.game_session = _currentSessionId;

          callback = callback instanceof Function ? callback : function() { };
          _fayeClient.publish('/' + Trampoline.util.getLocalStorage('supervision_channel_id'), message).then(callback);

          return true;
        },

        /**
         * Disconnects from the current connected channel on super vision
         * @memberof Trampoline.supervision
         */
        disconnect = function(){
          var okayButton = _supervisionContainer.querySelector('.okay');
          var cancelButton = _supervisionContainer.querySelector('.cancel');

          var _finishDisconnect = function() {
            // "forget" this existing channel
            Trampoline.util.setLocalStorage('supervision_channel_id', '');

            // notify any local listeners
            Trampoline.analytics.event('SuperVision', 'unpair');
            Trampoline.events.dispatchEvent(SUPER_VISION_UNPAIR);
            _isPaired = false;
          
            //Reset the status which controls the UI state.
            _syncCodeContainer.dataset.status = 'getting-code';

            // Allow the buttons to be used again (even though they're being hidden)
            okayButton.classList.remove('disabled');
            okayButton.disabled = false;
            cancelButton.classList.remove('disabled');
            cancelButton.disabled = false;

            // now, attempt to reconnect
            connectToNewChannel();
            _supervisionContainer.classList.remove('connected');
            _trampolineContainer.classList.remove('supervision-connected');
          };

          // mark the container that we're trying to disconnect
          okayButton.classList.add('disabled');
          okayButton.disabled = true;
          cancelButton.classList.add('disabled');
          cancelButton.disabled = true;

          // stop listening for faye messages on this channel
          if(_previousSubscription) {
            // Notify apps that we're unpairing
            return send({ type: 'status', value: 'unpaired' }, function() {
              _previousSubscription.cancel();
              _previousSubscription = null;
              _finishDisconnect();
            });
          }

          _finishDisconnect();
        },

        /**
         * Displays a given sync code in the sync code box on the screen
         * @memberof Trampoline.supervision
         * @param {String} syncCode The sync code to display
         */
        displaySyncCode = function(syncCode) {
          //Cycle through the sync code letters
          var _syncCodeCharacters = _syncCodeContainer.querySelectorAll('.character');
          
          if (!!syncCode && syncCode.length > 0){
            for (var i = 0; i < syncCode.length; i++) {
              _syncCodeCharacters[i].innerHTML = syncCode[i];
            }
            _syncCodeContainer.dataset.status = 'connecting';
          }else{
            _log('error displaying sync code');
            _syncCodeContainer.dataset.status = 'try-later';
            _clearSyncCode();
          }
        },

        /**
         * Displays the play timer overlay for a specific type
         * @memberof Trampoline.supervision
         * @param {String} type The type of play timer
         */
        showPlayTimerOverlay = function(type) {
          //TODO: The type is returning undefined
          _log(type);
          document.getElementById('trampoline-play-timer').classList.add('active');
          document.getElementById(type+'-overlay').classList.add('active');
          Trampoline.events.dispatchEvent(PLAYTIMER_SHOW);
        },

        /**
         * Sets a new play timer
         * @param {Object} timer The timer instance received from Faye
         */
        setPlayTimer = function(timer) {
          clearPlayTimer();

          // If the timer has passed, ignore it
          if(timer.value < 0) {
            return;
          }

          _currentPlayTimerTimeout = window.setTimeout(function() {
            showPlayTimerOverlay(timer.mode);
          }, timer.value);
        },

        /**
         * Clears the local play timer
         * @member Trampoline.supervision
         */
        clearPlayTimer = function() {
          window.clearTimeout(_currentPlayTimerTimeout);
          _currentPlayTimerTimeout = null;

          // hide the current play timer overlay (if there are any)
          document.getElementById('trampoline-play-timer').classList.remove('active');
          
          Array.prototype.forEach.call(document.querySelectorAll('.play-timer-overlay.active'), function(element) {
            element.classList.remove('active');
          });
          Trampoline.events.dispatchEvent(PLAYTIMER_HIDE);
        },
        
        /**
         * Toggle the Super Vision module view
         * @memberof Trampoline.supervision
         */
        toggleView = function (){
          if (!!_panelActive){
            _close();
          }else{
            _open();
          }
        },

        /**
         * Opens the Super Vision view
         * @memberof Trampoline.supervision
         */
        _open = function (){
          if(!_isPaired) {
            connectToNewChannel();

            // Set the panel to close after inactivity. That way, the parent doesn't accidentally try to sync with an expired code
            window.clearTimeout(_closePanelTimeout);
            _closePanelTimeout = window.setTimeout(Trampoline.state.back, 4 * 60 * 1000);
          }

          _panelActive = true;
          if (!Trampoline.util.isConnected){
            _supervisionContainer.classList.add('offline');
          }else if( _supervisionContainer.classList.contains('offline') ){
            _supervisionContainer.classList.remove('offline');
          }
          _supervisionContainer.classList.add('active');
          Trampoline.analytics.event('SuperVision', 'open');
          Trampoline.events.dispatchEvent(SUPER_VISION_OPEN);
        },

        /**
         * Closes the Super Vision view
         * @memberof Trampoline.supervision
         */
        _close = function (){
          _panelActive = false;
          _supervisionContainer.classList.remove('active');
          Trampoline.events.dispatchEvent(SUPER_VISION_CLOSE);
        },

        /**
         * Helper function to generate a random session id to uniquely distinguish sessions on the same device
         * @memberof Trampoline.supervision
         * @return {String}
         */
        _sessionId = function () {
          // Pad with timestamp, which will make collisions difficult
          var timestamp = new Date().getTime().toString(16),
              // Lastly, add a padded random integer as well to decrease collisions. The following number was chosen due to
              // https://bugs.chromium.org/p/v8/issues/detail?id=558 which indicates Math.random in "Chrome-like" browsers
              // have only 30 bits of entropy for random numbers.
              MAX_SAFE_INTEGER = 1073741824,
              random = Trampoline.util.randomInt(0, MAX_SAFE_INTEGER).toString(16),
              padLength = 8 - random.length;
          
          random = '00000000'.slice(0, padLength) + random;

          return timestamp + random;
        },

        /**
         * Removes the Super Vision and Play-Timer views from the DOM
         * @memberof Trampoline.supervision
         */
        remove = function (){
          if (!_supervisionContainer){
            _supervisionContainer = document.getElementById('trampoline').querySelector('section.supervision');
          }
          if (!_playTimerContainer){
            _playTimerContainer = document.getElementById('#trampoline-play-timer');
          }
          if (!!_supervisionContainer && !!_supervisionContainer.parentElement){
            _supervisionContainer.parentElement.removeChild(_supervisionContainer);
          }
          if (!!_playTimerContainer && !!_playTimerContainer.parentElement){
            _playTimerContainer.parentElement.removeChild(_playTimerContainer);
          }
        };
    
    return {
      init                : init,
      disconnect          : disconnect,
      remove              : remove,
      send                : send,
      toggleView          : toggleView,
      sync_state          : sync_state,
      connectionLost    : connectionLost,
      connectionResume  : connectionResume
    };

  }());
            
  return Trampoline;
  
}(Trampoline || {}));

(function e(t,n,r){function s(o,u){if(!n[o]){if(!t[o]){var a=typeof require=="function"&&require;if(!u&&a)return a(o,!0);if(i)return i(o,!0);var f=new Error("Cannot find module '"+o+"'");throw f.code="MODULE_NOT_FOUND",f}var l=n[o]={exports:{}};t[o][0].call(l.exports,function(e){var n=t[o][1][e];return s(n?n:e)},l,l.exports,e,t,n,r)}return n[o].exports}var i=typeof require=="function"&&require;for(var o=0;o<r.length;o++)s(r[o]);return s})({1:[function(require,module,exports){
'use strict';

var has = Object.prototype.hasOwnProperty;

//
// We store our EE objects in a plain object whose properties are event names.
// If `Object.create(null)` is not supported we prefix the event names with a
// `~` to make sure that the built-in object properties are not overridden or
// used as an attack vector.
// We also assume that `Object.create(null)` is available when the event name
// is an ES6 Symbol.
//
var prefix = typeof Object.create !== 'function' ? '~' : false;

/**
 * Representation of a single EventEmitter function.
 *
 * @param {Function} fn Event handler to be called.
 * @param {Mixed} context Context for function execution.
 * @param {Boolean} [once=false] Only emit once
 * @api private
 */
function EE(fn, context, once) {
  this.fn = fn;
  this.context = context;
  this.once = once || false;
}

/**
 * Minimal EventEmitter interface that is molded against the Node.js
 * EventEmitter interface.
 *
 * @constructor
 * @api public
 */
function EventEmitter() { /* Nothing to set */ }

/**
 * Hold the assigned EventEmitters by name.
 *
 * @type {Object}
 * @private
 */
EventEmitter.prototype._events = undefined;

/**
 * Return an array listing the events for which the emitter has registered
 * listeners.
 *
 * @returns {Array}
 * @api public
 */
EventEmitter.prototype.eventNames = function eventNames() {
  var events = this._events
    , names = []
    , name;

  if (!events) return names;

  for (name in events) {
    if (has.call(events, name)) names.push(prefix ? name.slice(1) : name);
  }

  if (Object.getOwnPropertySymbols) {
    return names.concat(Object.getOwnPropertySymbols(events));
  }

  return names;
};

/**
 * Return a list of assigned event listeners.
 *
 * @param {String} event The events that should be listed.
 * @param {Boolean} exists We only need to know if there are listeners.
 * @returns {Array|Boolean}
 * @api public
 */
EventEmitter.prototype.listeners = function listeners(event, exists) {
  var evt = prefix ? prefix + event : event
    , available = this._events && this._events[evt];

  if (exists) return !!available;
  if (!available) return [];
  if (available.fn) return [available.fn];

  for (var i = 0, l = available.length, ee = new Array(l); i < l; i++) {
    ee[i] = available[i].fn;
  }

  return ee;
};

/**
 * Emit an event to all registered event listeners.
 *
 * @param {String} event The name of the event.
 * @returns {Boolean} Indication if we've emitted an event.
 * @api public
 */
EventEmitter.prototype.emit = function emit(event, a1, a2, a3, a4, a5) {
  var evt = prefix ? prefix + event : event;

  if (!this._events || !this._events[evt]) return false;

  var listeners = this._events[evt]
    , len = arguments.length
    , args
    , i;

  if ('function' === typeof listeners.fn) {
    if (listeners.once) this.removeListener(event, listeners.fn, undefined, true);

    switch (len) {
      case 1: return listeners.fn.call(listeners.context), true;
      case 2: return listeners.fn.call(listeners.context, a1), true;
      case 3: return listeners.fn.call(listeners.context, a1, a2), true;
      case 4: return listeners.fn.call(listeners.context, a1, a2, a3), true;
      case 5: return listeners.fn.call(listeners.context, a1, a2, a3, a4), true;
      case 6: return listeners.fn.call(listeners.context, a1, a2, a3, a4, a5), true;
    }

    for (i = 1, args = new Array(len -1); i < len; i++) {
      args[i - 1] = arguments[i];
    }

    listeners.fn.apply(listeners.context, args);
  } else {
    var length = listeners.length
      , j;

    for (i = 0; i < length; i++) {
      if (listeners[i].once) this.removeListener(event, listeners[i].fn, undefined, true);

      switch (len) {
        case 1: listeners[i].fn.call(listeners[i].context); break;
        case 2: listeners[i].fn.call(listeners[i].context, a1); break;
        case 3: listeners[i].fn.call(listeners[i].context, a1, a2); break;
        default:
          if (!args) for (j = 1, args = new Array(len -1); j < len; j++) {
            args[j - 1] = arguments[j];
          }

          listeners[i].fn.apply(listeners[i].context, args);
      }
    }
  }

  return true;
};

/**
 * Register a new EventListener for the given event.
 *
 * @param {String} event Name of the event.
 * @param {Function} fn Callback function.
 * @param {Mixed} [context=this] The context of the function.
 * @api public
 */
EventEmitter.prototype.on = function on(event, fn, context) {
  var listener = new EE(fn, context || this)
    , evt = prefix ? prefix + event : event;

  if (!this._events) this._events = prefix ? {} : Object.create(null);
  if (!this._events[evt]) this._events[evt] = listener;
  else {
    if (!this._events[evt].fn) this._events[evt].push(listener);
    else this._events[evt] = [
      this._events[evt], listener
    ];
  }

  return this;
};

/**
 * Add an EventListener that's only called once.
 *
 * @param {String} event Name of the event.
 * @param {Function} fn Callback function.
 * @param {Mixed} [context=this] The context of the function.
 * @api public
 */
EventEmitter.prototype.once = function once(event, fn, context) {
  var listener = new EE(fn, context || this, true)
    , evt = prefix ? prefix + event : event;

  if (!this._events) this._events = prefix ? {} : Object.create(null);
  if (!this._events[evt]) this._events[evt] = listener;
  else {
    if (!this._events[evt].fn) this._events[evt].push(listener);
    else this._events[evt] = [
      this._events[evt], listener
    ];
  }

  return this;
};

/**
 * Remove event listeners.
 *
 * @param {String} event The event we want to remove.
 * @param {Function} fn The listener that we need to find.
 * @param {Mixed} context Only remove listeners matching this context.
 * @param {Boolean} once Only remove once listeners.
 * @api public
 */
EventEmitter.prototype.removeListener = function removeListener(event, fn, context, once) {
  var evt = prefix ? prefix + event : event;

  if (!this._events || !this._events[evt]) return this;

  var listeners = this._events[evt]
    , events = [];

  if (fn) {
    if (listeners.fn) {
      if (
           listeners.fn !== fn
        || (once && !listeners.once)
        || (context && listeners.context !== context)
      ) {
        events.push(listeners);
      }
    } else {
      for (var i = 0, length = listeners.length; i < length; i++) {
        if (
             listeners[i].fn !== fn
          || (once && !listeners[i].once)
          || (context && listeners[i].context !== context)
        ) {
          events.push(listeners[i]);
        }
      }
    }
  }

  //
  // Reset the array, or remove it completely if we have no more listeners.
  //
  if (events.length) {
    this._events[evt] = events.length === 1 ? events[0] : events;
  } else {
    delete this._events[evt];
  }

  return this;
};

/**
 * Remove all listeners or only the listeners for the specified event.
 *
 * @param {String} event The event want to remove all listeners for.
 * @api public
 */
EventEmitter.prototype.removeAllListeners = function removeAllListeners(event) {
  if (!this._events) return this;

  if (event) delete this._events[prefix ? prefix + event : event];
  else this._events = prefix ? {} : Object.create(null);

  return this;
};

//
// Alias methods names because people roll like that.
//
EventEmitter.prototype.off = EventEmitter.prototype.removeListener;
EventEmitter.prototype.addListener = EventEmitter.prototype.on;

//
// This function doesn't apply anymore.
//
EventEmitter.prototype.setMaxListeners = function setMaxListeners() {
  return this;
};

//
// Expose the prefix.
//
EventEmitter.prefixed = prefix;

//
// Expose the module.
//
if ('undefined' !== typeof module) {
  module.exports = EventEmitter;
}

},{}],2:[function(require,module,exports){
// shim for using process in browser

var process = module.exports = {};

// cached from whatever global is present so that test runners that stub it
// don't break things.  But we need to wrap it in a try catch in case it is
// wrapped in strict mode code which doesn't define any globals.  It's inside a
// function because try/catches deoptimize in certain engines.

var cachedSetTimeout;
var cachedClearTimeout;

(function () {
  try {
    cachedSetTimeout = setTimeout;
  } catch (e) {
    cachedSetTimeout = function () {
      throw new Error('setTimeout is not defined');
    }
  }
  try {
    cachedClearTimeout = clearTimeout;
  } catch (e) {
    cachedClearTimeout = function () {
      throw new Error('clearTimeout is not defined');
    }
  }
} ())
var queue = [];
var draining = false;
var currentQueue;
var queueIndex = -1;

function cleanUpNextTick() {
    if (!draining || !currentQueue) {
        return;
    }
    draining = false;
    if (currentQueue.length) {
        queue = currentQueue.concat(queue);
    } else {
        queueIndex = -1;
    }
    if (queue.length) {
        drainQueue();
    }
}

function drainQueue() {
    if (draining) {
        return;
    }
    var timeout = cachedSetTimeout(cleanUpNextTick);
    draining = true;

    var len = queue.length;
    while(len) {
        currentQueue = queue;
        queue = [];
        while (++queueIndex < len) {
            if (currentQueue) {
                currentQueue[queueIndex].run();
            }
        }
        queueIndex = -1;
        len = queue.length;
    }
    currentQueue = null;
    draining = false;
    cachedClearTimeout(timeout);
}

process.nextTick = function (fun) {
    var args = new Array(arguments.length - 1);
    if (arguments.length > 1) {
        for (var i = 1; i < arguments.length; i++) {
            args[i - 1] = arguments[i];
        }
    }
    queue.push(new Item(fun, args));
    if (queue.length === 1 && !draining) {
        cachedSetTimeout(drainQueue, 0);
    }
};

// v8 likes predictible objects
function Item(fun, array) {
    this.fun = fun;
    this.array = array;
}
Item.prototype.run = function () {
    this.fun.apply(null, this.array);
};
process.title = 'browser';
process.browser = true;
process.env = {};
process.argv = [];
process.version = ''; // empty string to avoid regexp issues
process.versions = {};

function noop() {}

process.on = noop;
process.addListener = noop;
process.once = noop;
process.off = noop;
process.removeListener = noop;
process.removeAllListeners = noop;
process.emit = noop;

process.binding = function (name) {
    throw new Error('process.binding is not supported');
};

process.cwd = function () { return '/' };
process.chdir = function (dir) {
    throw new Error('process.chdir is not supported');
};
process.umask = function() { return 0; };

},{}],3:[function(require,module,exports){
(function (global){
/*! https://mths.be/punycode v1.4.1 by @mathias */
;(function(root) {

	/** Detect free variables */
	var freeExports = typeof exports == 'object' && exports &&
		!exports.nodeType && exports;
	var freeModule = typeof module == 'object' && module &&
		!module.nodeType && module;
	var freeGlobal = typeof global == 'object' && global;
	if (
		freeGlobal.global === freeGlobal ||
		freeGlobal.window === freeGlobal ||
		freeGlobal.self === freeGlobal
	) {
		root = freeGlobal;
	}

	/**
	 * The `punycode` object.
	 * @name punycode
	 * @type Object
	 */
	var punycode,

	/** Highest positive signed 32-bit float value */
	maxInt = 2147483647, // aka. 0x7FFFFFFF or 2^31-1

	/** Bootstring parameters */
	base = 36,
	tMin = 1,
	tMax = 26,
	skew = 38,
	damp = 700,
	initialBias = 72,
	initialN = 128, // 0x80
	delimiter = '-', // '\x2D'

	/** Regular expressions */
	regexPunycode = /^xn--/,
	regexNonASCII = /[^\x20-\x7E]/, // unprintable ASCII chars + non-ASCII chars
	regexSeparators = /[\x2E\u3002\uFF0E\uFF61]/g, // RFC 3490 separators

	/** Error messages */
	errors = {
		'overflow': 'Overflow: input needs wider integers to process',
		'not-basic': 'Illegal input >= 0x80 (not a basic code point)',
		'invalid-input': 'Invalid input'
	},

	/** Convenience shortcuts */
	baseMinusTMin = base - tMin,
	floor = Math.floor,
	stringFromCharCode = String.fromCharCode,

	/** Temporary variable */
	key;

	/*--------------------------------------------------------------------------*/

	/**
	 * A generic error utility function.
	 * @private
	 * @param {String} type The error type.
	 * @returns {Error} Throws a `RangeError` with the applicable error message.
	 */
	function error(type) {
		throw new RangeError(errors[type]);
	}

	/**
	 * A generic `Array#map` utility function.
	 * @private
	 * @param {Array} array The array to iterate over.
	 * @param {Function} callback The function that gets called for every array
	 * item.
	 * @returns {Array} A new array of values returned by the callback function.
	 */
	function map(array, fn) {
		var length = array.length;
		var result = [];
		while (length--) {
			result[length] = fn(array[length]);
		}
		return result;
	}

	/**
	 * A simple `Array#map`-like wrapper to work with domain name strings or email
	 * addresses.
	 * @private
	 * @param {String} domain The domain name or email address.
	 * @param {Function} callback The function that gets called for every
	 * character.
	 * @returns {Array} A new string of characters returned by the callback
	 * function.
	 */
	function mapDomain(string, fn) {
		var parts = string.split('@');
		var result = '';
		if (parts.length > 1) {
			// In email addresses, only the domain name should be punycoded. Leave
			// the local part (i.e. everything up to `@`) intact.
			result = parts[0] + '@';
			string = parts[1];
		}
		// Avoid `split(regex)` for IE8 compatibility. See #17.
		string = string.replace(regexSeparators, '\x2E');
		var labels = string.split('.');
		var encoded = map(labels, fn).join('.');
		return result + encoded;
	}

	/**
	 * Creates an array containing the numeric code points of each Unicode
	 * character in the string. While JavaScript uses UCS-2 internally,
	 * this function will convert a pair of surrogate halves (each of which
	 * UCS-2 exposes as separate characters) into a single code point,
	 * matching UTF-16.
	 * @see `punycode.ucs2.encode`
	 * @see <https://mathiasbynens.be/notes/javascript-encoding>
	 * @memberOf punycode.ucs2
	 * @name decode
	 * @param {String} string The Unicode input string (UCS-2).
	 * @returns {Array} The new array of code points.
	 */
	function ucs2decode(string) {
		var output = [],
		    counter = 0,
		    length = string.length,
		    value,
		    extra;
		while (counter < length) {
			value = string.charCodeAt(counter++);
			if (value >= 0xD800 && value <= 0xDBFF && counter < length) {
				// high surrogate, and there is a next character
				extra = string.charCodeAt(counter++);
				if ((extra & 0xFC00) == 0xDC00) { // low surrogate
					output.push(((value & 0x3FF) << 10) + (extra & 0x3FF) + 0x10000);
				} else {
					// unmatched surrogate; only append this code unit, in case the next
					// code unit is the high surrogate of a surrogate pair
					output.push(value);
					counter--;
				}
			} else {
				output.push(value);
			}
		}
		return output;
	}

	/**
	 * Creates a string based on an array of numeric code points.
	 * @see `punycode.ucs2.decode`
	 * @memberOf punycode.ucs2
	 * @name encode
	 * @param {Array} codePoints The array of numeric code points.
	 * @returns {String} The new Unicode string (UCS-2).
	 */
	function ucs2encode(array) {
		return map(array, function(value) {
			var output = '';
			if (value > 0xFFFF) {
				value -= 0x10000;
				output += stringFromCharCode(value >>> 10 & 0x3FF | 0xD800);
				value = 0xDC00 | value & 0x3FF;
			}
			output += stringFromCharCode(value);
			return output;
		}).join('');
	}

	/**
	 * Converts a basic code point into a digit/integer.
	 * @see `digitToBasic()`
	 * @private
	 * @param {Number} codePoint The basic numeric code point value.
	 * @returns {Number} The numeric value of a basic code point (for use in
	 * representing integers) in the range `0` to `base - 1`, or `base` if
	 * the code point does not represent a value.
	 */
	function basicToDigit(codePoint) {
		if (codePoint - 48 < 10) {
			return codePoint - 22;
		}
		if (codePoint - 65 < 26) {
			return codePoint - 65;
		}
		if (codePoint - 97 < 26) {
			return codePoint - 97;
		}
		return base;
	}

	/**
	 * Converts a digit/integer into a basic code point.
	 * @see `basicToDigit()`
	 * @private
	 * @param {Number} digit The numeric value of a basic code point.
	 * @returns {Number} The basic code point whose value (when used for
	 * representing integers) is `digit`, which needs to be in the range
	 * `0` to `base - 1`. If `flag` is non-zero, the uppercase form is
	 * used; else, the lowercase form is used. The behavior is undefined
	 * if `flag` is non-zero and `digit` has no uppercase form.
	 */
	function digitToBasic(digit, flag) {
		//  0..25 map to ASCII a..z or A..Z
		// 26..35 map to ASCII 0..9
		return digit + 22 + 75 * (digit < 26) - ((flag != 0) << 5);
	}

	/**
	 * Bias adaptation function as per section 3.4 of RFC 3492.
	 * https://tools.ietf.org/html/rfc3492#section-3.4
	 * @private
	 */
	function adapt(delta, numPoints, firstTime) {
		var k = 0;
		delta = firstTime ? floor(delta / damp) : delta >> 1;
		delta += floor(delta / numPoints);
		for (/* no initialization */; delta > baseMinusTMin * tMax >> 1; k += base) {
			delta = floor(delta / baseMinusTMin);
		}
		return floor(k + (baseMinusTMin + 1) * delta / (delta + skew));
	}

	/**
	 * Converts a Punycode string of ASCII-only symbols to a string of Unicode
	 * symbols.
	 * @memberOf punycode
	 * @param {String} input The Punycode string of ASCII-only symbols.
	 * @returns {String} The resulting string of Unicode symbols.
	 */
	function decode(input) {
		// Don't use UCS-2
		var output = [],
		    inputLength = input.length,
		    out,
		    i = 0,
		    n = initialN,
		    bias = initialBias,
		    basic,
		    j,
		    index,
		    oldi,
		    w,
		    k,
		    digit,
		    t,
		    /** Cached calculation results */
		    baseMinusT;

		// Handle the basic code points: let `basic` be the number of input code
		// points before the last delimiter, or `0` if there is none, then copy
		// the first basic code points to the output.

		basic = input.lastIndexOf(delimiter);
		if (basic < 0) {
			basic = 0;
		}

		for (j = 0; j < basic; ++j) {
			// if it's not a basic code point
			if (input.charCodeAt(j) >= 0x80) {
				error('not-basic');
			}
			output.push(input.charCodeAt(j));
		}

		// Main decoding loop: start just after the last delimiter if any basic code
		// points were copied; start at the beginning otherwise.

		for (index = basic > 0 ? basic + 1 : 0; index < inputLength; /* no final expression */) {

			// `index` is the index of the next character to be consumed.
			// Decode a generalized variable-length integer into `delta`,
			// which gets added to `i`. The overflow checking is easier
			// if we increase `i` as we go, then subtract off its starting
			// value at the end to obtain `delta`.
			for (oldi = i, w = 1, k = base; /* no condition */; k += base) {

				if (index >= inputLength) {
					error('invalid-input');
				}

				digit = basicToDigit(input.charCodeAt(index++));

				if (digit >= base || digit > floor((maxInt - i) / w)) {
					error('overflow');
				}

				i += digit * w;
				t = k <= bias ? tMin : (k >= bias + tMax ? tMax : k - bias);

				if (digit < t) {
					break;
				}

				baseMinusT = base - t;
				if (w > floor(maxInt / baseMinusT)) {
					error('overflow');
				}

				w *= baseMinusT;

			}

			out = output.length + 1;
			bias = adapt(i - oldi, out, oldi == 0);

			// `i` was supposed to wrap around from `out` to `0`,
			// incrementing `n` each time, so we'll fix that now:
			if (floor(i / out) > maxInt - n) {
				error('overflow');
			}

			n += floor(i / out);
			i %= out;

			// Insert `n` at position `i` of the output
			output.splice(i++, 0, n);

		}

		return ucs2encode(output);
	}

	/**
	 * Converts a string of Unicode symbols (e.g. a domain name label) to a
	 * Punycode string of ASCII-only symbols.
	 * @memberOf punycode
	 * @param {String} input The string of Unicode symbols.
	 * @returns {String} The resulting Punycode string of ASCII-only symbols.
	 */
	function encode(input) {
		var n,
		    delta,
		    handledCPCount,
		    basicLength,
		    bias,
		    j,
		    m,
		    q,
		    k,
		    t,
		    currentValue,
		    output = [],
		    /** `inputLength` will hold the number of code points in `input`. */
		    inputLength,
		    /** Cached calculation results */
		    handledCPCountPlusOne,
		    baseMinusT,
		    qMinusT;

		// Convert the input in UCS-2 to Unicode
		input = ucs2decode(input);

		// Cache the length
		inputLength = input.length;

		// Initialize the state
		n = initialN;
		delta = 0;
		bias = initialBias;

		// Handle the basic code points
		for (j = 0; j < inputLength; ++j) {
			currentValue = input[j];
			if (currentValue < 0x80) {
				output.push(stringFromCharCode(currentValue));
			}
		}

		handledCPCount = basicLength = output.length;

		// `handledCPCount` is the number of code points that have been handled;
		// `basicLength` is the number of basic code points.

		// Finish the basic string - if it is not empty - with a delimiter
		if (basicLength) {
			output.push(delimiter);
		}

		// Main encoding loop:
		while (handledCPCount < inputLength) {

			// All non-basic code points < n have been handled already. Find the next
			// larger one:
			for (m = maxInt, j = 0; j < inputLength; ++j) {
				currentValue = input[j];
				if (currentValue >= n && currentValue < m) {
					m = currentValue;
				}
			}

			// Increase `delta` enough to advance the decoder's <n,i> state to <m,0>,
			// but guard against overflow
			handledCPCountPlusOne = handledCPCount + 1;
			if (m - n > floor((maxInt - delta) / handledCPCountPlusOne)) {
				error('overflow');
			}

			delta += (m - n) * handledCPCountPlusOne;
			n = m;

			for (j = 0; j < inputLength; ++j) {
				currentValue = input[j];

				if (currentValue < n && ++delta > maxInt) {
					error('overflow');
				}

				if (currentValue == n) {
					// Represent delta as a generalized variable-length integer
					for (q = delta, k = base; /* no condition */; k += base) {
						t = k <= bias ? tMin : (k >= bias + tMax ? tMax : k - bias);
						if (q < t) {
							break;
						}
						qMinusT = q - t;
						baseMinusT = base - t;
						output.push(
							stringFromCharCode(digitToBasic(t + qMinusT % baseMinusT, 0))
						);
						q = floor(qMinusT / baseMinusT);
					}

					output.push(stringFromCharCode(digitToBasic(q, 0)));
					bias = adapt(delta, handledCPCountPlusOne, handledCPCount == basicLength);
					delta = 0;
					++handledCPCount;
				}
			}

			++delta;
			++n;

		}
		return output.join('');
	}

	/**
	 * Converts a Punycode string representing a domain name or an email address
	 * to Unicode. Only the Punycoded parts of the input will be converted, i.e.
	 * it doesn't matter if you call it on a string that has already been
	 * converted to Unicode.
	 * @memberOf punycode
	 * @param {String} input The Punycoded domain name or email address to
	 * convert to Unicode.
	 * @returns {String} The Unicode representation of the given Punycode
	 * string.
	 */
	function toUnicode(input) {
		return mapDomain(input, function(string) {
			return regexPunycode.test(string)
				? decode(string.slice(4).toLowerCase())
				: string;
		});
	}

	/**
	 * Converts a Unicode string representing a domain name or an email address to
	 * Punycode. Only the non-ASCII parts of the domain name will be converted,
	 * i.e. it doesn't matter if you call it with a domain that's already in
	 * ASCII.
	 * @memberOf punycode
	 * @param {String} input The domain name or email address to convert, as a
	 * Unicode string.
	 * @returns {String} The Punycode representation of the given domain name or
	 * email address.
	 */
	function toASCII(input) {
		return mapDomain(input, function(string) {
			return regexNonASCII.test(string)
				? 'xn--' + encode(string)
				: string;
		});
	}

	/*--------------------------------------------------------------------------*/

	/** Define the public API */
	punycode = {
		/**
		 * A string representing the current Punycode.js version number.
		 * @memberOf punycode
		 * @type String
		 */
		'version': '1.4.1',
		/**
		 * An object of methods to convert from JavaScript's internal character
		 * representation (UCS-2) to Unicode code points, and back.
		 * @see <https://mathiasbynens.be/notes/javascript-encoding>
		 * @memberOf punycode
		 * @type Object
		 */
		'ucs2': {
			'decode': ucs2decode,
			'encode': ucs2encode
		},
		'decode': decode,
		'encode': encode,
		'toASCII': toASCII,
		'toUnicode': toUnicode
	};

	/** Expose `punycode` */
	// Some AMD build optimizers, like r.js, check for specific condition patterns
	// like the following:
	if (
		typeof define == 'function' &&
		typeof define.amd == 'object' &&
		define.amd
	) {
		define('punycode', function() {
			return punycode;
		});
	} else if (freeExports && freeModule) {
		if (module.exports == freeExports) {
			// in Node.js, io.js, or RingoJS v0.8.0+
			freeModule.exports = punycode;
		} else {
			// in Narwhal or RingoJS v0.7.0-
			for (key in punycode) {
				punycode.hasOwnProperty(key) && (freeExports[key] = punycode[key]);
			}
		}
	} else {
		// in Rhino or a web browser
		root.punycode = punycode;
	}

}(this));

}).call(this,typeof global !== "undefined" ? global : typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{}],4:[function(require,module,exports){
// Copyright Joyent, Inc. and other Node contributors.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

'use strict';

// If obj.hasOwnProperty has been overridden, then calling
// obj.hasOwnProperty(prop) will break.
// See: https://github.com/joyent/node/issues/1707
function hasOwnProperty(obj, prop) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

module.exports = function(qs, sep, eq, options) {
  sep = sep || '&';
  eq = eq || '=';
  var obj = {};

  if (typeof qs !== 'string' || qs.length === 0) {
    return obj;
  }

  var regexp = /\+/g;
  qs = qs.split(sep);

  var maxKeys = 1000;
  if (options && typeof options.maxKeys === 'number') {
    maxKeys = options.maxKeys;
  }

  var len = qs.length;
  // maxKeys <= 0 means that we should not limit keys count
  if (maxKeys > 0 && len > maxKeys) {
    len = maxKeys;
  }

  for (var i = 0; i < len; ++i) {
    var x = qs[i].replace(regexp, '%20'),
        idx = x.indexOf(eq),
        kstr, vstr, k, v;

    if (idx >= 0) {
      kstr = x.substr(0, idx);
      vstr = x.substr(idx + 1);
    } else {
      kstr = x;
      vstr = '';
    }

    k = decodeURIComponent(kstr);
    v = decodeURIComponent(vstr);

    if (!hasOwnProperty(obj, k)) {
      obj[k] = v;
    } else if (isArray(obj[k])) {
      obj[k].push(v);
    } else {
      obj[k] = [obj[k], v];
    }
  }

  return obj;
};

var isArray = Array.isArray || function (xs) {
  return Object.prototype.toString.call(xs) === '[object Array]';
};

},{}],5:[function(require,module,exports){
// Copyright Joyent, Inc. and other Node contributors.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

'use strict';

var stringifyPrimitive = function(v) {
  switch (typeof v) {
    case 'string':
      return v;

    case 'boolean':
      return v ? 'true' : 'false';

    case 'number':
      return isFinite(v) ? v : '';

    default:
      return '';
  }
};

module.exports = function(obj, sep, eq, name) {
  sep = sep || '&';
  eq = eq || '=';
  if (obj === null) {
    obj = undefined;
  }

  if (typeof obj === 'object') {
    return map(objectKeys(obj), function(k) {
      var ks = encodeURIComponent(stringifyPrimitive(k)) + eq;
      if (isArray(obj[k])) {
        return map(obj[k], function(v) {
          return ks + encodeURIComponent(stringifyPrimitive(v));
        }).join(sep);
      } else {
        return ks + encodeURIComponent(stringifyPrimitive(obj[k]));
      }
    }).join(sep);

  }

  if (!name) return '';
  return encodeURIComponent(stringifyPrimitive(name)) + eq +
         encodeURIComponent(stringifyPrimitive(obj));
};

var isArray = Array.isArray || function (xs) {
  return Object.prototype.toString.call(xs) === '[object Array]';
};

function map (xs, f) {
  if (xs.map) return xs.map(f);
  var res = [];
  for (var i = 0; i < xs.length; i++) {
    res.push(f(xs[i], i));
  }
  return res;
}

var objectKeys = Object.keys || function (obj) {
  var res = [];
  for (var key in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, key)) res.push(key);
  }
  return res;
};

},{}],6:[function(require,module,exports){
'use strict';

exports.decode = exports.parse = require('./decode');
exports.encode = exports.stringify = require('./encode');

},{"./decode":4,"./encode":5}],7:[function(require,module,exports){
(function (process){
/*!
 * async
 * https://github.com/caolan/async
 *
 * Copyright 2010-2014 Caolan McMahon
 * Released under the MIT license
 */
/*jshint onevar: false, indent:4 */
/*global setImmediate: false, setTimeout: false, console: false */
(function () {

    var async = {};

    // global on the server, window in the browser
    var root, previous_async;

    root = this;
    if (root != null) {
      previous_async = root.async;
    }

    async.noConflict = function () {
        root.async = previous_async;
        return async;
    };

    function only_once(fn) {
        var called = false;
        return function() {
            if (called) throw new Error("Callback was already called.");
            called = true;
            fn.apply(root, arguments);
        }
    }

    //// cross-browser compatiblity functions ////

    var _toString = Object.prototype.toString;

    var _isArray = Array.isArray || function (obj) {
        return _toString.call(obj) === '[object Array]';
    };

    var _each = function (arr, iterator) {
        for (var i = 0; i < arr.length; i += 1) {
            iterator(arr[i], i, arr);
        }
    };

    var _map = function (arr, iterator) {
        if (arr.map) {
            return arr.map(iterator);
        }
        var results = [];
        _each(arr, function (x, i, a) {
            results.push(iterator(x, i, a));
        });
        return results;
    };

    var _reduce = function (arr, iterator, memo) {
        if (arr.reduce) {
            return arr.reduce(iterator, memo);
        }
        _each(arr, function (x, i, a) {
            memo = iterator(memo, x, i, a);
        });
        return memo;
    };

    var _keys = function (obj) {
        if (Object.keys) {
            return Object.keys(obj);
        }
        var keys = [];
        for (var k in obj) {
            if (obj.hasOwnProperty(k)) {
                keys.push(k);
            }
        }
        return keys;
    };

    //// exported async module functions ////

    //// nextTick implementation with browser-compatible fallback ////
    if (typeof process === 'undefined' || !(process.nextTick)) {
        if (typeof setImmediate === 'function') {
            async.nextTick = function (fn) {
                // not a direct alias for IE10 compatibility
                setImmediate(fn);
            };
            async.setImmediate = async.nextTick;
        }
        else {
            async.nextTick = function (fn) {
                setTimeout(fn, 0);
            };
            async.setImmediate = async.nextTick;
        }
    }
    else {
        async.nextTick = process.nextTick;
        if (typeof setImmediate !== 'undefined') {
            async.setImmediate = function (fn) {
              // not a direct alias for IE10 compatibility
              setImmediate(fn);
            };
        }
        else {
            async.setImmediate = async.nextTick;
        }
    }

    async.each = function (arr, iterator, callback) {
        callback = callback || function () {};
        if (!arr.length) {
            return callback();
        }
        var completed = 0;
        _each(arr, function (x) {
            iterator(x, only_once(done) );
        });
        function done(err) {
          if (err) {
              callback(err);
              callback = function () {};
          }
          else {
              completed += 1;
              if (completed >= arr.length) {
                  callback();
              }
          }
        }
    };
    async.forEach = async.each;

    async.eachSeries = function (arr, iterator, callback) {
        callback = callback || function () {};
        if (!arr.length) {
            return callback();
        }
        var completed = 0;
        var iterate = function () {
            iterator(arr[completed], function (err) {
                if (err) {
                    callback(err);
                    callback = function () {};
                }
                else {
                    completed += 1;
                    if (completed >= arr.length) {
                        callback();
                    }
                    else {
                        iterate();
                    }
                }
            });
        };
        iterate();
    };
    async.forEachSeries = async.eachSeries;

    async.eachLimit = function (arr, limit, iterator, callback) {
        var fn = _eachLimit(limit);
        fn.apply(null, [arr, iterator, callback]);
    };
    async.forEachLimit = async.eachLimit;

    var _eachLimit = function (limit) {

        return function (arr, iterator, callback) {
            callback = callback || function () {};
            if (!arr.length || limit <= 0) {
                return callback();
            }
            var completed = 0;
            var started = 0;
            var running = 0;

            (function replenish () {
                if (completed >= arr.length) {
                    return callback();
                }

                while (running < limit && started < arr.length) {
                    started += 1;
                    running += 1;
                    iterator(arr[started - 1], function (err) {
                        if (err) {
                            callback(err);
                            callback = function () {};
                        }
                        else {
                            completed += 1;
                            running -= 1;
                            if (completed >= arr.length) {
                                callback();
                            }
                            else {
                                replenish();
                            }
                        }
                    });
                }
            })();
        };
    };


    var doParallel = function (fn) {
        return function () {
            var args = Array.prototype.slice.call(arguments);
            return fn.apply(null, [async.each].concat(args));
        };
    };
    var doParallelLimit = function(limit, fn) {
        return function () {
            var args = Array.prototype.slice.call(arguments);
            return fn.apply(null, [_eachLimit(limit)].concat(args));
        };
    };
    var doSeries = function (fn) {
        return function () {
            var args = Array.prototype.slice.call(arguments);
            return fn.apply(null, [async.eachSeries].concat(args));
        };
    };


    var _asyncMap = function (eachfn, arr, iterator, callback) {
        arr = _map(arr, function (x, i) {
            return {index: i, value: x};
        });
        if (!callback) {
            eachfn(arr, function (x, callback) {
                iterator(x.value, function (err) {
                    callback(err);
                });
            });
        } else {
            var results = [];
            eachfn(arr, function (x, callback) {
                iterator(x.value, function (err, v) {
                    results[x.index] = v;
                    callback(err);
                });
            }, function (err) {
                callback(err, results);
            });
        }
    };
    async.map = doParallel(_asyncMap);
    async.mapSeries = doSeries(_asyncMap);
    async.mapLimit = function (arr, limit, iterator, callback) {
        return _mapLimit(limit)(arr, iterator, callback);
    };

    var _mapLimit = function(limit) {
        return doParallelLimit(limit, _asyncMap);
    };

    // reduce only has a series version, as doing reduce in parallel won't
    // work in many situations.
    async.reduce = function (arr, memo, iterator, callback) {
        async.eachSeries(arr, function (x, callback) {
            iterator(memo, x, function (err, v) {
                memo = v;
                callback(err);
            });
        }, function (err) {
            callback(err, memo);
        });
    };
    // inject alias
    async.inject = async.reduce;
    // foldl alias
    async.foldl = async.reduce;

    async.reduceRight = function (arr, memo, iterator, callback) {
        var reversed = _map(arr, function (x) {
            return x;
        }).reverse();
        async.reduce(reversed, memo, iterator, callback);
    };
    // foldr alias
    async.foldr = async.reduceRight;

    var _filter = function (eachfn, arr, iterator, callback) {
        var results = [];
        arr = _map(arr, function (x, i) {
            return {index: i, value: x};
        });
        eachfn(arr, function (x, callback) {
            iterator(x.value, function (v) {
                if (v) {
                    results.push(x);
                }
                callback();
            });
        }, function (err) {
            callback(_map(results.sort(function (a, b) {
                return a.index - b.index;
            }), function (x) {
                return x.value;
            }));
        });
    };
    async.filter = doParallel(_filter);
    async.filterSeries = doSeries(_filter);
    // select alias
    async.select = async.filter;
    async.selectSeries = async.filterSeries;

    var _reject = function (eachfn, arr, iterator, callback) {
        var results = [];
        arr = _map(arr, function (x, i) {
            return {index: i, value: x};
        });
        eachfn(arr, function (x, callback) {
            iterator(x.value, function (v) {
                if (!v) {
                    results.push(x);
                }
                callback();
            });
        }, function (err) {
            callback(_map(results.sort(function (a, b) {
                return a.index - b.index;
            }), function (x) {
                return x.value;
            }));
        });
    };
    async.reject = doParallel(_reject);
    async.rejectSeries = doSeries(_reject);

    var _detect = function (eachfn, arr, iterator, main_callback) {
        eachfn(arr, function (x, callback) {
            iterator(x, function (result) {
                if (result) {
                    main_callback(x);
                    main_callback = function () {};
                }
                else {
                    callback();
                }
            });
        }, function (err) {
            main_callback();
        });
    };
    async.detect = doParallel(_detect);
    async.detectSeries = doSeries(_detect);

    async.some = function (arr, iterator, main_callback) {
        async.each(arr, function (x, callback) {
            iterator(x, function (v) {
                if (v) {
                    main_callback(true);
                    main_callback = function () {};
                }
                callback();
            });
        }, function (err) {
            main_callback(false);
        });
    };
    // any alias
    async.any = async.some;

    async.every = function (arr, iterator, main_callback) {
        async.each(arr, function (x, callback) {
            iterator(x, function (v) {
                if (!v) {
                    main_callback(false);
                    main_callback = function () {};
                }
                callback();
            });
        }, function (err) {
            main_callback(true);
        });
    };
    // all alias
    async.all = async.every;

    async.sortBy = function (arr, iterator, callback) {
        async.map(arr, function (x, callback) {
            iterator(x, function (err, criteria) {
                if (err) {
                    callback(err);
                }
                else {
                    callback(null, {value: x, criteria: criteria});
                }
            });
        }, function (err, results) {
            if (err) {
                return callback(err);
            }
            else {
                var fn = function (left, right) {
                    var a = left.criteria, b = right.criteria;
                    return a < b ? -1 : a > b ? 1 : 0;
                };
                callback(null, _map(results.sort(fn), function (x) {
                    return x.value;
                }));
            }
        });
    };

    async.auto = function (tasks, callback) {
        callback = callback || function () {};
        var keys = _keys(tasks);
        var remainingTasks = keys.length
        if (!remainingTasks) {
            return callback();
        }

        var results = {};

        var listeners = [];
        var addListener = function (fn) {
            listeners.unshift(fn);
        };
        var removeListener = function (fn) {
            for (var i = 0; i < listeners.length; i += 1) {
                if (listeners[i] === fn) {
                    listeners.splice(i, 1);
                    return;
                }
            }
        };
        var taskComplete = function () {
            remainingTasks--
            _each(listeners.slice(0), function (fn) {
                fn();
            });
        };

        addListener(function () {
            if (!remainingTasks) {
                var theCallback = callback;
                // prevent final callback from calling itself if it errors
                callback = function () {};

                theCallback(null, results);
            }
        });

        _each(keys, function (k) {
            var task = _isArray(tasks[k]) ? tasks[k]: [tasks[k]];
            var taskCallback = function (err) {
                var args = Array.prototype.slice.call(arguments, 1);
                if (args.length <= 1) {
                    args = args[0];
                }
                if (err) {
                    var safeResults = {};
                    _each(_keys(results), function(rkey) {
                        safeResults[rkey] = results[rkey];
                    });
                    safeResults[k] = args;
                    callback(err, safeResults);
                    // stop subsequent errors hitting callback multiple times
                    callback = function () {};
                }
                else {
                    results[k] = args;
                    async.setImmediate(taskComplete);
                }
            };
            var requires = task.slice(0, Math.abs(task.length - 1)) || [];
            var ready = function () {
                return _reduce(requires, function (a, x) {
                    return (a && results.hasOwnProperty(x));
                }, true) && !results.hasOwnProperty(k);
            };
            if (ready()) {
                task[task.length - 1](taskCallback, results);
            }
            else {
                var listener = function () {
                    if (ready()) {
                        removeListener(listener);
                        task[task.length - 1](taskCallback, results);
                    }
                };
                addListener(listener);
            }
        });
    };

    async.retry = function(times, task, callback) {
        var DEFAULT_TIMES = 5;
        var attempts = [];
        // Use defaults if times not passed
        if (typeof times === 'function') {
            callback = task;
            task = times;
            times = DEFAULT_TIMES;
        }
        // Make sure times is a number
        times = parseInt(times, 10) || DEFAULT_TIMES;
        var wrappedTask = function(wrappedCallback, wrappedResults) {
            var retryAttempt = function(task, finalAttempt) {
                return function(seriesCallback) {
                    task(function(err, result){
                        seriesCallback(!err || finalAttempt, {err: err, result: result});
                    }, wrappedResults);
                };
            };
            while (times) {
                attempts.push(retryAttempt(task, !(times-=1)));
            }
            async.series(attempts, function(done, data){
                data = data[data.length - 1];
                (wrappedCallback || callback)(data.err, data.result);
            });
        }
        // If a callback is passed, run this as a controll flow
        return callback ? wrappedTask() : wrappedTask
    };

    async.waterfall = function (tasks, callback) {
        callback = callback || function () {};
        if (!_isArray(tasks)) {
          var err = new Error('First argument to waterfall must be an array of functions');
          return callback(err);
        }
        if (!tasks.length) {
            return callback();
        }
        var wrapIterator = function (iterator) {
            return function (err) {
                if (err) {
                    callback.apply(null, arguments);
                    callback = function () {};
                }
                else {
                    var args = Array.prototype.slice.call(arguments, 1);
                    var next = iterator.next();
                    if (next) {
                        args.push(wrapIterator(next));
                    }
                    else {
                        args.push(callback);
                    }
                    async.setImmediate(function () {
                        iterator.apply(null, args);
                    });
                }
            };
        };
        wrapIterator(async.iterator(tasks))();
    };

    var _parallel = function(eachfn, tasks, callback) {
        callback = callback || function () {};
        if (_isArray(tasks)) {
            eachfn.map(tasks, function (fn, callback) {
                if (fn) {
                    fn(function (err) {
                        var args = Array.prototype.slice.call(arguments, 1);
                        if (args.length <= 1) {
                            args = args[0];
                        }
                        callback.call(null, err, args);
                    });
                }
            }, callback);
        }
        else {
            var results = {};
            eachfn.each(_keys(tasks), function (k, callback) {
                tasks[k](function (err) {
                    var args = Array.prototype.slice.call(arguments, 1);
                    if (args.length <= 1) {
                        args = args[0];
                    }
                    results[k] = args;
                    callback(err);
                });
            }, function (err) {
                callback(err, results);
            });
        }
    };

    async.parallel = function (tasks, callback) {
        _parallel({ map: async.map, each: async.each }, tasks, callback);
    };

    async.parallelLimit = function(tasks, limit, callback) {
        _parallel({ map: _mapLimit(limit), each: _eachLimit(limit) }, tasks, callback);
    };

    async.series = function (tasks, callback) {
        callback = callback || function () {};
        if (_isArray(tasks)) {
            async.mapSeries(tasks, function (fn, callback) {
                if (fn) {
                    fn(function (err) {
                        var args = Array.prototype.slice.call(arguments, 1);
                        if (args.length <= 1) {
                            args = args[0];
                        }
                        callback.call(null, err, args);
                    });
                }
            }, callback);
        }
        else {
            var results = {};
            async.eachSeries(_keys(tasks), function (k, callback) {
                tasks[k](function (err) {
                    var args = Array.prototype.slice.call(arguments, 1);
                    if (args.length <= 1) {
                        args = args[0];
                    }
                    results[k] = args;
                    callback(err);
                });
            }, function (err) {
                callback(err, results);
            });
        }
    };

    async.iterator = function (tasks) {
        var makeCallback = function (index) {
            var fn = function () {
                if (tasks.length) {
                    tasks[index].apply(null, arguments);
                }
                return fn.next();
            };
            fn.next = function () {
                return (index < tasks.length - 1) ? makeCallback(index + 1): null;
            };
            return fn;
        };
        return makeCallback(0);
    };

    async.apply = function (fn) {
        var args = Array.prototype.slice.call(arguments, 1);
        return function () {
            return fn.apply(
                null, args.concat(Array.prototype.slice.call(arguments))
            );
        };
    };

    var _concat = function (eachfn, arr, fn, callback) {
        var r = [];
        eachfn(arr, function (x, cb) {
            fn(x, function (err, y) {
                r = r.concat(y || []);
                cb(err);
            });
        }, function (err) {
            callback(err, r);
        });
    };
    async.concat = doParallel(_concat);
    async.concatSeries = doSeries(_concat);

    async.whilst = function (test, iterator, callback) {
        if (test()) {
            iterator(function (err) {
                if (err) {
                    return callback(err);
                }
                async.whilst(test, iterator, callback);
            });
        }
        else {
            callback();
        }
    };

    async.doWhilst = function (iterator, test, callback) {
        iterator(function (err) {
            if (err) {
                return callback(err);
            }
            var args = Array.prototype.slice.call(arguments, 1);
            if (test.apply(null, args)) {
                async.doWhilst(iterator, test, callback);
            }
            else {
                callback();
            }
        });
    };

    async.until = function (test, iterator, callback) {
        if (!test()) {
            iterator(function (err) {
                if (err) {
                    return callback(err);
                }
                async.until(test, iterator, callback);
            });
        }
        else {
            callback();
        }
    };

    async.doUntil = function (iterator, test, callback) {
        iterator(function (err) {
            if (err) {
                return callback(err);
            }
            var args = Array.prototype.slice.call(arguments, 1);
            if (!test.apply(null, args)) {
                async.doUntil(iterator, test, callback);
            }
            else {
                callback();
            }
        });
    };

    async.queue = function (worker, concurrency) {
        if (concurrency === undefined) {
            concurrency = 1;
        }
        function _insert(q, data, pos, callback) {
          if (!q.started){
            q.started = true;
          }
          if (!_isArray(data)) {
              data = [data];
          }
          if(data.length == 0) {
             // call drain immediately if there are no tasks
             return async.setImmediate(function() {
                 if (q.drain) {
                     q.drain();
                 }
             });
          }
          _each(data, function(task) {
              var item = {
                  data: task,
                  callback: typeof callback === 'function' ? callback : null
              };

              if (pos) {
                q.tasks.unshift(item);
              } else {
                q.tasks.push(item);
              }

              if (q.saturated && q.tasks.length === q.concurrency) {
                  q.saturated();
              }
              async.setImmediate(q.process);
          });
        }

        var workers = 0;
        var q = {
            tasks: [],
            concurrency: concurrency,
            saturated: null,
            empty: null,
            drain: null,
            started: false,
            paused: false,
            push: function (data, callback) {
              _insert(q, data, false, callback);
            },
            kill: function () {
              q.drain = null;
              q.tasks = [];
            },
            unshift: function (data, callback) {
              _insert(q, data, true, callback);
            },
            process: function () {
                if (!q.paused && workers < q.concurrency && q.tasks.length) {
                    var task = q.tasks.shift();
                    if (q.empty && q.tasks.length === 0) {
                        q.empty();
                    }
                    workers += 1;
                    var next = function () {
                        workers -= 1;
                        if (task.callback) {
                            task.callback.apply(task, arguments);
                        }
                        if (q.drain && q.tasks.length + workers === 0) {
                            q.drain();
                        }
                        q.process();
                    };
                    var cb = only_once(next);
                    worker(task.data, cb);
                }
            },
            length: function () {
                return q.tasks.length;
            },
            running: function () {
                return workers;
            },
            idle: function() {
                return q.tasks.length + workers === 0;
            },
            pause: function () {
                if (q.paused === true) { return; }
                q.paused = true;
            },
            resume: function () {
                if (q.paused === false) { return; }
                q.paused = false;
                // Need to call q.process once per concurrent
                // worker to preserve full concurrency after pause
                for (var w = 1; w <= q.concurrency; w++) {
                    async.setImmediate(q.process);
                }
            }
        };
        return q;
    };

    async.priorityQueue = function (worker, concurrency) {

        function _compareTasks(a, b){
          return a.priority - b.priority;
        };

        function _binarySearch(sequence, item, compare) {
          var beg = -1,
              end = sequence.length - 1;
          while (beg < end) {
            var mid = beg + ((end - beg + 1) >>> 1);
            if (compare(item, sequence[mid]) >= 0) {
              beg = mid;
            } else {
              end = mid - 1;
            }
          }
          return beg;
        }

        function _insert(q, data, priority, callback) {
          if (!q.started){
            q.started = true;
          }
          if (!_isArray(data)) {
              data = [data];
          }
          if(data.length == 0) {
             // call drain immediately if there are no tasks
             return async.setImmediate(function() {
                 if (q.drain) {
                     q.drain();
                 }
             });
          }
          _each(data, function(task) {
              var item = {
                  data: task,
                  priority: priority,
                  callback: typeof callback === 'function' ? callback : null
              };

              q.tasks.splice(_binarySearch(q.tasks, item, _compareTasks) + 1, 0, item);

              if (q.saturated && q.tasks.length === q.concurrency) {
                  q.saturated();
              }
              async.setImmediate(q.process);
          });
        }

        // Start with a normal queue
        var q = async.queue(worker, concurrency);

        // Override push to accept second parameter representing priority
        q.push = function (data, priority, callback) {
          _insert(q, data, priority, callback);
        };

        // Remove unshift function
        delete q.unshift;

        return q;
    };

    async.cargo = function (worker, payload) {
        var working     = false,
            tasks       = [];

        var cargo = {
            tasks: tasks,
            payload: payload,
            saturated: null,
            empty: null,
            drain: null,
            drained: true,
            push: function (data, callback) {
                if (!_isArray(data)) {
                    data = [data];
                }
                _each(data, function(task) {
                    tasks.push({
                        data: task,
                        callback: typeof callback === 'function' ? callback : null
                    });
                    cargo.drained = false;
                    if (cargo.saturated && tasks.length === payload) {
                        cargo.saturated();
                    }
                });
                async.setImmediate(cargo.process);
            },
            process: function process() {
                if (working) return;
                if (tasks.length === 0) {
                    if(cargo.drain && !cargo.drained) cargo.drain();
                    cargo.drained = true;
                    return;
                }

                var ts = typeof payload === 'number'
                            ? tasks.splice(0, payload)
                            : tasks.splice(0, tasks.length);

                var ds = _map(ts, function (task) {
                    return task.data;
                });

                if(cargo.empty) cargo.empty();
                working = true;
                worker(ds, function () {
                    working = false;

                    var args = arguments;
                    _each(ts, function (data) {
                        if (data.callback) {
                            data.callback.apply(null, args);
                        }
                    });

                    process();
                });
            },
            length: function () {
                return tasks.length;
            },
            running: function () {
                return working;
            }
        };
        return cargo;
    };

    var _console_fn = function (name) {
        return function (fn) {
            var args = Array.prototype.slice.call(arguments, 1);
            fn.apply(null, args.concat([function (err) {
                var args = Array.prototype.slice.call(arguments, 1);
                if (typeof console !== 'undefined') {
                    if (err) {
                        if (console.error) {
                            console.error(err);
                        }
                    }
                    else if (console[name]) {
                        _each(args, function (x) {
                            console[name](x);
                        });
                    }
                }
            }]));
        };
    };
    async.log = _console_fn('log');
    async.dir = _console_fn('dir');
    /*async.info = _console_fn('info');
    async.warn = _console_fn('warn');
    async.error = _console_fn('error');*/

    async.memoize = function (fn, hasher) {
        var memo = {};
        var queues = {};
        hasher = hasher || function (x) {
            return x;
        };
        var memoized = function () {
            var args = Array.prototype.slice.call(arguments);
            var callback = args.pop();
            var key = hasher.apply(null, args);
            if (key in memo) {
                async.nextTick(function () {
                    callback.apply(null, memo[key]);
                });
            }
            else if (key in queues) {
                queues[key].push(callback);
            }
            else {
                queues[key] = [callback];
                fn.apply(null, args.concat([function () {
                    memo[key] = arguments;
                    var q = queues[key];
                    delete queues[key];
                    for (var i = 0, l = q.length; i < l; i++) {
                      q[i].apply(null, arguments);
                    }
                }]));
            }
        };
        memoized.memo = memo;
        memoized.unmemoized = fn;
        return memoized;
    };

    async.unmemoize = function (fn) {
      return function () {
        return (fn.unmemoized || fn).apply(null, arguments);
      };
    };

    async.times = function (count, iterator, callback) {
        var counter = [];
        for (var i = 0; i < count; i++) {
            counter.push(i);
        }
        return async.map(counter, iterator, callback);
    };

    async.timesSeries = function (count, iterator, callback) {
        var counter = [];
        for (var i = 0; i < count; i++) {
            counter.push(i);
        }
        return async.mapSeries(counter, iterator, callback);
    };

    async.seq = function (/* functions... */) {
        var fns = arguments;
        return function () {
            var that = this;
            var args = Array.prototype.slice.call(arguments);
            var callback = args.pop();
            async.reduce(fns, args, function (newargs, fn, cb) {
                fn.apply(that, newargs.concat([function () {
                    var err = arguments[0];
                    var nextargs = Array.prototype.slice.call(arguments, 1);
                    cb(err, nextargs);
                }]))
            },
            function (err, results) {
                callback.apply(that, [err].concat(results));
            });
        };
    };

    async.compose = function (/* functions... */) {
      return async.seq.apply(null, Array.prototype.reverse.call(arguments));
    };

    var _applyEach = function (eachfn, fns /*args...*/) {
        var go = function () {
            var that = this;
            var args = Array.prototype.slice.call(arguments);
            var callback = args.pop();
            return eachfn(fns, function (fn, cb) {
                fn.apply(that, args.concat([cb]));
            },
            callback);
        };
        if (arguments.length > 2) {
            var args = Array.prototype.slice.call(arguments, 2);
            return go.apply(this, args);
        }
        else {
            return go;
        }
    };
    async.applyEach = doParallel(_applyEach);
    async.applyEachSeries = doSeries(_applyEach);

    async.forever = function (fn, callback) {
        function next(err) {
            if (err) {
                if (callback) {
                    return callback(err);
                }
                throw err;
            }
            fn(next);
        }
        next();
    };

    // Node.js
    if (typeof module !== 'undefined' && module.exports) {
        module.exports = async;
    }
    // AMD / RequireJS
    else if (typeof define !== 'undefined' && define.amd) {
        define([], function () {
            return async;
        });
    }
    // included directly via <script> tag
    else {
        root.async = async;
    }

}());

}).call(this,require('_process'))
},{"_process":2}],8:[function(require,module,exports){
var async       = require('async'),
    urlParser   = require('url'),
    Resource    = require('./Resource'),
    EventEmitter = require('eventemitter3');

/**
 * Manages the state and loading of multiple resources to load.
 *
 * @class
 * @param [baseUrl=''] {string} The base url for all resources loaded by this loader.
 * @param [concurrency=10] {number} The number of resources to load concurrently.
 */
function Loader(baseUrl, concurrency) {
    EventEmitter.call(this);

    concurrency = concurrency || 10;

    /**
     * The base url for all resources loaded by this loader.
     *
     * @member {string}
     */
    this.baseUrl = baseUrl || '';

    /**
     * The progress percent of the loader going through the queue.
     *
     * @member {number}
     */
    this.progress = 0;

    /**
     * Loading state of the loader, true if it is currently loading resources.
     *
     * @member {boolean}
     */
    this.loading = false;

    /**
     * The percentage of total progress that a single resource represents.
     *
     * @member {number}
     */
    this._progressChunk = 0;

    /**
     * The middleware to run before loading each resource.
     *
     * @member {function[]}
     */
    this._beforeMiddleware = [];

    /**
     * The middleware to run after loading each resource.
     *
     * @member {function[]}
     */
    this._afterMiddleware = [];

    /**
     * The `_loadResource` function bound with this object context.
     *
     * @private
     * @member {function}
     */
    this._boundLoadResource = this._loadResource.bind(this);

    /**
     * The `_onLoad` function bound with this object context.
     *
     * @private
     * @member {function}
     */
    this._boundOnLoad = this._onLoad.bind(this);

    /**
     * The resource buffer that fills until `load` is called to start loading resources.
     *
     * @private
     * @member {Resource[]}
     */
    this._buffer = [];

    /**
     * Used to track load completion.
     *
     * @private
     * @member {number}
     */
    this._numToLoad = 0;

    /**
     * The resources waiting to be loaded.
     *
     * @private
     * @member {Resource[]}
     */
    this._queue = async.queue(this._boundLoadResource, concurrency);

    /**
     * All the resources for this loader keyed by name.
     *
     * @member {object<string, Resource>}
     */
    this.resources = {};

    /**
     * Emitted once per loaded or errored resource.
     *
     * @event progress
     * @memberof Loader#
     */

    /**
     * Emitted once per errored resource.
     *
     * @event error
     * @memberof Loader#
     */

    /**
     * Emitted once per loaded resource.
     *
     * @event load
     * @memberof Loader#
     */

    /**
     * Emitted when the loader begins to process the queue.
     *
     * @event start
     * @memberof Loader#
     */

    /**
     * Emitted when the queued resources all load.
     *
     * @event complete
     * @memberof Loader#
     */
}

Loader.prototype = Object.create(EventEmitter.prototype);
Loader.prototype.constructor = Loader;
module.exports = Loader;

/**
 * Adds a resource (or multiple resources) to the loader queue.
 *
 * This function can take a wide variety of different parameters. The only thing that is always
 * required the url to load. All the following will work:
 *
 * ```js
 * loader
 *     // normal param syntax
 *     .add('key', 'http://...', function () {})
 *     .add('http://...', function () {})
 *     .add('http://...')
 *
 *     // object syntax
 *     .add({
 *         name: 'key2',
 *         url: 'http://...'
 *     }, function () {})
 *     .add({
 *         url: 'http://...'
 *     }, function () {})
 *     .add({
 *         name: 'key3',
 *         url: 'http://...'
 *         onComplete: function () {}
 *     })
 *     .add({
 *         url: 'https://...',
 *         onComplete: function () {},
 *         crossOrigin: true
 *     })
 *
 *     // you can also pass an array of objects or urls or both
 *     .add([
 *         { name: 'key4', url: 'http://...', onComplete: function () {} },
 *         { url: 'http://...', onComplete: function () {} },
 *         'http://...'
 *     ]);
 * ```
 *
 * @alias enqueue
 * @param [name] {string} The name of the resource to load, if not passed the url is used.
 * @param url {string} The url for this resource, relative to the baseUrl of this loader.
 * @param [options] {object} The options for the load.
 * @param [options.crossOrigin] {boolean} Is this request cross-origin? Default is to determine automatically.
 * @param [options.loadType=Resource.LOAD_TYPE.XHR] {Resource.XHR_LOAD_TYPE} How should this resource be loaded?
 * @param [options.xhrType=Resource.XHR_RESPONSE_TYPE.DEFAULT] {Resource.XHR_RESPONSE_TYPE} How should the data being
 *      loaded be interpreted when using XHR?
 * @param [callback] {function} Function to call when this specific resource completes loading.
 * @return {Loader}
 */
Loader.prototype.add = Loader.prototype.enqueue = function (name, url, options, cb) {
    // special case of an array of objects or urls
    if (Array.isArray(name)) {
        for (var i = 0; i < name.length; ++i) {
            this.add(name[i]);
        }

        return this;
    }

    // if an object is passed instead of params
    if (typeof name === 'object') {
        cb = url || name.callback || name.onComplete;
        options = name;
        url = name.url;
        name = name.name || name.key || name.url;
    }

    // case where no name is passed shift all args over by one.
    if (typeof url !== 'string') {
        cb = options;
        options = url;
        url = name;
    }

    // now that we shifted make sure we have a proper url.
    if (typeof url !== 'string') {
        throw new Error('No url passed to add resource to loader.');
    }

    // options are optional so people might pass a function and no options
    if (typeof options === 'function') {
        cb = options;
        options = null;
    }

    // check if resource already exists.
    if (this.resources[name]) {
        throw new Error('Resource with name "' + name + '" already exists.');
    }

    // add base url if this isn't an absolute url
    url = this._handleBaseUrl(url);

    // create the store the resource
    this.resources[name] = new Resource(name, url, options);

    if (typeof cb === 'function') {
        this.resources[name].once('afterMiddleware', cb);
    }

    this._numToLoad++;

    // if already loading add it to the worker queue
    if (this._queue.started) {
        this._queue.push(this.resources[name]);
        this._progressChunk = (100 - this.progress) / (this._queue.length() + this._queue.running());
    }
    // otherwise buffer it to be added to the queue later
    else {
        this._buffer.push(this.resources[name]);
        this._progressChunk = 100 / this._buffer.length;
    }

    return this;
};

Loader.prototype._handleBaseUrl = function (url) {
    var parsedUrl = urlParser.parse(url);

    // absolute url, just use it as is.
    if (parsedUrl.protocol || !parsedUrl.pathname || parsedUrl.pathname.indexOf('//') === 0) {
        return url;
    }

    // if baseUrl doesn't end in slash and url doesn't start with slash, then add a slash inbetween
    if (
        this.baseUrl.length &&
        this.baseUrl.lastIndexOf('/') !== this.baseUrl.length - 1 &&
        url.charAt(0) !== '/'
    ) {
        return this.baseUrl + '/' + url;
    }
    else {
        return this.baseUrl + url;
    }
};


/**
 * Sets up a middleware function that will run *before* the
 * resource is loaded.
 *
 * @alias pre
 * @param middleware {function} The middleware function to register.
 * @return {Loader}
 */
Loader.prototype.before = Loader.prototype.pre = function (fn) {
    this._beforeMiddleware.push(fn);

    return this;
};

/**
 * Sets up a middleware function that will run *after* the
 * resource is loaded.
 *
 * @alias use
 * @param middleware {function} The middleware function to register.
 * @return {Loader}
 */
Loader.prototype.after = Loader.prototype.use = function (fn) {
    this._afterMiddleware.push(fn);

    return this;
};

/**
 * Resets the queue of the loader to prepare for a new load.
 *
 * @return {Loader}
 */
Loader.prototype.reset = function () {
    // this.baseUrl = baseUrl || '';

    this.progress = 0;

    this.loading = false;

    this._progressChunk = 0;

    // this._beforeMiddleware.length = 0;
    // this._afterMiddleware.length = 0;

    this._buffer.length = 0;

    this._numToLoad = 0;

    this._queue.kill();
    this._queue.started = false;

    this.resources = {};
};

/**
 * Starts loading the queued resources.
 *
 * @fires start
 * @param [callback] {function} Optional callback that will be bound to the `complete` event.
 * @return {Loader}
 */
Loader.prototype.load = function (cb) {
    // register complete callback if they pass one
    if (typeof cb === 'function') {
        this.once('complete', cb);
    }

    // if the queue has already started we are done here
    if (this._queue.started) {
        return this;
    }

    // notify of start
    this.emit('start', this);

    // update loading state
    this.loading = true;

    // start the internal queue
    for (var i = 0; i < this._buffer.length; ++i) {
        this._queue.push(this._buffer[i]);
    }

    // empty the buffer
    this._buffer.length = 0;

    return this;
};

/**
 * Loads a single resource.
 *
 * @fires progress
 * @private
 */
Loader.prototype._loadResource = function (resource, dequeue) {
    var self = this;

    resource._dequeue = dequeue;

    this._runMiddleware(resource, this._beforeMiddleware, function () {
        // resource.on('progress', self.emit.bind(self, 'progress'));

        resource.load(self._boundOnLoad);
    });
};

/**
 * Called once each resource has loaded.
 *
 * @fires complete
 * @private
 */
Loader.prototype._onComplete = function () {
    this.loading = false;

    this.emit('complete', this, this.resources);
};

/**
 * Called each time a resources is loaded.
 *
 * @fires progress
 * @fires error
 * @fires load
 * @private
 */
Loader.prototype._onLoad = function (resource) {
    // run middleware, this *must* happen before dequeue so sub-assets get added properly
    this._runMiddleware(resource, this._afterMiddleware, function () {
        resource.emit('afterMiddleware', resource);

        this._numToLoad--;
        
        this.progress += this._progressChunk;
        this.emit('progress', this, resource);

        if (resource.error) {
            this.emit('error', resource.error, this, resource);
        }
        else {
            this.emit('load', this, resource);
        }

        // do completion check
        if (this._numToLoad === 0) {
            this.progress = 100;
            this._onComplete();
        }
    });
    


    // remove this resource from the async queue
    resource._dequeue();
};

/**
 * Run middleware functions on a resource.
 *
 * @private
 */
Loader.prototype._runMiddleware = function (resource, fns, cb) {
    var self = this;

    async.eachSeries(fns, function (fn, next) {
        fn.call(self, resource, next);
    }, cb.bind(this, resource));
};

Loader.LOAD_TYPE = Resource.LOAD_TYPE;
Loader.XHR_READY_STATE = Resource.XHR_READY_STATE;
Loader.XHR_RESPONSE_TYPE = Resource.XHR_RESPONSE_TYPE;

},{"./Resource":9,"async":7,"eventemitter3":1,"url":14}],9:[function(require,module,exports){
var EventEmitter = require('eventemitter3'),
    _url = require('url'),
    // tests is CORS is supported in XHR, if not we need to use XDR
    useXdr = !!(window.XDomainRequest && !('withCredentials' in (new XMLHttpRequest()))),
    tempAnchor = null;

/**
 * Manages the state and loading of a single resource represented by
 * a single URL.
 *
 * @class
 * @param name {string} The name of the resource to load.
 * @param url {string|string[]} The url for this resource, for audio/video loads you can pass an array of sources.
 * @param [options] {object} The options for the load.
 * @param [options.crossOrigin] {string|boolean} Is this request cross-origin? Default is to determine automatically.
 * @param [options.loadType=Resource.LOAD_TYPE.XHR] {Resource.LOAD_TYPE} How should this resource be loaded?
 * @param [options.xhrType=Resource.XHR_RESPONSE_TYPE.DEFAULT] {Resource.XHR_RESPONSE_TYPE} How should the data being
 *      loaded be interpreted when using XHR?
 * @param [options.metadata] {object} Extra info for middleware.
 */
function Resource(name, url, options) {
    EventEmitter.call(this);

    options = options || {};

    if (typeof name !== 'string' || typeof url !== 'string') {
        throw new Error('Both name and url are required for constructing a resource.');
    }

    /**
     * The name of this resource.
     *
     * @member {string}
     * @readonly
     */
    this.name = name;

    /**
     * The url used to load this resource.
     *
     * @member {string}
     * @readonly
     */
    this.url = url;

    /**
     * Stores whether or not this url is a data url.
     *
     * @member {boolean}
     * @readonly
     */
    this.isDataUrl = this.url.indexOf('data:') === 0;

    /**
     * The data that was loaded by the resource.
     *
     * @member {any}
     */
    this.data = null;

    /**
     * Is this request cross-origin? If unset, determined automatically.
     *
     * @member {string}
     */
    this.crossOrigin = options.crossOrigin === true ? 'anonymous' : options.crossOrigin;

    /**
     * The method of loading to use for this resource.
     *
     * @member {Resource.LOAD_TYPE}
     */
    this.loadType = options.loadType || this._determineLoadType();

    /**
     * The type used to load the resource via XHR. If unset, determined automatically.
     *
     * @member {string}
     */
    this.xhrType = options.xhrType;

    /**
     * Extra info for middleware
     *
     * @member {object}
     */
    this.metadata = options.metadata || {};

    /**
     * The error that occurred while loading (if any).
     *
     * @member {Error}
     * @readonly
     */
    this.error = null;

    /**
     * The XHR object that was used to load this resource. This is only set
     * when `loadType` is `Resource.LOAD_TYPE.XHR`.
     *
     * @member {XMLHttpRequest}
     */
    this.xhr = null;

    /**
     * Describes if this resource was loaded as json. Only valid after the resource
     * has completely loaded.
     *
     * @member {boolean}
     */
    this.isJson = false;

    /**
     * Describes if this resource was loaded as xml. Only valid after the resource
     * has completely loaded.
     *
     * @member {boolean}
     */
    this.isXml = false;

    /**
     * Describes if this resource was loaded as an image tag. Only valid after the resource
     * has completely loaded.
     *
     * @member {boolean}
     */
    this.isImage = false;

    /**
     * Describes if this resource was loaded as an audio tag. Only valid after the resource
     * has completely loaded.
     *
     * @member {boolean}
     */
    this.isAudio = false;

    /**
     * Describes if this resource was loaded as a video tag. Only valid after the resource
     * has completely loaded.
     *
     * @member {boolean}
     */
    this.isVideo = false;

    /**
     * The `dequeue` method that will be used a storage place for the async queue dequeue method
     * used privately by the loader.
     *
     * @member {function}
     * @private
     */
    this._dequeue = null;

    /**
     * The `complete` function bound to this resource's context.
     *
     * @member {function}
     * @private
     */
    this._boundComplete = this.complete.bind(this);

    /**
     * The `_onError` function bound to this resource's context.
     *
     * @member {function}
     * @private
     */
    this._boundOnError = this._onError.bind(this);

    /**
     * The `_onProgress` function bound to this resource's context.
     *
     * @member {function}
     * @private
     */
    this._boundOnProgress = this._onProgress.bind(this);

    // xhr callbacks
    this._boundXhrOnError = this._xhrOnError.bind(this);
    this._boundXhrOnAbort = this._xhrOnAbort.bind(this);
    this._boundXhrOnLoad = this._xhrOnLoad.bind(this);
    this._boundXdrOnTimeout = this._xdrOnTimeout.bind(this);

    /**
     * Emitted when the resource beings to load.
     *
     * @event start
     * @memberof Resource#
     */

    /**
     * Emitted each time progress of this resource load updates.
     * Not all resources types and loader systems can support this event
     * so sometimes it may not be available. If the resource
     * is being loaded on a modern browser, using XHR, and the remote server
     * properly sets Content-Length headers, then this will be available.
     *
     * @event progress
     * @memberof Resource#
     */

    /**
     * Emitted once this resource has loaded, if there was an error it will
     * be in the `error` property.
     *
     * @event complete
     * @memberof Resource#
     */
}

Resource.prototype = Object.create(EventEmitter.prototype);
Resource.prototype.constructor = Resource;
module.exports = Resource;

/**
 * Marks the resource as complete.
 *
 * @fires complete
 */
Resource.prototype.complete = function () {
    // TODO: Clean this up in a wrapper or something...gross....
    if (this.data && this.data.removeEventListener) {
        this.data.removeEventListener('error', this._boundOnError);
        this.data.removeEventListener('load', this._boundComplete);
        this.data.removeEventListener('progress', this._boundOnProgress);
        this.data.removeEventListener('canplaythrough', this._boundComplete);
    }

    if (this.xhr) {
        if (this.xhr.removeEventListener) {
            this.xhr.removeEventListener('error', this._boundXhrOnError);
            this.xhr.removeEventListener('abort', this._boundXhrOnAbort);
            this.xhr.removeEventListener('progress', this._boundOnProgress);
            this.xhr.removeEventListener('load', this._boundXhrOnLoad);
        }
        else {
            this.xhr.onerror = null;
            this.xhr.ontimeout = null;
            this.xhr.onprogress = null;
            this.xhr.onload = null;
        }
    }

    this.emit('complete', this);
};

/**
 * Kicks off loading of this resource.
 *
 * @fires start
 * @param [callback] {function} Optional callback to call once the resource is loaded.
 */
Resource.prototype.load = function (cb) {
    this.emit('start', this);

    // if a callback is set, listen for complete event
    if (cb) {
        this.once('complete', cb);
    }

    // if unset, determine the value
    if (this.crossOrigin === false || typeof this.crossOrigin !== 'string') {
        this.crossOrigin = this._determineCrossOrigin(this.url);
    }

    switch(this.loadType) {
        case Resource.LOAD_TYPE.IMAGE:
            this._loadImage();
            break;

        case Resource.LOAD_TYPE.AUDIO:
            this._loadElement('audio');
            break;

        case Resource.LOAD_TYPE.VIDEO:
            this._loadElement('video');
            break;

        case Resource.LOAD_TYPE.XHR:
            /* falls through */
        default:
            if (useXdr && this.crossOrigin) {
                this._loadXdr();
            }
            else {
                this._loadXhr();
            }
            break;
    }
};

/**
 * Loads this resources using an Image object.
 *
 * @private
 */
Resource.prototype._loadImage = function () {
    this.data = new Image();

    if (this.crossOrigin) {
        this.data.crossOrigin = this.crossOrigin;
    }

    this.data.src = this.url;

    this.isImage = true;

    this.data.addEventListener('error', this._boundOnError, false);
    this.data.addEventListener('load', this._boundComplete, false);
    this.data.addEventListener('progress', this._boundOnProgress, false);
};

/**
 * Loads this resources using an HTMLAudioElement or HTMLVideoElement.
 *
 * @private
 */
Resource.prototype._loadElement = function (type) {
    if (type === 'audio' && typeof Audio !== 'undefined') {
        this.data = new Audio();
    }
    else {
        this.data = document.createElement(type);
    }

    if (this.data === null) {
        this.error = new Error('Unsupported element ' + type);
        this.complete();
        return;
    }

    // support for CocoonJS Canvas+ runtime, lacks document.createElement('source')
    if (navigator.isCocoonJS) {
        this.data.src = Array.isArray(this.url) ? this.url[0] : this.url;
    }
    else {
        if (Array.isArray(this.url)) {
            for (var i = 0; i < this.url.length; ++i) {
                this.data.appendChild(this._createSource(type, this.url[i]));
            }
        }
        else {
            this.data.appendChild(this._createSource(type, this.url));
        }
    }

    this['is' + type[0].toUpperCase() + type.substring(1)] = true;

    this.data.addEventListener('error', this._boundOnError, false);
    this.data.addEventListener('load', this._boundComplete, false);
    this.data.addEventListener('progress', this._boundOnProgress, false);
    this.data.addEventListener('canplaythrough', this._boundComplete, false);

    this.data.load();
};

/**
 * Loads this resources using an XMLHttpRequest.
 *
 * @private
 */
Resource.prototype._loadXhr = function () {
    // if unset, determine the value
    if (typeof this.xhrType !== 'string') {
        this.xhrType = this._determineXhrType();
    }

    var xhr = this.xhr = new XMLHttpRequest();

    // set the request type and url
    xhr.open('GET', this.url, true);

    // load json as text and parse it ourselves. We do this because some browsers
    // *cough* safari *cough* can't deal with it.
    if (this.xhrType === Resource.XHR_RESPONSE_TYPE.JSON || this.xhrType === Resource.XHR_RESPONSE_TYPE.DOCUMENT) {
        xhr.responseType = Resource.XHR_RESPONSE_TYPE.TEXT;
    }
    else {
        xhr.responseType = this.xhrType;
    }

    xhr.addEventListener('error', this._boundXhrOnError, false);
    xhr.addEventListener('abort', this._boundXhrOnAbort, false);
    xhr.addEventListener('progress', this._boundOnProgress, false);
    xhr.addEventListener('load', this._boundXhrOnLoad, false);

    xhr.send();
};

/**
 * Loads this resources using an XDomainRequest. This is here because we need to support IE9 (gross).
 *
 * @private
 */
Resource.prototype._loadXdr = function () {
    // if unset, determine the value
    if (typeof this.xhrType !== 'string') {
        this.xhrType = this._determineXhrType();
    }

    var xdr = this.xhr = new XDomainRequest();

    // XDomainRequest has a few quirks. Occasionally it will abort requests
    // A way to avoid this is to make sure ALL callbacks are set even if not used
    // More info here: http://stackoverflow.com/questions/15786966/xdomainrequest-aborts-post-on-ie-9
    xdr.timeout = 5000;

    xdr.onerror = this._boundXhrOnError;
    xdr.ontimeout = this._boundXdrOnTimeout;
    xdr.onprogress = this._boundOnProgress;
    xdr.onload = this._boundXhrOnLoad;

    xdr.open('GET', this.url, true);

    //  Note: The xdr.send() call is wrapped in a timeout to prevent an issue with the interface where some requests are lost
    //  if multiple XDomainRequests are being sent at the same time.
    // Some info here: https://github.com/photonstorm/phaser/issues/1248
    setTimeout(function () {
        xdr.send();
    }, 0);
};

/**
 * Creates a source used in loading via an element.
 *
 * @param type {string} The element type (video or audio).
 * @param url {string} The source URL to load from.
 * @param [mime] {string} The mime type of the video
 * @private
 */
Resource.prototype._createSource = function (type, url, mime) {
    if (!mime) {
        mime = type + '/' + url.substr(url.lastIndexOf('.') + 1);
    }

    var source = document.createElement('source');

    source.src = url;
    source.type = mime;

    return source;
};

/**
 * Called if a load errors out.
 *
 * @param event {Event} The error event from the element that emits it.
 * @private
 */
Resource.prototype._onError = function (event) {
    this.error = new Error('Failed to load element using ' + event.target.nodeName);
    this.complete();
};

/**
 * Called if a load progress event fires for xhr/xdr.
 *
 * @fires progress
 * @param event {XMLHttpRequestProgressEvent|Event}
 * @private
 */
Resource.prototype._onProgress =  function (event) {
    if (event && event.lengthComputable) {
        this.emit('progress', this, event.loaded / event.total);
    }
};

/**
 * Called if an error event fires for xhr/xdr.
 *
 * @param event {XMLHttpRequestErrorEvent|Event}
 * @private
 */
Resource.prototype._xhrOnError = function () {
    this.error = new Error(
        reqType(this.xhr) + ' Request failed. ' +
        'Status: ' + this.xhr.status + ', text: "' + this.xhr.statusText + '"'
    );

    this.complete();
};

/**
 * Called if an abort event fires for xhr.
 *
 * @param event {XMLHttpRequestAbortEvent}
 * @private
 */
Resource.prototype._xhrOnAbort = function () {
    this.error = new Error(reqType(this.xhr) + ' Request was aborted by the user.');
    this.complete();
};

/**
 * Called if a timeout event fires for xdr.
 *
 * @param event {Event}
 * @private
 */
Resource.prototype._xdrOnTimeout = function () {
    this.error = new Error(reqType(this.xhr) + ' Request timed out.');
    this.complete();
};

/**
 * Called when data successfully loads from an xhr/xdr request.
 *
 * @param event {XMLHttpRequestLoadEvent|Event}
 * @private
 */
Resource.prototype._xhrOnLoad = function () {
    var xhr = this.xhr,
        status = xhr.status !== undefined ? xhr.status : 200; //XDR has no `.status`, assume 200.

    // status can be 0 when using the file:// protocol, also check if a response was found
    if (status === 200 || status === 204 || (status === 0 && xhr.responseText.length > 0)) {
        // if text, just return it
        if (this.xhrType === Resource.XHR_RESPONSE_TYPE.TEXT) {
            this.data = xhr.responseText;
        }
        // if json, parse into json object
        else if (this.xhrType === Resource.XHR_RESPONSE_TYPE.JSON) {
            try {
                this.data = JSON.parse(xhr.responseText);
                this.isJson = true;
            } catch(e) {
                this.error = new Error('Error trying to parse loaded json:', e);
            }
        }
        // if xml, parse into an xml document or div element
        else if (this.xhrType === Resource.XHR_RESPONSE_TYPE.DOCUMENT) {
            try {
                if (window.DOMParser) {
                    var domparser = new DOMParser();
                    this.data = domparser.parseFromString(xhr.responseText, 'text/xml');
                }
                else {
                    var div = document.createElement('div');
                    div.innerHTML = xhr.responseText;
                    this.data = div;
                }
                this.isXml = true;
            } catch (e) {
                this.error = new Error('Error trying to parse loaded xml:', e);
            }
        }
        // other types just return the response
        else {
            this.data = xhr.response || xhr.responseText;
        }
    }
    else {
        this.error = new Error('[' + xhr.status + ']' + xhr.statusText + ':' + xhr.responseURL);
    }

    this.complete();
};

function reqType(xhr) {
    return xhr.toString().replace('object ', '');
}

/**
 * Sets the `crossOrigin` property for this resource based on if the url
 * for this resource is cross-origin. If crossOrigin was manually set, this
 * function does nothing.
 *
 * @private
 * @param url {string} The url to test.
 * @param [location=window.location] {object} The location object to test against.
 * @return {string} The crossOrigin value to use (or empty string for none).
 */
Resource.prototype._determineCrossOrigin = function (url, loc) {
    // data: and javascript: urls are considered same-origin
    if (url.indexOf('data:') === 0) {
        return '';
    }

    // default is window.location
    loc = loc || window.location;

    if (!tempAnchor) {
        tempAnchor = document.createElement('a');
    }

    // let the browser determine the full href for the url of this resource and then
    // parse with the node url lib, we can't use the properties of the anchor element
    // because they don't work in IE9 :(
    tempAnchor.href = url;
    url = _url.parse(tempAnchor.href);

    var samePort = (!url.port && loc.port === '') || (url.port === loc.port);

    // if cross origin
    if (url.hostname !== loc.hostname || !samePort || url.protocol !== loc.protocol) {
        return 'anonymous';
    }

    return '';
};

/**
 * Determines the responseType of an XHR request based on the extension of the
 * resource being loaded.
 *
 * @private
 * @return {Resource.XHR_RESPONSE_TYPE} The responseType to use.
 */
Resource.prototype._determineXhrType = function () {
    return Resource._xhrTypeMap[this._getExtension()] || Resource.XHR_RESPONSE_TYPE.TEXT;
};

Resource.prototype._determineLoadType = function () {
    return Resource._loadTypeMap[this._getExtension()] || Resource.LOAD_TYPE.XHR;
};

Resource.prototype._getExtension = function () {
    var url = this.url,
        ext;

    if (this.isDataUrl) {
        var slashIndex = url.indexOf('/');
        ext = url.substring(slashIndex + 1, url.indexOf(';', slashIndex));
    }
    else {
        var queryStart = url.indexOf('?');
        if (queryStart !== -1) {
            url = url.substring(0, queryStart);
        }

        ext = url.substring(url.lastIndexOf('.') + 1);
    }

    return ext.toLowerCase();
};

/**
 * Determines the mime type of an XHR request based on the responseType of
 * resource being loaded.
 *
 * @private
 * @return {string} The mime type to use.
 */
Resource.prototype._getMimeFromXhrType = function (type) {
    switch(type) {
        case Resource.XHR_RESPONSE_TYPE.BUFFER:
            return 'application/octet-binary';

        case Resource.XHR_RESPONSE_TYPE.BLOB:
            return 'application/blob';

        case Resource.XHR_RESPONSE_TYPE.DOCUMENT:
            return 'application/xml';

        case Resource.XHR_RESPONSE_TYPE.JSON:
            return 'application/json';

        case Resource.XHR_RESPONSE_TYPE.DEFAULT:
        case Resource.XHR_RESPONSE_TYPE.TEXT:
            /* falls through */
        default:
            return 'text/plain';

    }
};

/**
 * The types of loading a resource can use.
 *
 * @static
 * @constant
 * @property {object} LOAD_TYPE
 * @property {number} LOAD_TYPE.XHR - Uses XMLHttpRequest to load the resource.
 * @property {number} LOAD_TYPE.IMAGE - Uses an `Image` object to load the resource.
 * @property {number} LOAD_TYPE.AUDIO - Uses an `Audio` object to load the resource.
 * @property {number} LOAD_TYPE.VIDEO - Uses a `Video` object to load the resource.
 */
Resource.LOAD_TYPE = {
    XHR:    1,
    IMAGE:  2,
    AUDIO:  3,
    VIDEO:  4
};

/**
 * The XHR ready states, used internally.
 *
 * @static
 * @constant
 * @property {object} XHR_READY_STATE
 * @property {number} XHR_READY_STATE.UNSENT - open()has not been called yet.
 * @property {number} XHR_READY_STATE.OPENED - send()has not been called yet.
 * @property {number} XHR_READY_STATE.HEADERS_RECEIVED - send() has been called, and headers and status are available.
 * @property {number} XHR_READY_STATE.LOADING - Downloading; responseText holds partial data.
 * @property {number} XHR_READY_STATE.DONE - The operation is complete.
 */
Resource.XHR_READY_STATE = {
    UNSENT: 0,
    OPENED: 1,
    HEADERS_RECEIVED: 2,
    LOADING: 3,
    DONE: 4
};

/**
 * The XHR ready states, used internally.
 *
 * @static
 * @constant
 * @property {object} XHR_RESPONSE_TYPE
 * @property {string} XHR_RESPONSE_TYPE.DEFAULT - defaults to text
 * @property {string} XHR_RESPONSE_TYPE.BUFFER - ArrayBuffer
 * @property {string} XHR_RESPONSE_TYPE.BLOB - Blob
 * @property {string} XHR_RESPONSE_TYPE.DOCUMENT - Document
 * @property {string} XHR_RESPONSE_TYPE.JSON - Object
 * @property {string} XHR_RESPONSE_TYPE.TEXT - String
 */
Resource.XHR_RESPONSE_TYPE = {
    DEFAULT:    'text',
    BUFFER:     'arraybuffer',
    BLOB:       'blob',
    DOCUMENT:   'document',
    JSON:       'json',
    TEXT:       'text'
};

Resource._loadTypeMap = {
    'gif':      Resource.LOAD_TYPE.IMAGE,
    'png':      Resource.LOAD_TYPE.IMAGE,
    'bmp':      Resource.LOAD_TYPE.IMAGE,
    'jpg':      Resource.LOAD_TYPE.IMAGE,
    'jpeg':     Resource.LOAD_TYPE.IMAGE,
    'tif':      Resource.LOAD_TYPE.IMAGE,
    'tiff':     Resource.LOAD_TYPE.IMAGE,
    'webp':     Resource.LOAD_TYPE.IMAGE,
    'tga':      Resource.LOAD_TYPE.IMAGE,
    'svg+xml':  Resource.LOAD_TYPE.IMAGE
};

Resource._xhrTypeMap = {
    // xml
    'xhtml':    Resource.XHR_RESPONSE_TYPE.DOCUMENT,
    'html':     Resource.XHR_RESPONSE_TYPE.DOCUMENT,
    'htm':      Resource.XHR_RESPONSE_TYPE.DOCUMENT,
    'xml':      Resource.XHR_RESPONSE_TYPE.DOCUMENT,
    'tmx':      Resource.XHR_RESPONSE_TYPE.DOCUMENT,
    'tsx':      Resource.XHR_RESPONSE_TYPE.DOCUMENT,
    'svg':      Resource.XHR_RESPONSE_TYPE.DOCUMENT,

    // images
    'gif':      Resource.XHR_RESPONSE_TYPE.BLOB,
    'png':      Resource.XHR_RESPONSE_TYPE.BLOB,
    'bmp':      Resource.XHR_RESPONSE_TYPE.BLOB,
    'jpg':      Resource.XHR_RESPONSE_TYPE.BLOB,
    'jpeg':     Resource.XHR_RESPONSE_TYPE.BLOB,
    'tif':      Resource.XHR_RESPONSE_TYPE.BLOB,
    'tiff':     Resource.XHR_RESPONSE_TYPE.BLOB,
    'webp':     Resource.XHR_RESPONSE_TYPE.BLOB,
    'tga':      Resource.XHR_RESPONSE_TYPE.BLOB,

    // json
    'json':     Resource.XHR_RESPONSE_TYPE.JSON,

    // text
    'text':     Resource.XHR_RESPONSE_TYPE.TEXT,
    'txt':      Resource.XHR_RESPONSE_TYPE.TEXT
};

/**
 * Sets the load type to be used for a specific extension.
 *
 * @static
 * @param extname {string} The extension to set the type for, e.g. "png" or "fnt"
 * @param loadType {Resource.LOAD_TYPE} The load type to set it to.
 */
Resource.setExtensionLoadType = function (extname, loadType) {
    setExtMap(Resource._loadTypeMap, extname, loadType);
};

/**
 * Sets the load type to be used for a specific extension.
 *
 * @static
 * @param extname {string} The extension to set the type for, e.g. "png" or "fnt"
 * @param xhrType {Resource.XHR_RESPONSE_TYPE} The xhr type to set it to.
 */
Resource.setExtensionXhrType = function (extname, xhrType) {
    setExtMap(Resource._xhrTypeMap, extname, xhrType);
};

function setExtMap(map, extname, val) {
    if (extname && extname.indexOf('.') === 0) {
        extname = extname.substring(1);
    }

    if (!extname) {
        return;
    }

    map[extname] = val;
}

},{"eventemitter3":1,"url":14}],10:[function(require,module,exports){
module.exports = {

    // private property
    _keyStr: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

    encodeBinary: function (input) {
        var output = "";
        var bytebuffer;
        var encodedCharIndexes = new Array(4);
        var inx = 0;
        var jnx = 0;
        var paddingBytes = 0;

        while (inx < input.length) {
            // Fill byte buffer array
            bytebuffer = new Array(3);
            for (jnx = 0; jnx < bytebuffer.length; jnx++) {
                if (inx < input.length) {
                    // throw away high-order byte, as documented at:
                    // https://developer.mozilla.org/En/Using_XMLHttpRequest#Handling_binary_data
                    bytebuffer[jnx] = input.charCodeAt(inx++) & 0xff;
                }
                else {
                    bytebuffer[jnx] = 0;
                }
            }

            // Get each encoded character, 6 bits at a time
            // index 1: first 6 bits
            encodedCharIndexes[0] = bytebuffer[0] >> 2;
            // index 2: second 6 bits (2 least significant bits from input byte 1 + 4 most significant bits from byte 2)
            encodedCharIndexes[1] = ((bytebuffer[0] & 0x3) << 4) | (bytebuffer[1] >> 4);
            // index 3: third 6 bits (4 least significant bits from input byte 2 + 2 most significant bits from byte 3)
            encodedCharIndexes[2] = ((bytebuffer[1] & 0x0f) << 2) | (bytebuffer[2] >> 6);
            // index 3: forth 6 bits (6 least significant bits from input byte 3)
            encodedCharIndexes[3] = bytebuffer[2] & 0x3f;

            // Determine whether padding happened, and adjust accordingly
            paddingBytes = inx - (input.length - 1);
            switch (paddingBytes) {
                case 2:
                    // Set last 2 characters to padding char
                    encodedCharIndexes[3] = 64;
                    encodedCharIndexes[2] = 64;
                    break;

                case 1:
                    // Set last character to padding char
                    encodedCharIndexes[3] = 64;
                    break;

                default:
                    break; // No padding - proceed
            }

            // Now we will grab each appropriate character out of our keystring
            // based on our index array and append it to the output string
            for (jnx = 0; jnx < encodedCharIndexes.length; jnx++) {
                output += this._keyStr.charAt(encodedCharIndexes[jnx]);
            }
        }
        return output;
    }
};

},{}],11:[function(require,module,exports){
module.exports = require('./Loader');

module.exports.Resource = require('./Resource');

module.exports.middleware = {
    caching: {
        memory: require('./middlewares/caching/memory')
    },
    parsing: {
        blob: require('./middlewares/parsing/blob')
    }
};

},{"./Loader":8,"./Resource":9,"./middlewares/caching/memory":12,"./middlewares/parsing/blob":13}],12:[function(require,module,exports){
// a simple in-memory cache for resources
var cache = {};

module.exports = function () {
    return function (resource, next) {
        // if cached, then set data and complete the resource
        if (cache[resource.url]) {
            resource.data = cache[resource.url];
            resource.complete();
        }
        // if not cached, wait for complete and store it in the cache.
        else {
            resource.once('complete', function () {
               cache[this.url] = this.data;
            });
        }
        
        next();
    };
};

},{}],13:[function(require,module,exports){
var Resource = require('../../Resource'),
    b64 = require('../../b64');

var Url = window.URL || window.webkitURL;

// a middleware for transforming XHR loaded Blobs into more useful objects

module.exports = function () {
    return function (resource, next) {
        if (!resource.data) {
            return next();
        }

        // if this was an XHR load of a blob
        if (resource.xhr && resource.xhrType === Resource.XHR_RESPONSE_TYPE.BLOB) {
            // if there is no blob support we probably got a binary string back
            if (!window.Blob || typeof resource.data === 'string') {
                var type = resource.xhr.getResponseHeader('content-type');

                // this is an image, convert the binary string into a data url
                if (type && type.indexOf('image') === 0) {
                    resource.data = new Image();
                    resource.data.src = 'data:' + type + ';base64,' + b64.encodeBinary(resource.xhr.responseText);

                    resource.isImage = true;

                    // wait until the image loads and then callback
                    resource.data.onload = function () {
                        resource.data.onload = null;

                        next();
                    };
                }
            }
            // if content type says this is an image, then we should transform the blob into an Image object
            else if (resource.data.type.indexOf('image') === 0) {
                var src = Url.createObjectURL(resource.data);

                resource.blob = resource.data;
                resource.data = new Image();
                resource.data.src = src;

                resource.isImage = true;

                // cleanup the no longer used blob after the image loads
                resource.data.onload = function () {
                    Url.revokeObjectURL(src);
                    resource.data.onload = null;

                    next();
                };
            }
        }
        else {
            next();
        }
    };
};

},{"../../Resource":9,"../../b64":10}],14:[function(require,module,exports){
// Copyright Joyent, Inc. and other Node contributors.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

'use strict';

var punycode = require('punycode');
var util = require('./util');

exports.parse = urlParse;
exports.resolve = urlResolve;
exports.resolveObject = urlResolveObject;
exports.format = urlFormat;

exports.Url = Url;

function Url() {
  this.protocol = null;
  this.slashes = null;
  this.auth = null;
  this.host = null;
  this.port = null;
  this.hostname = null;
  this.hash = null;
  this.search = null;
  this.query = null;
  this.pathname = null;
  this.path = null;
  this.href = null;
}

// Reference: RFC 3986, RFC 1808, RFC 2396

// define these here so at least they only have to be
// compiled once on the first module load.
var protocolPattern = /^([a-z0-9.+-]+:)/i,
    portPattern = /:[0-9]*$/,

    // Special case for a simple path URL
    simplePathPattern = /^(\/\/?(?!\/)[^\?\s]*)(\?[^\s]*)?$/,

    // RFC 2396: characters reserved for delimiting URLs.
    // We actually just auto-escape these.
    delims = ['<', '>', '"', '`', ' ', '\r', '\n', '\t'],

    // RFC 2396: characters not allowed for various reasons.
    unwise = ['{', '}', '|', '\\', '^', '`'].concat(delims),

    // Allowed by RFCs, but cause of XSS attacks.  Always escape these.
    autoEscape = ['\''].concat(unwise),
    // Characters that are never ever allowed in a hostname.
    // Note that any invalid chars are also handled, but these
    // are the ones that are *expected* to be seen, so we fast-path
    // them.
    nonHostChars = ['%', '/', '?', ';', '#'].concat(autoEscape),
    hostEndingChars = ['/', '?', '#'],
    hostnameMaxLen = 255,
    hostnamePartPattern = /^[+a-z0-9A-Z_-]{0,63}$/,
    hostnamePartStart = /^([+a-z0-9A-Z_-]{0,63})(.*)$/,
    // protocols that can allow "unsafe" and "unwise" chars.
    unsafeProtocol = {
      'javascript': true,
      'javascript:': true
    },
    // protocols that never have a hostname.
    hostlessProtocol = {
      'javascript': true,
      'javascript:': true
    },
    // protocols that always contain a // bit.
    slashedProtocol = {
      'http': true,
      'https': true,
      'ftp': true,
      'gopher': true,
      'file': true,
      'http:': true,
      'https:': true,
      'ftp:': true,
      'gopher:': true,
      'file:': true
    },
    querystring = require('querystring');

function urlParse(url, parseQueryString, slashesDenoteHost) {
  if (url && util.isObject(url) && url instanceof Url) return url;

  var u = new Url;
  u.parse(url, parseQueryString, slashesDenoteHost);
  return u;
}

Url.prototype.parse = function(url, parseQueryString, slashesDenoteHost) {
  if (!util.isString(url)) {
    throw new TypeError("Parameter 'url' must be a string, not " + typeof url);
  }

  // Copy chrome, IE, opera backslash-handling behavior.
  // Back slashes before the query string get converted to forward slashes
  // See: https://code.google.com/p/chromium/issues/detail?id=25916
  var queryIndex = url.indexOf('?'),
      splitter =
          (queryIndex !== -1 && queryIndex < url.indexOf('#')) ? '?' : '#',
      uSplit = url.split(splitter),
      slashRegex = /\\/g;
  uSplit[0] = uSplit[0].replace(slashRegex, '/');
  url = uSplit.join(splitter);

  var rest = url;

  // trim before proceeding.
  // This is to support parse stuff like "  http://foo.com  \n"
  rest = rest.trim();

  if (!slashesDenoteHost && url.split('#').length === 1) {
    // Try fast path regexp
    var simplePath = simplePathPattern.exec(rest);
    if (simplePath) {
      this.path = rest;
      this.href = rest;
      this.pathname = simplePath[1];
      if (simplePath[2]) {
        this.search = simplePath[2];
        if (parseQueryString) {
          this.query = querystring.parse(this.search.substr(1));
        } else {
          this.query = this.search.substr(1);
        }
      } else if (parseQueryString) {
        this.search = '';
        this.query = {};
      }
      return this;
    }
  }

  var proto = protocolPattern.exec(rest);
  if (proto) {
    proto = proto[0];
    var lowerProto = proto.toLowerCase();
    this.protocol = lowerProto;
    rest = rest.substr(proto.length);
  }

  // figure out if it's got a host
  // user@server is *always* interpreted as a hostname, and url
  // resolution will treat //foo/bar as host=foo,path=bar because that's
  // how the browser resolves relative URLs.
  if (slashesDenoteHost || proto || rest.match(/^\/\/[^@\/]+@[^@\/]+/)) {
    var slashes = rest.substr(0, 2) === '//';
    if (slashes && !(proto && hostlessProtocol[proto])) {
      rest = rest.substr(2);
      this.slashes = true;
    }
  }

  if (!hostlessProtocol[proto] &&
      (slashes || (proto && !slashedProtocol[proto]))) {

    // there's a hostname.
    // the first instance of /, ?, ;, or # ends the host.
    //
    // If there is an @ in the hostname, then non-host chars *are* allowed
    // to the left of the last @ sign, unless some host-ending character
    // comes *before* the @-sign.
    // URLs are obnoxious.
    //
    // ex:
    // http://a@b@c/ => user:a@b host:c
    // http://a@b?@c => user:a host:c path:/?@c

    // v0.12 TODO(isaacs): This is not quite how Chrome does things.
    // Review our test case against browsers more comprehensively.

    // find the first instance of any hostEndingChars
    var hostEnd = -1;
    for (var i = 0; i < hostEndingChars.length; i++) {
      var hec = rest.indexOf(hostEndingChars[i]);
      if (hec !== -1 && (hostEnd === -1 || hec < hostEnd))
        hostEnd = hec;
    }

    // at this point, either we have an explicit point where the
    // auth portion cannot go past, or the last @ char is the decider.
    var auth, atSign;
    if (hostEnd === -1) {
      // atSign can be anywhere.
      atSign = rest.lastIndexOf('@');
    } else {
      // atSign must be in auth portion.
      // http://a@b/c@d => host:b auth:a path:/c@d
      atSign = rest.lastIndexOf('@', hostEnd);
    }

    // Now we have a portion which is definitely the auth.
    // Pull that off.
    if (atSign !== -1) {
      auth = rest.slice(0, atSign);
      rest = rest.slice(atSign + 1);
      this.auth = decodeURIComponent(auth);
    }

    // the host is the remaining to the left of the first non-host char
    hostEnd = -1;
    for (var i = 0; i < nonHostChars.length; i++) {
      var hec = rest.indexOf(nonHostChars[i]);
      if (hec !== -1 && (hostEnd === -1 || hec < hostEnd))
        hostEnd = hec;
    }
    // if we still have not hit it, then the entire thing is a host.
    if (hostEnd === -1)
      hostEnd = rest.length;

    this.host = rest.slice(0, hostEnd);
    rest = rest.slice(hostEnd);

    // pull out port.
    this.parseHost();

    // we've indicated that there is a hostname,
    // so even if it's empty, it has to be present.
    this.hostname = this.hostname || '';

    // if hostname begins with [ and ends with ]
    // assume that it's an IPv6 address.
    var ipv6Hostname = this.hostname[0] === '[' &&
        this.hostname[this.hostname.length - 1] === ']';

    // validate a little.
    if (!ipv6Hostname) {
      var hostparts = this.hostname.split(/\./);
      for (var i = 0, l = hostparts.length; i < l; i++) {
        var part = hostparts[i];
        if (!part) continue;
        if (!part.match(hostnamePartPattern)) {
          var newpart = '';
          for (var j = 0, k = part.length; j < k; j++) {
            if (part.charCodeAt(j) > 127) {
              // we replace non-ASCII char with a temporary placeholder
              // we need this to make sure size of hostname is not
              // broken by replacing non-ASCII by nothing
              newpart += 'x';
            } else {
              newpart += part[j];
            }
          }
          // we test again with ASCII char only
          if (!newpart.match(hostnamePartPattern)) {
            var validParts = hostparts.slice(0, i);
            var notHost = hostparts.slice(i + 1);
            var bit = part.match(hostnamePartStart);
            if (bit) {
              validParts.push(bit[1]);
              notHost.unshift(bit[2]);
            }
            if (notHost.length) {
              rest = '/' + notHost.join('.') + rest;
            }
            this.hostname = validParts.join('.');
            break;
          }
        }
      }
    }

    if (this.hostname.length > hostnameMaxLen) {
      this.hostname = '';
    } else {
      // hostnames are always lower case.
      this.hostname = this.hostname.toLowerCase();
    }

    if (!ipv6Hostname) {
      // IDNA Support: Returns a punycoded representation of "domain".
      // It only converts parts of the domain name that
      // have non-ASCII characters, i.e. it doesn't matter if
      // you call it with a domain that already is ASCII-only.
      this.hostname = punycode.toASCII(this.hostname);
    }

    var p = this.port ? ':' + this.port : '';
    var h = this.hostname || '';
    this.host = h + p;
    this.href += this.host;

    // strip [ and ] from the hostname
    // the host field still retains them, though
    if (ipv6Hostname) {
      this.hostname = this.hostname.substr(1, this.hostname.length - 2);
      if (rest[0] !== '/') {
        rest = '/' + rest;
      }
    }
  }

  // now rest is set to the post-host stuff.
  // chop off any delim chars.
  if (!unsafeProtocol[lowerProto]) {

    // First, make 100% sure that any "autoEscape" chars get
    // escaped, even if encodeURIComponent doesn't think they
    // need to be.
    for (var i = 0, l = autoEscape.length; i < l; i++) {
      var ae = autoEscape[i];
      if (rest.indexOf(ae) === -1)
        continue;
      var esc = encodeURIComponent(ae);
      if (esc === ae) {
        esc = escape(ae);
      }
      rest = rest.split(ae).join(esc);
    }
  }


  // chop off from the tail first.
  var hash = rest.indexOf('#');
  if (hash !== -1) {
    // got a fragment string.
    this.hash = rest.substr(hash);
    rest = rest.slice(0, hash);
  }
  var qm = rest.indexOf('?');
  if (qm !== -1) {
    this.search = rest.substr(qm);
    this.query = rest.substr(qm + 1);
    if (parseQueryString) {
      this.query = querystring.parse(this.query);
    }
    rest = rest.slice(0, qm);
  } else if (parseQueryString) {
    // no query string, but parseQueryString still requested
    this.search = '';
    this.query = {};
  }
  if (rest) this.pathname = rest;
  if (slashedProtocol[lowerProto] &&
      this.hostname && !this.pathname) {
    this.pathname = '/';
  }

  //to support http.request
  if (this.pathname || this.search) {
    var p = this.pathname || '';
    var s = this.search || '';
    this.path = p + s;
  }

  // finally, reconstruct the href based on what has been validated.
  this.href = this.format();
  return this;
};

// format a parsed object into a url string
function urlFormat(obj) {
  // ensure it's an object, and not a string url.
  // If it's an obj, this is a no-op.
  // this way, you can call url_format() on strings
  // to clean up potentially wonky urls.
  if (util.isString(obj)) obj = urlParse(obj);
  if (!(obj instanceof Url)) return Url.prototype.format.call(obj);
  return obj.format();
}

Url.prototype.format = function() {
  var auth = this.auth || '';
  if (auth) {
    auth = encodeURIComponent(auth);
    auth = auth.replace(/%3A/i, ':');
    auth += '@';
  }

  var protocol = this.protocol || '',
      pathname = this.pathname || '',
      hash = this.hash || '',
      host = false,
      query = '';

  if (this.host) {
    host = auth + this.host;
  } else if (this.hostname) {
    host = auth + (this.hostname.indexOf(':') === -1 ?
        this.hostname :
        '[' + this.hostname + ']');
    if (this.port) {
      host += ':' + this.port;
    }
  }

  if (this.query &&
      util.isObject(this.query) &&
      Object.keys(this.query).length) {
    query = querystring.stringify(this.query);
  }

  var search = this.search || (query && ('?' + query)) || '';

  if (protocol && protocol.substr(-1) !== ':') protocol += ':';

  // only the slashedProtocols get the //.  Not mailto:, xmpp:, etc.
  // unless they had them to begin with.
  if (this.slashes ||
      (!protocol || slashedProtocol[protocol]) && host !== false) {
    host = '//' + (host || '');
    if (pathname && pathname.charAt(0) !== '/') pathname = '/' + pathname;
  } else if (!host) {
    host = '';
  }

  if (hash && hash.charAt(0) !== '#') hash = '#' + hash;
  if (search && search.charAt(0) !== '?') search = '?' + search;

  pathname = pathname.replace(/[?#]/g, function(match) {
    return encodeURIComponent(match);
  });
  search = search.replace('#', '%23');

  return protocol + host + pathname + search + hash;
};

function urlResolve(source, relative) {
  return urlParse(source, false, true).resolve(relative);
}

Url.prototype.resolve = function(relative) {
  return this.resolveObject(urlParse(relative, false, true)).format();
};

function urlResolveObject(source, relative) {
  if (!source) return relative;
  return urlParse(source, false, true).resolveObject(relative);
}

Url.prototype.resolveObject = function(relative) {
  if (util.isString(relative)) {
    var rel = new Url();
    rel.parse(relative, false, true);
    relative = rel;
  }

  var result = new Url();
  var tkeys = Object.keys(this);
  for (var tk = 0; tk < tkeys.length; tk++) {
    var tkey = tkeys[tk];
    result[tkey] = this[tkey];
  }

  // hash is always overridden, no matter what.
  // even href="" will remove it.
  result.hash = relative.hash;

  // if the relative url is empty, then there's nothing left to do here.
  if (relative.href === '') {
    result.href = result.format();
    return result;
  }

  // hrefs like //foo/bar always cut to the protocol.
  if (relative.slashes && !relative.protocol) {
    // take everything except the protocol from relative
    var rkeys = Object.keys(relative);
    for (var rk = 0; rk < rkeys.length; rk++) {
      var rkey = rkeys[rk];
      if (rkey !== 'protocol')
        result[rkey] = relative[rkey];
    }

    //urlParse appends trailing / to urls like http://www.example.com
    if (slashedProtocol[result.protocol] &&
        result.hostname && !result.pathname) {
      result.path = result.pathname = '/';
    }

    result.href = result.format();
    return result;
  }

  if (relative.protocol && relative.protocol !== result.protocol) {
    // if it's a known url protocol, then changing
    // the protocol does weird things
    // first, if it's not file:, then we MUST have a host,
    // and if there was a path
    // to begin with, then we MUST have a path.
    // if it is file:, then the host is dropped,
    // because that's known to be hostless.
    // anything else is assumed to be absolute.
    if (!slashedProtocol[relative.protocol]) {
      var keys = Object.keys(relative);
      for (var v = 0; v < keys.length; v++) {
        var k = keys[v];
        result[k] = relative[k];
      }
      result.href = result.format();
      return result;
    }

    result.protocol = relative.protocol;
    if (!relative.host && !hostlessProtocol[relative.protocol]) {
      var relPath = (relative.pathname || '').split('/');
      while (relPath.length && !(relative.host = relPath.shift()));
      if (!relative.host) relative.host = '';
      if (!relative.hostname) relative.hostname = '';
      if (relPath[0] !== '') relPath.unshift('');
      if (relPath.length < 2) relPath.unshift('');
      result.pathname = relPath.join('/');
    } else {
      result.pathname = relative.pathname;
    }
    result.search = relative.search;
    result.query = relative.query;
    result.host = relative.host || '';
    result.auth = relative.auth;
    result.hostname = relative.hostname || relative.host;
    result.port = relative.port;
    // to support http.request
    if (result.pathname || result.search) {
      var p = result.pathname || '';
      var s = result.search || '';
      result.path = p + s;
    }
    result.slashes = result.slashes || relative.slashes;
    result.href = result.format();
    return result;
  }

  var isSourceAbs = (result.pathname && result.pathname.charAt(0) === '/'),
      isRelAbs = (
          relative.host ||
          relative.pathname && relative.pathname.charAt(0) === '/'
      ),
      mustEndAbs = (isRelAbs || isSourceAbs ||
                    (result.host && relative.pathname)),
      removeAllDots = mustEndAbs,
      srcPath = result.pathname && result.pathname.split('/') || [],
      relPath = relative.pathname && relative.pathname.split('/') || [],
      psychotic = result.protocol && !slashedProtocol[result.protocol];

  // if the url is a non-slashed url, then relative
  // links like ../.. should be able
  // to crawl up to the hostname, as well.  This is strange.
  // result.protocol has already been set by now.
  // Later on, put the first path part into the host field.
  if (psychotic) {
    result.hostname = '';
    result.port = null;
    if (result.host) {
      if (srcPath[0] === '') srcPath[0] = result.host;
      else srcPath.unshift(result.host);
    }
    result.host = '';
    if (relative.protocol) {
      relative.hostname = null;
      relative.port = null;
      if (relative.host) {
        if (relPath[0] === '') relPath[0] = relative.host;
        else relPath.unshift(relative.host);
      }
      relative.host = null;
    }
    mustEndAbs = mustEndAbs && (relPath[0] === '' || srcPath[0] === '');
  }

  if (isRelAbs) {
    // it's absolute.
    result.host = (relative.host || relative.host === '') ?
                  relative.host : result.host;
    result.hostname = (relative.hostname || relative.hostname === '') ?
                      relative.hostname : result.hostname;
    result.search = relative.search;
    result.query = relative.query;
    srcPath = relPath;
    // fall through to the dot-handling below.
  } else if (relPath.length) {
    // it's relative
    // throw away the existing file, and take the new path instead.
    if (!srcPath) srcPath = [];
    srcPath.pop();
    srcPath = srcPath.concat(relPath);
    result.search = relative.search;
    result.query = relative.query;
  } else if (!util.isNullOrUndefined(relative.search)) {
    // just pull out the search.
    // like href='?foo'.
    // Put this after the other two cases because it simplifies the booleans
    if (psychotic) {
      result.hostname = result.host = srcPath.shift();
      //occationaly the auth can get stuck only in host
      //this especially happens in cases like
      //url.resolveObject('mailto:local1@domain1', 'local2@domain2')
      var authInHost = result.host && result.host.indexOf('@') > 0 ?
                       result.host.split('@') : false;
      if (authInHost) {
        result.auth = authInHost.shift();
        result.host = result.hostname = authInHost.shift();
      }
    }
    result.search = relative.search;
    result.query = relative.query;
    //to support http.request
    if (!util.isNull(result.pathname) || !util.isNull(result.search)) {
      result.path = (result.pathname ? result.pathname : '') +
                    (result.search ? result.search : '');
    }
    result.href = result.format();
    return result;
  }

  if (!srcPath.length) {
    // no path at all.  easy.
    // we've already handled the other stuff above.
    result.pathname = null;
    //to support http.request
    if (result.search) {
      result.path = '/' + result.search;
    } else {
      result.path = null;
    }
    result.href = result.format();
    return result;
  }

  // if a url ENDs in . or .., then it must get a trailing slash.
  // however, if it ends in anything else non-slashy,
  // then it must NOT get a trailing slash.
  var last = srcPath.slice(-1)[0];
  var hasTrailingSlash = (
      (result.host || relative.host || srcPath.length > 1) &&
      (last === '.' || last === '..') || last === '');

  // strip single dots, resolve double dots to parent dir
  // if the path tries to go above the root, `up` ends up > 0
  var up = 0;
  for (var i = srcPath.length; i >= 0; i--) {
    last = srcPath[i];
    if (last === '.') {
      srcPath.splice(i, 1);
    } else if (last === '..') {
      srcPath.splice(i, 1);
      up++;
    } else if (up) {
      srcPath.splice(i, 1);
      up--;
    }
  }

  // if the path is allowed to go above the root, restore leading ..s
  if (!mustEndAbs && !removeAllDots) {
    for (; up--; up) {
      srcPath.unshift('..');
    }
  }

  if (mustEndAbs && srcPath[0] !== '' &&
      (!srcPath[0] || srcPath[0].charAt(0) !== '/')) {
    srcPath.unshift('');
  }

  if (hasTrailingSlash && (srcPath.join('/').substr(-1) !== '/')) {
    srcPath.push('');
  }

  var isAbsolute = srcPath[0] === '' ||
      (srcPath[0] && srcPath[0].charAt(0) === '/');

  // put the host back
  if (psychotic) {
    result.hostname = result.host = isAbsolute ? '' :
                                    srcPath.length ? srcPath.shift() : '';
    //occationaly the auth can get stuck only in host
    //this especially happens in cases like
    //url.resolveObject('mailto:local1@domain1', 'local2@domain2')
    var authInHost = result.host && result.host.indexOf('@') > 0 ?
                     result.host.split('@') : false;
    if (authInHost) {
      result.auth = authInHost.shift();
      result.host = result.hostname = authInHost.shift();
    }
  }

  mustEndAbs = mustEndAbs || (result.host && srcPath.length);

  if (mustEndAbs && !isAbsolute) {
    srcPath.unshift('');
  }

  if (!srcPath.length) {
    result.pathname = null;
    result.path = null;
  } else {
    result.pathname = srcPath.join('/');
  }

  //to support request.http
  if (!util.isNull(result.pathname) || !util.isNull(result.search)) {
    result.path = (result.pathname ? result.pathname : '') +
                  (result.search ? result.search : '');
  }
  result.auth = relative.auth || result.auth;
  result.slashes = result.slashes || relative.slashes;
  result.href = result.format();
  return result;
};

Url.prototype.parseHost = function() {
  var host = this.host;
  var port = portPattern.exec(host);
  if (port) {
    port = port[0];
    if (port !== ':') {
      this.port = port.substr(1);
    }
    host = host.substr(0, host.length - port.length);
  }
  if (host) this.hostname = host;
};

},{"./util":15,"punycode":3,"querystring":6}],15:[function(require,module,exports){
'use strict';

module.exports = {
  isString: function(arg) {
    return typeof(arg) === 'string';
  },
  isObject: function(arg) {
    return typeof(arg) === 'object' && arg !== null;
  },
  isNull: function(arg) {
    return arg === null;
  },
  isNullOrUndefined: function(arg) {
    return arg == null;
  }
};

},{}],16:[function(require,module,exports){

var Trampoline = typeof window === 'undefined' ? {} : window.Trampoline;
var ResourceLoader = require( 'resource-loader' );

Trampoline.classes = {
  ResourceLoader: ResourceLoader
};

},{"resource-loader":11}]},{},[16]);

//# sourceMappingURL=maps/trampoline.js.map
